package gov.nih.nci.lexbig.ext;

import java.util.*;
import java.io.*;

import org.w3c.dom.*;
import org.apache.xerces.dom.DocumentImpl;
import org.w3c.dom.Document;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;

/**
  * <!-- LICENSE_TEXT_START -->
* Copyright 2007 NGIT. This software was developed in conjunction with the National Cancer Institute,
* and so to the extent government employees are co-authors, any rights in such works shall be subject to Title 17 of the United States Code, section 105.
* Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
* 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the disclaimer of Article 3, below. Redistributions
* in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
* 2. The end-user documentation included with the redistribution, if any, must include the following acknowledgment:
* "This product includes software developed by NGIT and the National Cancer Institute."
* If no such end-user documentation is to be included, this acknowledgment shall appear in the software itself,
* wherever such third-party acknowledgments normally appear.
* 3. The names "The National Cancer Institute", "NCI" and "NGIT" must not be used to endorse or promote products derived from this software.
* 4. This license does not authorize the incorporation of this software into any third party proprietary programs. This license does not authorize
* the recipient to use any trademarks owned by either NCI or NGIT
* 5. THIS SOFTWARE IS PROVIDED "AS IS," AND ANY EXPRESSED OR IMPLIED WARRANTIES, (INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE) ARE DISCLAIMED. IN NO EVENT SHALL THE NATIONAL CANCER INSTITUTE,
* NGIT, OR THEIR AFFILIATES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * <!-- LICENSE_TEXT_END -->
  */

/** ConceptAssociation
 *
 * @since DLBExt 1.0
 * @author Northrop Grumman Information Technology
 * @version 1.0
 */

public class ConceptAssociation implements Serializable {

	/**
   * 
   */
  private static final long serialVersionUID = 1L;

  /** Association type name
	 */
	String name;

	/** The name of associated concept
	 */
	String value;

	/** qualifiers
	 */
	Vector qualifiers;

	/** contains Qualifier
	 */
	boolean containsQualifier;


	/** default constructor
	 *
	 * @since DLBExt 1.0
	 */
	public ConceptAssociation() {
		this.name = "";
		this.value = "";
		this.containsQualifier = false;
		this.qualifiers = new Vector();
	}

	/** Constructs a ConceptAssociation with the given name and value.
	 *
	 * @since DLBExt 1.0
	 * @param name name of the ConceptAssociation.
	 * @param value value of the ConceptAssociation.
	 */
	public ConceptAssociation(String name, String value) {
		this.name = name;
		this.value = value;
		this.containsQualifier = false;
		this.qualifiers = new Vector();
	}

	/** Assigns a name to concept association
	 *
	 * @since DLBExt 1.0
	 * @param name ConceptAssociation name
	 */
	public void setName(String name) {
		this.name = name;
	}

	/** Assigns a value to concept association
	 *
	 * @since DLBExt 1.0
	 * @param value ConceptAssociation name
	 */
	public void setValue(String value) {
		this.value = value;
	}

	/** Gets name
	 *
	 * @since DLBExt 1.0
	 * @return name
	 */
	public String getName() {
		return this.name;
	}

	/** Gets value
	 *
	 * @since DLBExt 1.0
	 * @return value
	 */
	public String getValue() {
		return this.value;
	}

	/** Adds the specific qualifier to the ConceptAssociation.
	 *
	 * @since DLBExt 1.0
	 * @param qualifierName the qualifier name.
	 * @param qualifierValue the qualifier value.
	 */
	public void addQualifier(String qualifierName, String qualifierValue) {
		Qualifier qualifier = new Qualifier(qualifierName, qualifierValue);
		qualifiers.add(qualifier);
		if (qualifiers.size() > 0) containsQualifier = true;
	}


	/** Adds the specific qualifier to the ConceptAssociation.
	 *
	 * @since DLBExt 1.0
	 * @param qualifier an instance of Qualifier
	 */
	public void addQualifier(Qualifier qualifier) {
		if (qualifier == null)
		{
		    return;
		}
		qualifiers.add(qualifier);
		if (qualifiers.size() > 0) containsQualifier = true;
	}



	/** Gets qualifiers
	 *
	 * @since DLBExt 1.0
	 * @return qualifiers
	 */
	public Vector getQualifiers() {
		return this.qualifiers;
	}

	/** Gets qualifiers
	 *
	 * @since DLBExt 1.0
	 * @return qualifiers
	 */
	public Vector getQualifierCollection() {
		return this.qualifiers;
	}


	/** Checks if the ConceptAssociation instance contains any qualifier
	 *
	 * since lexrpc 2.1
	 *
	 * @since DLBExt 1.0
	 * @return qualifiers
	 */
	public boolean hasQualifier() {
		return this.containsQualifier;
	}


	/** Gets a XML representation of a ConceptAssociation object in String
	 *
	 * @since DLBExt 1.0
	 * @return a XML representation of a ConceptAssociation object in String
	 */
	public String toXML() {
		return writeDocument(toDOM());
	}

	/** Converts a XCML Document to a String
	 *
	 * @since DLBExt 1.0
	 * @param doc a XML document
	 * @return a String representation of a XML document
	 */
	private String writeDocument(Document doc)
	{
	    try {
	        OutputFormat format = new OutputFormat(doc);
			StringWriter stringOut = new StringWriter();
	        XMLSerializer serial = new XMLSerializer(stringOut, format);
	        serial.asDOMSerializer();
	        serial.serialize(doc.getDocumentElement());
	        return stringOut.toString();
	    } catch (Exception ex) {
	        ex.printStackTrace();
	    }
	    return null;
	}

	/** Gets a String representation of a ConceptAssociation object
	 *
	 * @since DLBExt 1.0
	 * @return a String representation of a ConceptAssociation object
	 */
	public String toString() {
		return this.name + "$" + this.value;
	}

	/** Gets a XML Document representation of a ConceptAssociation object
	 *
	 * @since DLBExt 1.0
	 * @return a XML Document representation of a ConceptAssociation object
	 */
	public Document toDOM()
	{
		Document doc= new DocumentImpl();
		try {
			Element root = doc.createElement("ConceptAssociation");
			Element element;
			element = doc.createElement("name");
			element.appendChild(doc.createTextNode(name));
			root.appendChild(element);

			element = doc.createElement("value");
			element.appendChild(doc.createTextNode(value));
			root.appendChild(element);

			doc.appendChild(root);
	    } catch (Exception ex) {
	        ex.printStackTrace();
	    }
		return doc;
	}


} // end of ConceptAssociation
