package gov.nih.nci.lexbig.ext;

import gov.nih.nci.system.applicationservice.EVSApplicationService;
import gov.nih.nci.system.client.ApplicationServiceProvider;

import java.io.ByteArrayOutputStream;
import java.io.ObjectOutputStream;
import java.io.PrintWriter;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;

import javax.swing.tree.DefaultMutableTreeNode;

import org.LexGrid.LexBIG.DataModel.Collections.AbsoluteCodingSchemeVersionReferenceList;
import org.LexGrid.LexBIG.DataModel.Collections.CodingSchemeRenderingList;
import org.LexGrid.LexBIG.DataModel.Collections.MetadataPropertyList;
import org.LexGrid.LexBIG.DataModel.Core.AbsoluteCodingSchemeVersionReference;
import org.LexGrid.LexBIG.DataModel.Core.AssociatedConcept;
import org.LexGrid.LexBIG.DataModel.Core.CodingSchemeSummary;
import org.LexGrid.LexBIG.DataModel.Core.CodingSchemeVersionOrTag;
import org.LexGrid.LexBIG.DataModel.Core.MetadataProperty;
import org.LexGrid.LexBIG.DataModel.Core.NameAndValue;
import org.LexGrid.LexBIG.DataModel.InterfaceElements.CodingSchemeRendering;
import org.LexGrid.LexBIG.DataModel.NCIHistory.NCIChangeEvent;
import org.LexGrid.LexBIG.DataModel.NCIHistory.types.ChangeType;
import org.LexGrid.LexBIG.LexBIGService.LexBIGServiceMetadata;
import org.LexGrid.codingSchemes.CodingScheme;
import org.LexGrid.codingSchemes.CodingSchemeVersion;
import org.LexGrid.commonTypes.PropertyQualifier;
import org.LexGrid.concepts.CodedEntry;
import org.LexGrid.concepts.Comment;
import org.LexGrid.concepts.Definition;
import org.LexGrid.concepts.Instruction;
import org.LexGrid.concepts.Presentation;
import org.LexGrid.naming.SupportedAssociation;
import org.LexGrid.naming.SupportedProperty;
import org.LexGrid.relations.Relations;

/**
  * <!-- LICENSE_TEXT_START -->
* Copyright 2007 NGIT. This software was developed in conjunction with the National Cancer Institute,
* and so to the extent government employees are co-authors, any rights in such works shall be subject to Title 17 of the United States Code, section 105.
* Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
* 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the disclaimer of Article 3, below. Redistributions
* in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
* 2. The end-user documentation included with the redistribution, if any, must include the following acknowledgment:
* "This product includes software developed by NGIT and the National Cancer Institute."
* If no such end-user documentation is to be included, this acknowledgment shall appear in the software itself,
* wherever such third-party acknowledgments normally appear.
* 3. The names "The National Cancer Institute", "NCI" and "NGIT" must not be used to endorse or promote products derived from this software.
* 4. This license does not authorize the incorporation of this software into any third party proprietary programs. This license does not authorize
* the recipient to use any trademarks owned by either NCI or NGIT
* 5. THIS SOFTWARE IS PROVIDED "AS IS," AND ANY EXPRESSED OR IMPLIED WARRANTIES, (INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE) ARE DISCLAIMED. IN NO EVENT SHALL THE NATIONAL CANCER INSTITUTE,
* NGIT, OR THEIR AFFILIATES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * <!-- LICENSE_TEXT_END -->
  */

/**
 * DLBAdapter
 *
 * @since DLBExt 1.0
 * @author Northrop Grumman Information Technology
 * @version 1.0
 */
public final class DLBAdapter extends DLBWrapper {

	/** a static constant, 0
	 */
	public static final int MATCH_BY_NAME = 0;
	/** a static constant, 1
	 */
	public static final int MATCH_BY_ROLE = 1;
	/** a static constant, 2
	 */
	public static final int MATCH_BY_PROPERTY = 2;
	/** a static constant, 3
	 */
	public static final int MATCH_BY_SILO = 3;
	/** a static constant, 4
	 */
	public static final int MATCH_BY_ASSOCIATION = 4;
	/** a static constant, 5
	 */
	public static final int MATCH_BY_SYNONYM = 5;
	/** a static constant, 6
	 */
	public static final int MATCH_BY_INVERSE_ROLE = 6;
	/** a static constant, 7
	 */
	public static final int MATCH_BY_INVERSE_ASSOCIATION = 7;
	/** a static constant, 8
	 */
	public static final int MATCH_BY_ROLE_PROPERTY = 8;

	/** a static constant, 9
	 */
	public static final int MATCH_BY_SOURCE_CODE = 9;



	/** A static constant, 0
	 */
	public static final int WITH_NO_ATTRIBUTES = 0;
	/** A static constant, 1
	 */
	public static final int WITH_ALL_ATTRIBUTES = 1;
	/** A static constant, 2
	 */
	public static final int WITH_ALL_ROLES = 2;
	/** A static constant, 3
	 */
	public static final int WITH_ALL_PROPERTIES = 3;

	/** A static constant, 4
	 */
	public static final int WITH_ALL_ASSOCIATIONS = 4;


    /** a constance 1000
     */
    final static int MAX_LEVEL = 1000;

    /** a namespace id
     */
    int namespaceId = -1;

    /** default search algorithm name.
     */
    private String defaultAlgorithm = "LuceneQuery";

    private String defaultTag = "PRODUCTION";

    /** default Namespace Id.
     */
    //private int defaultNamespaceId = 0;

    /** An instance of Category.
     */
    //Category cat;


///////////////////////
//  Constructors
///////////////////////

    /** Default constructor
     *
     * @since DLBExt 1.0
     */
    public DLBAdapter()
    {
		super();
	}


    public DLBAdapter(EVSApplicationService lbSvc)
    {
		super(lbSvc);
	}

    public String getCurrentCodingSchemeName()
    {
		return this.codingScheme;
	}
    public String getCurrentVersion()
    {
		return this.version;
	}
    public String getCurrentTag()
    {
		return this.tag;
	}



  public static HashMap getAdapterHashMap(String url)
  {
	  HashMap<String,DLBAdapter> hmap = new HashMap<String,DLBAdapter>();
	  DLBAdapter adapter0 = new DLBAdapter();
      Vector<String> name_vec = adapter0.getVocabularyNames();

      for (int i=0; i<name_vec.size(); i++)
      {
		  DLBAdapter adapter = null;
		  if (url != null)
		  {
                try {
		  		    adapter = new DLBAdapter((EVSApplicationService)ApplicationServiceProvider.getApplicationServiceFromUrl(url));
			    } catch (Exception e) {
					e.printStackTrace();
				}

		  } else {
			  adapter = new DLBAdapter();
		  }
		  String name = (String) name_vec.elementAt(i);
		  String version = null;
		  String tag = null;

		  Boolean canConnect = null;

		  Vector<String> v = adapter.getAvailableCodingSchemeTags(name, null);
		  if (v.size() == 0 || !v.contains("PRODUCTION"))
		  {
			  version = adapter.getVocabularyVersionByTag(name, tag);
			  //GF9072
			  //adapter.setVocabulary(name, version, tag);
			  canConnect = adapter.setVocabulary(name, version, tag);
		  }
		  else
		  {
		      if (v.contains("PRODUCTION"))
		      {
				  tag = "PRODUCTION";
				  version = adapter.getVocabularyVersionByTag(name, tag);
				  //GF9072
				  //adapter.setVocabulary(name, version, tag);
				  canConnect = adapter.setVocabulary(name, version, tag);
			  }
			  else
			  {
				  version = adapter.getVocabularyVersionByTag(name, tag);
				  //GF9072
				  //adapter.setVocabulary(name, version, tag);
				  canConnect = adapter.setVocabulary(name, version, tag);

			  }
		  }
		  //GF9072
		  if (canConnect != null && canConnect.equals(Boolean.TRUE))
		  {
		  	  hmap.put(name, adapter);
		  }
	  }
	  return hmap;
  }




	/** Gets the name of the default search algorithm.
	 *
	 * @since DLBExt 1.0
	 * @return the name of the default search algorithm.
	 */
	public String getDefaultAlgorithm()
	{
		return defaultAlgorithm;
	}

	/** Sets the name of the default search algorithm.
	 *
	 * @since DLBExt 1.0
	 * @param algorithm the name of the default search algorithm
	 */
	public void setDefaultAlgorithm(String algorithm)
	{
		defaultAlgorithm = algorithm;
	}

	/** Sets vocabulary name
	 *
	 * @since DLBExt 1.0
	 * @param name vocabulary name
	 * @param nsid
	 * @return boolean variable, true if successful; false, otherwise
	 */

    private String localName2FormalName(String name)
	{
        //return localName2FormalName(name, version, tag);
        return localName2FormalName(name, null, null);
	}

	private String localName2FormalName(String name, String version, String tag)
	{
		try {
			if (lbSvc == null)
			{
				return null;
			}

			CodingScheme cs = getCodingScheme(name, version, tag);
			String formalName = cs.getFormalName();
			if (formalName.compareTo(name) == 0) return name;

			Vector localnames = array2Vector(cs.getLocalName());
			for (int j=0; j<localnames.size(); j++)
			{
				 String localname = (String) localnames.elementAt(j);
				 if (localname.compareTo(name) == 0)
				 {
					 return cs.getFormalName();
				 }
			}

		} catch (Exception e) {
			getLogger().warn("Error in localName2FormalName");
		}
		return null;
	}

//	used
    /** Sets vocabulary (coding scheme) name.
     *
     * @since DLBExt 1.0
     * @param name coding scheme name
     * @return a boolean variable - true, if successful; false, if otherwise.
     */
    public Boolean setVocabulary(String name)
    {
		String defaultVersion = getVocabularyVersionByTag(name, defaultTag);
        Boolean ret_obj = setVocabulary(name, defaultVersion, defaultTag);
        return ret_obj;
	}


	/** Sets vocabulary name.
	 *
	 * @since DLBExt 1.0
	 * @param name coding scheme name
	 * @param version coding scheme version
	 * @param tag coding scheme tag
	 * @return boolean variable; set to true, if successful; false, otherwise
	 */
	public Boolean setVocabulary(String name, String version, String tag)
	{
		// GF9072
		Boolean isActive = isCodingSchemeActive(lbSvc, name, version);
		if (isActive == null || isActive.equals(Boolean.FALSE)) return Boolean.FALSE;

        if (tag != null && tag.compareTo("") != 0)
        {
		    Vector tags = getAvailableCodingSchemeTags(name, version);
			if (tags.size() == 0)
			{
				getLogger().info("No tag is found for coding scheme " + name);
				return Boolean.FALSE;
			}
			else
			{
				if (!tags.contains(tag))
				{
					getLogger().warn("Tag " + tag + " is not found.");
					return Boolean.FALSE;
				}
		    }
		}

        CodingSchemeVersionOrTag versionOrTag = createCodingSchemeVersionOrTag(version, tag);
        defaultCodingScheme = null;
        try {
         	defaultCodingScheme = lbSvc.resolveCodingScheme(name, versionOrTag);
         	codingScheme = defaultCodingScheme.getCodingScheme();

 		} catch (Exception e) {
 			e.printStackTrace();
		}
        //if (codingScheme == null || codingScheme.compareTo("") == 0)
        if (defaultCodingScheme == null) return Boolean.FALSE;

        //this.codingScheme = localName2FormalName(name, version, tag);//name;
        this.version = version;
        this.tag = tag;

		return Boolean.TRUE;
	}


    public String getFormalCodingSchemeName(String name)
    {
		return localName2FormalName(name);
	}

    /** Converts a local name to a formal name of a coding scheme.
     *
     * @since DLBExt 1.0
     * @param name a local name of a coding scheme
     * @return a formal name of a coding scheme
     */

    public String getVocabularyDescription(String name)
    {
		return localName2FormalName(name);
	}


    /** Gets all local names of a coding scheme.
     *
     * @since DLBExt 1.0
     * @param codingSchemeName coding scheme name
     * @return all local names of a coding scheme.
     */
    public Vector getLocalNames(String codingSchemeName)
    {
		CodingScheme cs = getCodingScheme(codingSchemeName, version, tag);
		if (cs == null) return null;
		return array2Vector(cs.getLocalName());
	}


	/** Gets the namespace ID corresponding to a given coding scheme or vocabulary name.
	 *
	 * @since DLBExt 1.0
	 * @param namespaceName a given coding scheme or vocabulary name
	 * @return the namespace ID corresponding to a given coding scheme or
	 *     vocabulary name
	 */
//TODO this method is meaningless.  There is no namespace ID in LexBIG.
	public int getNamespaceId(String namespaceName)
	{
        Integer int_obj = getNamespaceId(namespaceName, version, tag);
        if (int_obj == null) return -1;
        return int_obj.intValue();
	}

	/** Finds formal name of a coding scheme for a given namespace code.
	 *
	 * @since DLBExt 1.0
	 * @param namespaceCode namespace code (a local name of a coding scheme)
	 * @return formal name of a coding scheme for a given namespace code.
	 */
	public String findNamespaceByCode(String namespaceCode)
	{
		return localName2FormalName(namespaceCode);
	}

	/** Gets namespace id (local name of a vocabulary name).
	 *
	 * @since DLBExt 1.0
	 * @param namespaceName coding scheme or vocabulary name
	 * @param version coding scheme or vocabulary version
	 * @param tag coding scheme or vocabulary tag
	 * @return namespace id (local name of a vocabulary name)
	 */
	public Integer getNamespaceId(String namespaceName, String version, String tag)
	{
		CodingScheme cs = getCodingScheme(namespaceName, version, tag);
		if (cs == null) return null;
		String[] names = cs.getLocalName();

		if (names.length == 0)
		{
			return null;
		}

		String name = null;
		for (int i=0; i<names.length; i++)
		{
			name = names[i];
			boolean isInteger = true;
			for (int j=0; j<names.length; j++)
			{
				char ch = name.charAt(j);
				if (!Character.isDigit(ch))
				{
					isInteger = false;
					break;
				}
			}
			if (isInteger)
			{
				int id = Integer.parseInt(name);
				return new Integer(id);
			}
		}
		return null;
	}


    /** Checks if a concept has any subconcept
     *
     * @since DLBExt 1.0
     * @param conceptname the specified concept name
     * @return a Boolean. true - conceptname has any subconcept; false - it
     *     has no subconcept; null, otherwise.
     */


    public boolean hasSubconcepts(String conceptname)
    {
		Boolean Bool_obj = hasChildren(conceptname);
		if (Bool_obj == null) return false;
		return Bool_obj.booleanValue();
	}
//  used
	/** Checks if a coded entry (concept) has any subconcepts.
	 *
	 * @since DLBExt 1.0
	 * @param conceptname concept name
	 * @return Boolean TRUE if it has; FALSE, if otherwise
	 */
	public Boolean hasChildren(String conceptname)
    {
		return getFetchedHasSubs(conceptname);
    }



     /** Determines if concept1 is a subconcept of concept2 based on their names.
      *
      * @since DLBExt 1.0
      * @param concept1 - the name of the first concept
      * @param concept2 - the name of the second concept
      * @return a Boolean. True if concept1 is a subconcept of concept2, False
      *      if concept1 isn't a sub concept of concept2; null, otherwise.
      */


     public boolean isSubConcept(java.lang.String concept1,
		                         java.lang.String concept2)
	 {
		Boolean Bool_obj = isChild(concept1, concept2);
		if (Bool_obj == null) return false;
		return Bool_obj.booleanValue();
	 }


//   used
	/** Check if the second concept is a subconcept of the first concept.
	 *
	 * @since DLBExt 1.0
	 * @param concept1 the nane of the first concept
	 * @param concept2 the name of the second concept
	 * @return Boolean TRUE if it is; FALSE, if otherwise
	 */
	public Boolean isChild(java.lang.String concept1,
	                       java.lang.String concept2)

    {
		try{

			String code1 = nameToCode(codingScheme, version, tag, concept1);
			if (code1 == null)
			{
				getLogger().warn("Concept " + concept1 + " not found.");
				return null;
			}

			String code2 = nameToCode(codingScheme, version, tag, concept2);
			if (code2 == null)
			{
				getLogger().warn("Concept " + concept2 + " not found.");
				return null;
			}

			//String associationName = IS_A;

		boolean direction = false;
		String is_a = getHasSubtype(codingScheme);
		if (is_a.compareTo(IS_A_OBO) == 0) direction = !direction;

			//String associationName = getHasSubtype(codingScheme);
			return areAssociatedConcepts(codingScheme, version, tag,
										code1,
										code2,
										direction,
										is_a);
		}
		catch (Exception e){
			//TODO add logger
			return Boolean.FALSE;
		}
    }


	/** Checks if concept with code1 is a subconcept of the concept with code2
	 *
	 * @since DLBExt 1.0
	 * @param code1 the first concept code
	 * @param code2 the second concept code
	 * @return boolean, true, if it is; false, otherwise
	 */

	public boolean isSubConceptByCode(java.lang.String code1, java.lang.String code2)
	{
	    Boolean Bool_val = isChildByCode(code1, code2);
	    if (Bool_val == null) return false;

	    if (Bool_val.equals(Boolean.TRUE)) return true;
	    return false;
	}


	/** Checks if concept with code1 is a subconcept of the concept with code2
	 *
	 * @since DLBExt 1.0
	 * @param code1 the first concept code
	 * @param code2 the second concept code
	 * @return boolean, true, if it is; false, otherwise
	 */
	public Boolean isChildByCode(java.lang.String code1,
								 java.lang.String code2)
	{
		try {
			Boolean isValidCode = isConceptCodeValid(code1);
			if (!isValidCode)
			{
				getLogger().warn("Invalid code: " + code1);
				return null;
			}
			isValidCode = isConceptCodeValid(code2);
			if (!isValidCode)
			{
				getLogger().warn("Invalid code: " + code2);
				return null;
			}

			boolean direction = false;
			String is_a = getHasSubtype(codingScheme);
			if (is_a.compareTo(IS_A_OBO) == 0) direction = !direction;


			return areAssociatedConcepts(codingScheme,
										version, tag,
										code1,
										code2,
										direction,
										is_a);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			return Boolean.FALSE;
		}
	}



    /** Gets all CodedEntry instances corresponding to subconcepts of a given concept.
     *
     * @since DLBExt 1.0
     * @param code a concept code
     * @return a V ector containing all CodedEntry instances corresponding to
     *     subconcepts of a given concept.
     */


    private Vector<CodedEntry> getAllSubconcepts(String code)
	{

		boolean direction = true;
		String is_a = getHasSubtype(codingScheme);
		if (is_a.compareTo(IS_A_OBO) == 0) direction = !direction;


        return getAssociatedCodedEntries(codingScheme,
										 version,
										 tag,
	                                     code,
										 null,
										 direction,
										 //IS_A,
										 is_a,
										 -1,
										 -1
										 );
	}
//  used
	/** Gets the names of all subconcepts of the conceptName argument.
	 *
	 * @since DLBExt 1.0
	 * @param conceptName a Concept name
	 * @return an array of Concept names which are sub concepts of conceptName;
	 *     return null, if connection fails or an exception is thrown.
	 */
	public Vector getAllSubConceptNames(java.lang.String conceptName)
    {
		String code = nameToCode(codingScheme, version, tag, conceptName);

		if (code == null) return null;
        Vector<CodedEntry> w = getAllSubconcepts(code);
        if (w == null) return null;

        try {
			Vector<String> v = new Vector<String>();
			for (int i=0; i<w.size(); i++)
			{
				CodedEntry ce = (CodedEntry) w.elementAt(i);
				//v.add(codeToName(codingScheme, ce.getConceptCode()));
				v.add(codeToName(codingScheme, version, ce.getConceptCode()));
			}
			return v;
	    } catch (Exception e) {
			return null;
		}
    }


    /** Gets the codes, or the names, of all subconcepts of the specified concept
     *
     * @since DLBExt 1.0
     * @param inputvalue input value
     * @param inputcodeObj Boolean TRUE if the input value is a concept
     *      code; FALSE, if it's a name
     * @param outputcodeObj Boolean TRUE if the output value is a
     *     concept code; FALSE, if it's a name
     * @return a vector containing the codes, or the names, of all subconcepts
     *      of the specified concept; return null, if connection fails or an
     *     exception is thrown.
     */

    public Vector getSubConcepts(String inputvalue, Boolean inputcodeObj, Boolean outputcodeObj)
    {
		return getSubConcepts(inputvalue, inputcodeObj.booleanValue(),
		                      outputcodeObj.booleanValue());
	}

//  used
    /** Gets the codes, or the names, of all subconcepts of the specified concept
     *
     * @param inputvalue input value
     * @param inputcode boolean TRUE if the input value is a concept
     *     code; FALSE, if it's a name
     * @param outputcode boolean TRUE if the output value is a concept
     *     code; FALSE, if it's a name
     * @return a vector containing the codes, or the names, of all subconcepts
     *      of the specified concept; return null, if connection fails or an
     *     exception is thrown.
     */
    public Vector getSubConcepts(String inputvalue, boolean inputcode, boolean outputcode)
	{
		try {
			String code = inputvalue;

			if (!inputcode)
			{
				code = nameToCode(codingScheme, version, tag, inputvalue);
			}

			//String relation = IS_A;
			//String relation = getHasSubtype(codingScheme);
            boolean direction = true;
            String is_a = getHasSubtype(codingScheme);
            if (is_a.compareTo(IS_A_OBO) == 0) direction = !direction;

			Vector w = findRelatedCodes(codingScheme, version, tag, code, is_a, direction);

			Vector<String> v = new Vector<String>();

			if (w == null)
			{
				getLogger().warn("findRelatedCodes returns null.");
				return null;
			}
			else if (w.size() == 0)
			{
				return v;
			}

			if (outputcode)
			{
				return w;
			}

			for (int i=0; i<w.size(); i++)
			{
				String t = (String) w.elementAt(i);

				t = findConceptNameByCode(codingScheme,
									   version,
									   t);
				v.add(t);
			}
			return v;
	    } catch (Exception e) {
			getLogger().error("findRelatedCodes threw exception.");
			return null;
		}
	}

    /** Checks if a concept has any superconcept.
     *
     * @since DLBExt 1.0
     * @param conceptname the specified concept name
     * @return a Boolean. true - conceptname has any superconcept; false - it
     *     has no superconcept; null, if connection fails or an exception is
     *     thrown.
     */

    public boolean hasSuperconcepts(String conceptname)
    {
	    Boolean Bool_val = hasParents(conceptname);
	    if (Bool_val == null) return false;
	    return Bool_val.booleanValue();
	}
//  used
	/** Checks if a concept has any super concepts.
	 *
	 * @since DLBExt 1.0
	 * @param conceptname concept name
	 * @return Boolean TRUE if the concept has; FALSE, if otherwise
	 */
	public Boolean hasParents(String conceptname)
    {
		return getFetchedHasSups(conceptname);
    }


    /** Checks if a concept has any super concepts.
     *
     * @param inputvalue concept name
     * @param isCode concept name
     * @return boolean TRUE if the concept has; FALSE, if otherwise
     */
    public boolean hasSuperconcepts(String inputvalue, boolean isCode)
    {
	    Boolean Bool_val = hasParents(inputvalue, isCode);
	    if (Bool_val == null) return false;
	    return Bool_val.booleanValue();
	}

	/** Checks if a concept has any super concepts.
	 *
	 * @since DLBExt 1.0
	 * @param inputvalue input value
	 * @param isCode boolean TRUE if the input value is a concept code;
	 *     FALSE, if it's a name
	 * @return Boolean TRUE if the concept has; FALSE, if otherwise
	 */
	public Boolean hasParents(String inputvalue, boolean isCode)
    {
		return getFetchedHasSups(inputvalue, isCode);

    }



   /** Gets the codes (or the names) of all superconcepts of the specified concepts.
    *
    * @since DLBExt 1.0
    * @param inputvalue code or the specified concept name
    * @param inputcodeObj boolean TRUE if the input value is a concept
    *     code; FALSE, if it's a name
    * @param outputcodeObj boolean TRUE if the output value is a concept
    *      code; FALSE, if it's a name
    * @return a vector containing the codes (or the names) of all
    *     superconcepts of the specified concepts; return null, if an
    *     exception is thrown.
    */
//	used
   public Vector getSuperConcepts(String inputvalue, Boolean inputcodeObj, Boolean outputcodeObj)
   {
	   return getSuperConcepts(inputvalue, inputcodeObj.booleanValue(), outputcodeObj.booleanValue());
   }
// used
   /** Gets the codes (or the names) of all superconcepts of the specified concepts.
    *
    * @since DLBExt 1.0
    * @param inputvalue code or the specified concept name
    * @param inputcode boolean TRUE if the input value is a concept
    *     code; FALSE, if it's a name
    * @param outputcode boolean TRUE if the output value is a concept
    *     code; FALSE, if it's a name
    * @return the codes (or the names) of all superconcepts of the specified
    *     concepts.
    */
   public Vector<String> getSuperConcepts(String inputvalue, boolean inputcode, boolean outputcode)
   {
		try {
			String code = inputvalue;
			if (!inputcode)
			{
				code = nameToCode(codingScheme, version, tag, inputvalue);
			}

			if (code == null)
			{
				System.out.println("WARNING: getSuperConcepts -- Code " + inputvalue + " not identifiable.");
			}


            boolean direction = false;
            String is_a = getHasSubtype(codingScheme);

            if (is_a.compareTo(IS_A_OBO) == 0) direction = !direction;

			Vector<String> w = findRelatedCodes(codingScheme, version, tag, code, is_a, direction);
			Vector<String> v = new Vector<String>();

			if (w == null)
			{
				return null;
			}
			else if (w.size() == 0)
			{
				return v;
			}

			if (outputcode)
			{
				return w;
			}

			for (int i=0; i<w.size(); i++)
			{
				String t = (String) w.elementAt(i);
				//t = findConceptNameByCode(codingScheme, version, tag, t);
				t = codeToName(codingScheme, version, tag, t);
				v.add(t);
			}
			return v;
	    } catch (Exception e) {
			return null;
		}
	}


	/** Gets associated concepts
	 *
	 * @since DLBExt 1.0
	 * @param code concept code
	 * @param relation association name
	 * @return a Vector of AssociatedConcept objects
	 */
	private Vector getAssociatedConcepts( String code,
	                                      String relation)
    {
        return getAssociatedConcepts(
			                               codingScheme,
			                               version,
			                               tag,
	                                       code,
	                                       relation);
    }



	/** Gets tree data from the server (method for supporting DTS Browser development).
	 *
	 * @since DLBExt 1.0
	 * @param rootname the name of the root concept
	 * @param directionB a boolean variable set to true if tree traverses up,
	 *      to false, if otherwise
	 * @param levels maximum number of levels
	 * @param propertyname - Provides the property name with which to
	 *     populate returned concepts.
	 * @return The root (DefaultMutableTreeNode) of the tree
	 */

	// For browser use
	public DefaultMutableTreeNode getTree(String rootname,
							boolean directionB,
							int levels,
							String propertyname
							)
	{
		String code = nameToCode(codingScheme, version, tag, rootname);

		if (code == null) return null;

		int ASD_index = 0;

		CodedEntry ce = getConcept(code, ASD_index);
		TreeNode node = new TreeNode();
		node.setConcept(ce);
		DefaultMutableTreeNode dmtn = new DefaultMutableTreeNode(node);

		dmtn = populateTreeNodes(dmtn);
		return dmtn;

	}


	/** Populate the content of a TreeNode instance (the UserObject instance of a DefaultMutableTreeNode).
	 *
	 * @since DLBExt 1.0
	 * @param dmtn a  DefaultMutableTreeNode instance
	 * @return a  DefaultMutableTreeNode instance
	 */
	private DefaultMutableTreeNode populateTreeNodes(DefaultMutableTreeNode dmtn)
	{
		TreeNode nd = (TreeNode) dmtn.getUserObject();
		CodedEntry ce = nd.getConcept();
		String code = ce.getConceptCode();
		boolean buildReferencedEntries = true;

		boolean direction = true;
		String is_a = getHasSubtype(codingScheme);
		if (is_a.compareTo(IS_A_OBO) == 0) direction = !direction;

        Vector v = getAssociatedCodedEntries(codingScheme,
                                           version, //null
                                           is_a,
                                           code,
                                           direction,
                                           buildReferencedEntries);

        if (v == null || v.size() == 0)
        {
			return dmtn;
		}

        //int ASD_index = 0;
        for (int i=0; i<v.size(); i++)
        {
			CodedEntry c = (CodedEntry) v.elementAt(i);
			if (c != null)
			{
				String subconcept_code = c.getConceptCode();
				//String subconcept_name = codeToName(codingScheme, version, subconcept_code);

				CodedEntry child_concept = new CodedEntry();
				child_concept.setConceptCode(subconcept_code);

				TreeNode node = new TreeNode();
				node.setConcept(child_concept);

				DefaultMutableTreeNode childnode = new DefaultMutableTreeNode(node);
				childnode = populateTreeNodes(childnode);
				dmtn.add(childnode);
		    }
		}
		return dmtn;
	}



	//////////////////////////////////////////////////////////////////////////////////////////
	//  SORT
	//////////////////////////////////////////////////////////////////////////////////////////

	/** A private method for performing quick sort
	 *
	 * @since DLBExt 1.0
	 * @param a input vector of strings
	 * @param lo0 initial low value
	 * @param hi0 initial high value //081403
	 */
	private void QuickSort(Vector<String> a, int lo0, int hi0)
	{
		int lo = lo0;
		int hi = hi0;
		String mid;

		if ( hi0 > lo0)
		{
			mid = (String) a.elementAt( ( lo0 + hi0 ) / 2 );
			while( lo <= hi )
			{
				while(lo < hi0)
				{
					String s_lo = (String) a.elementAt(lo);
					if (s_lo.compareTo(mid) < 0)
						++lo;
					else
						break;
				}

				while(hi > lo0)
				{
					String s_hi = (String) a.elementAt(hi);
					if (s_hi.compareTo(mid) > 0)
						--hi;
					else
						break;
				}

				if( lo <= hi )
				{
					swap(a, lo, hi);
					++lo;
					--hi;
				}
			}
			if( lo0 < hi )
				QuickSort( a, lo0, hi );

			if( lo < hi0 )
				QuickSort( a, lo, hi0 );

		}
	}

	/** A private method for swapping two string elements in a vector
	 *
	 * @since DLBExt 1.0
	 * @param a input vector of strings
	 * @param i first position
	 * @param j second position
	 */
	private void swap(Vector<String> a, int i, int j)
	{
		String T;
		T = (String) a.elementAt(i);
		a.setElementAt(a.elementAt(j), i);
		a.setElementAt(T, j);
	}

	/** A private method for QuickSort.
	 *
	 * @since DLBExt 1.0
	 * @param a an array of strings
	 */
	private void sort(Vector<String> a)
	{
		if (a.size() == 0)
			return;

		QuickSort(a, 0, a.size() - 1);
	}

	//112603
	/** Construct an object to a byte array
	 *
	 * @since DLBExt 1.0
	 * @param ob an Object instance
	 * @param root a DefaultMutableTreeNode object
	 * @return a byte array
	 * @since.lexrpc 1.3.3
	 */
	private byte[] objectToByteArray(Object ob)
	{
		if (ob == null) return null;

		byte[] byteNode= null;
		try	{
			ByteArrayOutputStream baos=new ByteArrayOutputStream();
			ObjectOutputStream oos=new ObjectOutputStream(baos);
			oos.writeObject(ob); // Serialize the Class object
			byteNode=baos.toByteArray();
			oos.close();
		}catch (Exception e){
			e.printStackTrace();
		}
		return byteNode;
	}


    /** Searchs concepts
     *  by name.
     *
     * @since DLBExt 1.0
     * @param pattern match pattern
     * @param searchLimit search limit
     * @return matched concepts in a byte array
     */
	//public byte[] searchConcepts(String pattern, int max, String matchAlgorithm)


    public Vector<CodedEntry> findConceptsWithNameMatching(String pattern, int searchLimit)
    {
		CodedEntry[] a = searchConcepts(pattern, searchLimit, defaultAlgorithm);
		return toVector(a);
	}


	/** Gets the Concept intance corresponding to a specific concept code
	 *
	 * @since DLBExt 1.0
	 * @param code concept code
	 * @param ASD_Index
	 * @return the Concept intance corresponding to a specific concept code
	 */
	private CodedEntry getConcept(String code, int ASD_Index)
	{
        return getCodedEntry(codingScheme,
                                    version,
	                                tag,
									code,
									ASD_Index);
	}


	//////////////////////////////////////////////////////////////////////////////////////////
	//  Utility
	//////////////////////////////////////////////////////////////////////////////////////////


	/** Converts a java.util.date to a java.sql.date
	 *
	 * @param utildate a java.util.date object
	 * @return java.sql.Date
	 */
	private java.sql.Date toSqldate(Date utildate)
	{
		if (utildate == null) return null;

		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
		String s = sdf.format(utildate);
		return java.sql.Date.valueOf(s);
	}


	//////////////////////////////////////////////////////////////////////////////////////////
	//  HISTORY
	//////////////////////////////////////////////////////////////////////////////////////////
	/** Checks if a concept code is valid.
	 *
	 * @param code concept code
	 * @return boolean true - concept code is valid; false - if it is not.
	 */


	public Boolean isConceptCodeValid(String code){
		try {
            return isCodeInGraph( codingScheme,
								          version,
								          tag,
								          code);

       }catch (Exception e) {
			getLogger().error("ConvenienceMethods isConceptCodeValid "+e);
		}
		return null;
	}

	/** Checks if a concept name is valid.
	 *
	 * @param conceptname concept name
	 * @return boolean true - concept name is valid; false - if it is not.
	 */
	public Boolean isConceptNameValid(String conceptname){
		String code = nameToCode(codingScheme, version, tag, conceptname);
		if (code == null)
		{
			return Boolean.FALSE;
		}
		return Boolean.TRUE;
	}

	/** Checks if a concept history is available.
	 *
	 * @return boolean true - if concept history is available; false - if it is
	 *     not.
	 */
	public boolean isHistoryAvailable(){
		//if (lbSvc != null) return true;
		//072807
		Vector v = getHistoryDates();
		if (v == null || v.size() == 0) return false;
		return true;
	}


	public boolean isHistoryAvailable(String codingSchemeName){
		//if (lbSvc != null) return true;
		//072807
		Vector v = getHistoryBaselineDates(codingSchemeName);
		if (v == null || v.size() == 0) return false;
		return true;
	}

	/** Checks if a concept in a namespace has a specific role
	 * and if so whether the role has a specific value
	 *
	 * @param role_name role name
	 * @param role_value role value
	 * @param Concept_in the name of the specific ontylog concept
	 * @param isCode
	 * @return A Boolean variable with value set to TRUE if it has; FALSE,
	 *     otherwise
	 */


	public Boolean containsRole(String role_name, String role_value,
					            String Concept_in, boolean isCode)
	{
		if (!hasRelation(codingScheme, ROLE_RELATION)) return Boolean.FALSE;

		Vector v = getFetchedRoles(Concept_in, isCode);
		if (v == null) return null;

		//Vector u = new Vector();
		for (int i=0; i<v.size(); i++)
		{
			Role r = (Role) v.elementAt(i);
			String s = r.getName() + "$" + r.getValue();
			if (s.compareTo(role_name + "$" + role_value) == 0) return Boolean.TRUE;
		}
		return Boolean.FALSE;
	}


//Used
	public Boolean containsRole(String role_name, String role_value,
					String Concept_in)
	{
		return containsRole(role_name, role_value, Concept_in, false);
	}


	/** Checks if a ontylog concept in a namespace has a specific inverse role
	 * and if so whether the role has a specific value
	 *
	 * @since DLBExt 1.0
	 * @param role_name role name
	 * @param role_value role value
	 * @param Concept_in the name of the specific ontylog concept
	 * @param isCode
	 * @return A Boolean variable with value set to TRUE if it has; FALSE,
	 *     otherwise
	 */

	public Boolean containsInverseRole(String role_name, String role_value,
					                   String Concept_in, boolean isCode)
	{
		if (!hasRelation(codingScheme, ROLE_RELATION)) return Boolean.FALSE;

		Vector v = getFetchedInverseRoles(Concept_in, isCode);
		if (v == null) return null;

		//Vector u = new Vector();
		for (int i=0; i<v.size(); i++)
		{
			Role r = (Role) v.elementAt(i);
			String s = r.getName() + "$" + r.getValue();
			if (s.compareTo(role_name + "$" + role_value) == 0) return Boolean.TRUE;
		}
		return Boolean.FALSE;

		/*
		String s = role_name + "$" + role_value;
		if (v.contains(s))
		{
			return Boolean.TRUE;
		}
		return Boolean.FALSE;
		*/
	}


//Used
	public Boolean containsInverseRole(String role_name, String role_value,
					                   String Concept_in)
	{
		return containsInverseRole(role_name, role_value, Concept_in, false);
	}


	/** Finds the code of the specified concept.
	 *
	 * @since DLBExt 1.0
	 * @param conceptname the specified concept name
	 * @return the specified concept code; return null, if connection fails or an
	 *      exception is thrown.
	 */
//	used
	public String getConceptCodeByName(java.lang.String conceptname)
    {
		return nameToCode(codingScheme, version, tag, conceptname);
    }
//	used
	/** Finds the code of the specified concept.
	 *
	 * @since DLBExt 1.0
	 * @param conceptCode concept code
	 * @return the specified concept code; return null, if connection fails or an
	 *      exception is thrown.
	 */
	public String getConceptNameByCode(java.lang.String conceptCode)
    {
		return codeToName(codingScheme, version, tag, conceptCode);
    }

	//////////////////////////////////////////////////////////////////////////////////////////
	//  ROLE RELATIONSHIPS
	//////////////////////////////////////////////////////////////////////////////////////////
    /** Gets the names of all role types defined in the vocabulary.
     *
     * @since DLBExt 1.0
     * @return a vector containing the names of all role types defined in the
     *     vocabulary; null, if connection fails or an exception is thrown.
     */
//	used
    public Vector getAllRoleNames()
	{
		return getSupportedRoleNames(codingScheme);
	}

    public Vector getAllRoleNames(String codingScheme)
	{
		return getSupportedRoleNames(codingScheme);
	}


	/** Gets the values of a particular role owned by the specified concept.
	 *
	 * @since DLBExt 1.0
	 * @param conceptname the specified concept name
	 * @param rolename the specified role name
	 * @return the values of a particular role owned by the specified concept;
	 *     null, if connection fails or an exception is thrown.
	 */
	public Vector getRoleValues(String conceptname,
	                            String rolename)
    {
		String code = nameToCode(codingScheme, version, tag, conceptname);
		if (code == null) return null;
		String tag = null;

  	    Vector<Role> w = getRoles(codingScheme,
										  version,
										  tag,
										  code,
										  rolename);

        if (w == null) return null;
        Vector<String> v = new Vector<String>();
        for (int i=0; i<w.size(); i++)
        {
		    NameAndValue nv = w.elementAt(i).toNameAndValue();
	        v.add(nv.getContent());
		}
		return v;
    }

//	used
	/** Gets the names and values of roles owned by the specified concept.
	 * Each name and value pair is returned as a string delimited by a '$' sign (i.e., name$value).
	 *
	 * @since DLBExt 1.0
	 * @param conceptname the specified concept name
	 * @return a vector containing the names and values of roles owned by a
	 *     specified concept; null, if connection fails or an exception is
	 *     thrown.
	 */
	public Vector getRolesByConceptName(String conceptname)
    {
		return getFetchedRoles(conceptname);
	}


	//////////////////////////////////////////////////////////////////////////////////////////
	//  PROPERTIES
	//////////////////////////////////////////////////////////////////////////////////////////

    /** Gets the names of all property types defined in the vocabulary.
     *
     * @since DLBExt 1.0
     * @return a vector containing the names of all property types defined in
     *     the vocabulary; null, if connection fails or an exception is
     *     thrown.
     */
    public Vector getPropertyTypes() // a CodingScheme
	{
		/*
		//org.LexGrid.LexBIG.LexBIGService.CodedNodeSet.PropertyType[] propertytypes = getSupportedPropertyNames(codingScheme, version, null);
		Vector v = new Vector();
		for (int i=0; i<propertytypes.length; i++)
		{
			v.add(propertytypes[i]);
		}
		return v;
		*/
		return getSupportedPropertyNames(codingScheme, version, null);
	}


	/** Gets the names and values of roles owned by the specified concept.
	 *
	 * @since DLBExt 1.0
	 * @param conceptname the specified concept name
	 * @return a vector containing the names and values of properties owned by a
	 *     specified concept; null, if connection fails or an exception is
	 *     thrown.
	 */

//  used
	public Vector getPropertiesByConceptName(String conceptname)
	{
		return getPropertiesByConceptName(codingScheme, version, tag, conceptname);
	}


	public Vector getPropertiesByCode(String conceptcode)
	{
		return getPropertiesByCode(codingScheme, version, conceptcode);
	}
//	used
	/** Gets the values of a particular role owned by the specified concept.
	 *
	 * @since DLBExt 1.0
	 * @param conceptname the specified concept name
	 * @param propertyname property name
	 * @return the values of a particular role owned by the specified concept;
	 *     null, if connection fails or an exception is thrown.
	 */
	public Vector getPropertyValues(String conceptname, String propertyname)
    {
        return getPropertyValues(codingScheme,
	                                     version,
									     tag,
									     conceptname,
									     propertyname);
    }


		/**
		 * Determines if concept1 is a subconcept of concept2 in the specified
		 * namespace. Concept1 and concept2 are names of Ontylog concepts.
		 *
		 * @since DLBExt 1.0
		 * @param concept1 - a concept name
		 * @param concept2 - a concept name
		 * @return true if concept1 is a subconcept of concept2, false if concept1
		 *     isn't a subconcept of concept2 Returns null, if there is an
		 *     exception.
		 */


		public Boolean isSubConceptByName(String concept1,
						String concept2)
		{
			try{
				String code1 = nameToCode(codingScheme, version, tag, concept1);
				String code2 = nameToCode(codingScheme, version, tag, concept2);
				return isSubConceptByCode(code1, code2);
			}
			catch (Exception e){
				return Boolean.FALSE;
			}
		}



		/**
		 * Determines if concept1 is a subconcept of a collection of concepts in the specified
		 * namespace.
		 *
		 * @since DLBExt 1.0
		 * @param concept1 - a concept name
		 * @param concepts_in a list of concept names
		 * @return a vector of Boolean variables indicating whether concept1 is a
		 *     subconcept of each concept in the list
		 */
		public Vector<Boolean> isSubConceptByName(String concept1,
						Vector concepts_in)
		{
            String code1 = nameToCode(codingScheme, version, tag, concept1);
            Vector<Boolean> v = new Vector<Boolean>();
            for (int i=0; i<concepts_in.size(); i++)
            {
				String name = (String) concepts_in.elementAt(i);
				String code2 = nameToCode(codingScheme, version, tag, name);
		        v.add(isSubConceptByCode(code1, code2));
			}
			return v;
		}



		/** "@since DLBExt 1.0
		 *
		 * @param associationtypeName
		 */
		private SupportedAssociation findAssociationTypeByName(String associationtypeName) {
            CodingScheme cs = getCodingScheme(codingScheme, version);
		    SupportedAssociation[] assos = cs.getMappings().getSupportedAssociation();
		    for (int i=0; i<assos.length; i++)
		    {
				if (assos[i].getContent().compareTo(associationtypeName) == 0) return assos[i];
			}
			return null;
	    }


		/**
		 * Gets an array of Concept objects that represent the same concepts as the argument
		 * array of concept codes.
		 *
		 * @since DLBExt 1.0
		 * @param conceptCodes_in a vector of concept codes
		 * @return a vector of Concept corresponding to the argument concept codes,
		 *     with all attributes filled; it returns null, if there is an
		 *     exception.
		 */
		public Vector<CodedEntry> findConceptsByCode(Vector<String> conceptCodes_in)
		{
			if (conceptCodes_in == null) return null;
            Vector<CodedEntry> v = new Vector<CodedEntry>();
            try {

				for (int i=0; i<conceptCodes_in.size(); i++)
				{
					String code = (String) conceptCodes_in.elementAt(i);
					CodedEntry ce = getCodedEntry(codingScheme, version, tag, code); // asd??? (to be implemented)
					if (ce == null)
					{
						getLogger().warn("WARNING: findConceptsByCode ce is null.");
						v.add(null);
					}
					else
					{
						v.add(ce);
					}
				}

			} catch (Exception e) {
				getLogger().error("ConvenienceMethods findConceptsByCode exception thrown.");
				return null;
			}
			return v;
		}

//		used
		/**
		 * Retrieves all concept property types.
		 *
		 * @since DLBExt 1.0
		 * @return a vector of all supported property types. Returns an empty
		 *     vector if there are no supported property type. Returns null, if
		 *     there is an exception.
		 */
		public Vector getAllConceptPropertyTypes()
		{
			return getConceptPropertyTypes();
		}


		/**
		 * Retrieves all concept property types within a given namespace.
		 *
		 * @since DLBExt 1.0
		 * @return a vector of PropertyType in the namespace. Returns an empty
		 *     vector if there are none. Returns null, if
		 *     there is an exception.
		 */
		public Vector<String> getConceptPropertyTypes()
		{
			Vector<SupportedProperty> w = getSupportedProperties(codingScheme, version, tag);
			if (w == null) return null;

			Vector<String> v = new Vector<String>();
			for (int i=0; i<w.size(); i++)
			{
				SupportedProperty sp = (SupportedProperty) w.elementAt(i);
				//v.add(sp.getContent());
				v.add(sp.getLocalId());
			}
			return v;
		}


//////////////////
// History
//////////////////

		/**
		 * Gets all distinct dates in concept_history table.
		 *
		 * @since DLBExt 1.0
		 * @return a vector of Date All distinct dates in concept_history table.
		 *     Returns null, if there is an exception.
		 */


		public Vector getHistoryDates()
		{
			return getHistoryBaselineDates(codingScheme);
		}



		/**
		 * Gets the baseline date of creation of the specified concept.
		 *
		 * @since DLBExt 1.0
		 * @param code concept code
		 * @return Date The baseline date of creation of inCode from either "Create"
		 *      or "Split" edit actions. Returns null, if there is an exception.
		 */


		public java.util.Date getConceptCreationDate(String code)
		{
			return getConceptCreationDate(codingScheme, code);
		}


		public java.util.Date getCreationDate(String code)
		{
			return getConceptCreationDate(codingScheme, code);
		}

//		used
		/**
		 * Gets the descendant concept codes of the specified concept. The search extends to a final
		 * baseline date starting from an initial baseline date. The search is based on a boolean value. If
		 * the value is true, the method only searches for the active concepts at the final baseline date. If
		 * the value is false, the method searches for all descendant concepts, whether active or retired.
		 * The search is only applied to "split/merge" actions and the specified concept could be the
		 * descendant concept of itself.
		 *
		 * @since DLBExt 1.0
		 * @param inCode The specified concept's code.
		 * @param atBaseline The boolean value to specify the search type.
		 * @param iBaselineDate_in initial baseline date
		 * @param fBaselineDate_in final baseline date
		 * @return a vector of String The returned concept codes which are the
		 *     descendant concepts of inCode. Returns null, if there is an
		 *     exception.
		 */
		public Vector getDescendantCodes(
						String inCode,
						boolean atBaseline,
						java.util.Date iBaselineDate_in,
						java.util.Date fBaselineDate_in)
		{
		    return getDescendantCodes(
				        codingScheme,
						inCode,
						atBaseline,
						iBaselineDate_in,
						fBaselineDate_in);
		}



		/**
		 * Gets the descendant concepts' codes of the specified concept involved in "split or merge"
		 * actions that occurred in the specified baseline date. The specified concept is included if it's
		 * not retired in the specified baseline date.
		 *
		 * @since DLBExt 1.0
		 * @param inCode - The specified concept's code.
		 * @param baselineDate_in baseline date
		 * @return a vector of String The returned concept codes are the descendant
		 *     concepts of inCode which are involved in "split or merge" actions
		 *     that occurred in baselineDate. Returns null, if there is an
		 *     exception.
		 */
		public Vector getCodeActionChildren(
						String inCode,
						java.util.Date baselineDate_in)
		{
		    return getCodeActionChildren(
				        codingScheme,
						inCode,
						baselineDate_in);
 		}



		/**
		 * Gets the descendant concepts' CodeActionDate of the specified concept. A CodeActionDate
		 * contains a concept code and a baseline date. The method searches for all descendant
		 * concepts of the specified concept, whether active or retired. The search is only applied to the
		 * "split/merge" actions and the specified concept could be the descendant concept of itself.
		 *
		 * @since DLBExt 1.0
		 * @param inCode - The specified concept's code.
		 * @param iBaselineDate_in initial baseline date
		 * @param fBaselineDate_in final baseline date
		 * @return a vector of CodeActionDate The returned CodeActionDates contain
		 *     the descendant concepts of inCode and the correspond baseline dates.
		 *     Returns null, if there is an exception.
		 */
		public Vector getDescendantCodeActionDates(
						String inCode,
						java.util.Date iBaselineDate_in,
						java.util.Date fBaselineDate_in)
		{
		     return getDescendantCodeActionDates(
				        codingScheme,
						inCode,
						iBaselineDate_in,
						fBaselineDate_in);

		}


//		used
		/**
		 * Gets the ancestor concept codes of the specified concept. The search extends to a final
		 * baseline date starting from an initial baseline date. The search is based on a boolean value. If
		 * the value is true, the method only searches for the active concepts at the initial baseline date.
		 * If the value is false, the method searches for all ancestor concepts, whether active or retired.
		 * The search is only applied to "split/merge" actions and the specified concept could be the
		 * ancestor concept of itself.
		 *
		 * @since DLBExt 1.0
		 * @param inCode - The specified concept's code.
		 * @param atBaseline - The boolean value to specify the search type.
		 * @param iBaselineDate_in initial baseline date
		 * @param fBaselineDate_in final baseline date
		 * @return a vector of String The returned concept codes which are the
		 *     ancestor concepts of inCode. Returns null, if there is an exception.
		 */
		public Vector getAncestorCodes(
						String inCode,
						boolean atBaseline,
						java.util.Date iBaselineDate_in,
						java.util.Date fBaselineDate_in)
		{
		    return getAncestorCodes(
				        codingScheme,
						inCode,
						atBaseline,
						iBaselineDate_in,
						fBaselineDate_in);

		}

		// 073006 (v1rc6 addition)
		public Vector getAncestors(String inCode)
		{
		    return getAncestors(
				        codingScheme,
						inCode);
		}

 		// 073006 (v1rc6 addition)
		public Vector getDescendants(String inCode)
		{
		    return getDescendants(
				        codingScheme,
						inCode);
		}


		/**
		 * Gets the ancestor concepts' codes of the specified concept involved in "split or merge"
		 * actions that occurred in the specified baseline date. The specified concept is included if it's
		 * an ancestor of itself.
		 *
		 * @since DLBExt 1.0
		 * @param inCode - The specified concept's code.
		 * @param baselineDate_in baseline date
		 * @return a vector of String The returned concept codes are the ancestor
		 *     concepts of inCode which are involved in "split or merge" actions
		 *     that occurred in baselineDate. Returns null, if there is an
		 *     exception.
		 */
		public Vector getCodeActionParents(
						String inCode,
						java.util.Date baselineDate_in)
		{
		    return getCodeActionParents(
				        codingScheme,
						inCode,
						baselineDate_in);

		}



		/**
		 * Gets the ancestor concepts' CodeActionDate of the specified concept. A CodeActionDate
		 * contains a concept code and a baseline date. The method searches for all ancestor concepts
		 * of the specified concept, whether active or retired. The search is only applied to
		 * "split/merge" actions and the specified concept could be the ancestor concept of itself.
		 *
		 * @since DLBExt 1.0
		 * @param inCode - The specified concept's code.
		 * @param iBaselineDate_in initial baseline date
		 * @param fBaselineDate_in final baseline date
		 * @return a vector of CodeActionDate The returned CodeActionDates contain
		 *     the ancestor concepts of inCode and the correspond baseline dates.
		 *     Returns null, if there is an exception.
		 */
		public Vector getAncestorCodeActionDates(
						String inCode,
						java.util.Date iBaselineDate_in,
						java.util.Date fBaselineDate_in)
		{
	         return getAncestorCodeActionDates(
		                codingScheme,
						inCode,
						iBaselineDate_in,
						fBaselineDate_in);
		}



		/**
		 * Gets all dates in which a specified action takes place on the specified concept. The returned
		 * dates might not be distinct when the specified action is "split/merge".
		 *
		 * @since DLBExt 1.0
		 * @param code concept code
		 * @param action
		 * @return a vector of Date All dates in which iAction takes place on
		 *     inCode. Returns null, if there is an exception.
		 */

//		used
		public Vector getConceptEditActionDates(
						String code,
						String action)
		{
			 int iAction = toActionIndex(action);
             return getConceptEditActionDates(codingScheme, code, iAction);
		}


		public Vector getConceptEditActionDates(
						String code,
						int iAction)
		{
             return getConceptEditActionDates(codingScheme, code, iAction);
		}


		/**
		 * Gets all EditAction of the specified concept. A EditAction contains a edit date and an editing
		 * action. The method searches for all editing actions and the corresponding editing dates on the
		 * specified concept.
		 *
		 * @since DLBExt 1.0
		 * @param inCode - The specified concept's code.
		 * @return a vector of EditAction The returned EditActions contain all
		 *     editing actions and the corresponding editing dates on inCode.
		 *     Returns null, if there is an exception.
		 */
//		used
		public Vector getConceptEditAction(String inCode)
		{
			return getConceptEditAction(codingScheme, inCode);
		}

		/**
		 * Checks if the specified concept is retired or not.
		 *
		 * @since DLBExt 1.0
		 * @param inCode - The specified concept's code.
		 * @return a boolean True - inCode is retired. False - inCode is active.
		 *     Returns null, if there is an exception.
		 */

		public boolean isConceptRetired(String inCode)
		{
			Boolean Bool_val = isRetired(inCode);
			if (Bool_val == null) return false;
			return Bool_val.booleanValue();
		}

//		used
		public Boolean isRetired(String inCode)
		{
            return isCodeRetired(codingScheme, inCode);
		}


       /**
        * Checks if the specified concept is edited or not during a date range. The check is depended
        * on a boolean value. If the boolean value is true, the method only checks if the specified
        * concept was merged or split in the date range. If the boolean value is false, the method
        * checks if the specified concept was edited in any way in the date range.
        *
        * @since DLBExt 1.0
        * @param inCode - The specified concept's code.
		* @param mrg_splt - True - only check for "merge or split" editing action.
		*      False - check for any editing action.
		* @param iBaselineDate_in - The initial baseline date.
		* @param fBaselineDate_in - The final baseline date.
			* @return a boolean True - inCode is edited. False - inCode isn't edited.
		*     Returns null, if there is an exception.
			*/


       public boolean isConceptEdited(String inCode,
			boolean mrg_splt,
			java.util.Date iBaselineDate_in,
			java.util.Date fBaselineDate_in)
	   {
           Boolean Bool_val = isEdited(inCode,
			 mrg_splt,
			 iBaselineDate_in,
			 fBaselineDate_in);
		   return Bool_val.booleanValue();
	   }



		public Boolean isEdited(
						String inCode,
						boolean mrg_splt,
						java.util.Date iBaselineDate_in,
						java.util.Date fBaselineDate_in)
		{
			return isConceptEdited(
		                codingScheme,
						inCode,
						mrg_splt,
						iBaselineDate_in,
						fBaselineDate_in);
		}


		/**
		 * Gets the number of editing actions on the specified concept registered in the specified
		 * baseline date.
		 *
		 * @since DLBExt 1.0
		 * @param inCode - The specified concept's code.
		 * @param baselineDate_in - The specified baseline date.
		 * @return an int The number of editing actions on inCode registered in
		 *     baselineDate. Returns null, if there is an exception.
		 */
		public int getNumberEditActions(
						String inCode,
						java.util.Date baselineDate_in)
		{
            Integer int_obj = getNumberEditActions(codingScheme, inCode, baselineDate_in);
            return int_obj.intValue();
		}


		/**
		 * Gets the number of editing actions on the specified concept registered in the specified
		 * baseline date.
		 *
		 * @since DLBExt 1.0
		 * @param inCode - The specified concept's code.
		 * @param iBaselineDate_in initial baseline date
		 * @param fBaselineDate_in final baseline date
		 * @return an int The number of editing actions on inCode registered in
		 *     baselineDate. Returns null, if there is an exception.
		 */
		public int getNumberEditActions(
						String inCode,
						java.util.Date iBaselineDate_in,
						java.util.Date fBaselineDate_in)
		{
            Integer int_obj = getNumberEditActions(codingScheme, inCode, iBaselineDate_in,
                fBaselineDate_in);
            return int_obj.intValue();
		}


		/**
		 * Gets all EditActionDate of the specified concept. A EditAction contains a edit date and an editing
		 * action. The method searches for all editing actions and the corresponding editing dates on the
		 * specified concept.
		 *
		 * @since DLBExt 1.0
		 * @param inCode - The specified concept's code.
		 * @param baselineDate_in baseline date
		 * @return a vector of EditActionDate The returned EditActionDates contain
		 *     all editing actions and the corresponding editing dates on inCode.
		 *     Returns null, if there is an exception.
		 */
		public Vector<EditActionDate> getConceptEditAction(
						String inCode,
						java.util.Date baselineDate_in)
		{
			Vector<EditActionDate> w = new Vector<EditActionDate>();
			Vector v = getConceptHistory(codingScheme,
										inCode,
										baselineDate_in,
										baselineDate_in);
			for (int i=0; i<v.size(); i++)
			{
				NCIChangeEvent nce = (NCIChangeEvent) v.elementAt(i);
				EditActionDate e = new EditActionDate();
				e.setEditDate(nce.getEditDate());
				e.setAction(nce.getEditaction().getType());
				w.add(e);
			}
			return w;
	    }


/////////////////////////////////////////////////////
//// API Class: License
/////////////////////////////////////////////////////

		/**
		 * Gets all licenses from the server. If no condition exists, then pass in a empty
		 * String.
		 *
		 * @since DLBExt 1.0
		 * @return Vector a vector of Strings describing the licenses Returns null,
		 *     if there is an exception.
		 */

		public String getLicense()
		{
			return getCopyRight(codingScheme);
		}


		/**
		 * Gets all licenses from the server. If no condition exists, then pass in a empty
		 * String.
		 *
		 * @since DLBExt 1.0
		 * @return Vector a vector of Strings describing the licenses Returns null,
		 *     if there is an exception.
		 */
//		used
		public Vector<String> getAllLicenses()
		{
			Vector<String> v = new Vector<String>();
			v.add(getCopyRight(codingScheme));
			return v;
		}


		/**
		 * Retrieves an array of ContentVersions which are the list of versions for
		 * the given namespace Id in the knowledgebase.
		 *
		 * @since DLBExt 1.0
		 * @param name
		 * @return the ContentVersions of the given namespace Returns null, if there
		 *      is an exception.
		 */
//		used
		public String getVocabularyVersion(String name)
		{
			try {
				if (lbSvc == null)
				{
				   getLogger().warn("WARNING: lbSvc is null.");
				   return null;
				}

//				HashSet hset = new HashSet();
				CodingSchemeRenderingList csrl = lbSvc.getSupportedCodingSchemes();
				CodingSchemeRendering[] csrs = csrl.getCodingSchemeRendering();
				for (int i=0; i<csrs.length; i++)
				{
					CodingSchemeRendering csr = csrs[i];
					CodingSchemeSummary css = csr.getCodingSchemeSummary();
					String formalname = css.getFormalName();
					String localname = css.getLocalName();
					if (formalname.compareTo(name) == 0 || localname.compareTo(name) == 0)
					{
						return css.getRepresentsVersion();
					}
                }
			} catch (Exception e) {
				return null;
		    }
			return null;
		}




		public Vector getVersions()
		{
			return getAvailableCodingSchemeVersions(codingScheme);
		}



      /**
       * Retrieves a vector of root concepts in the default coding system
       *
       * @since DLBExt 1.0
       * @param v
       * @return a vector of root concepts in the default coding system with
       *     attributes specified by ASD_index
       */


      private CodedEntry[] toArray(Vector v)
      {
		  if (v == null) return null;
		  CodedEntry[] a = new CodedEntry[v.size()];
		  for (int i=0; i<v.size(); i++)
		  {
			  a[i] = (CodedEntry) v.elementAt(i);
		  }
		  return a;
	  }

	  public CodedEntry[] getConceptChildRoots()
	  {
		  return getConceptChildRoots(1);
	  }


	  public CodedEntry[] getConceptChildRoots(int ASD_index)
	  {
			Vector w = getRootCodedEntries(codingScheme, version, tag, null, getHasSubtype(codingScheme));

            return toArray(w);
	  }

	  /**
	   * Retrieves a vector of root concepts in the default coding system
	   *
	   * @since DLBExt 1.0
	   * @param ASD_index attribute set descriptor index (0: no attributes; 1:
	   *     all attributes)
	   * @param relationName relation name
	   * @param associationName association name
	   * @return a vector of root concepts in the default coding system with
	   *     attributes specified by ASD_index
	   */
	  public CodedEntry[] getConceptChildRoots(int ASD_index, String relationName, String associationName)
	  {
			Vector w = getRootCodedEntries(codingScheme, version, tag, relationName, associationName);
            return toArray(w);
	  }


		/**
		 * Gets the roles which have been retrieved for this concept.
		 *
		 * @since DLBExt 1.0
		 * @param Concept_in the name of a concept, if isCode is false; it is the
		 *     code of a concept, if isCode is true;
		 * @param isCode a boolean variable indicating whether Concept_in is a
		 *     name or code
		 * @return a vector of the roles Returns null, if there is an exception.
		 */

		public Vector getFetchedRoles(String Concept_in, boolean isCode)
		{
			return getFetchedRoles(codingScheme, Concept_in, isCode);
		}

		public Vector getFetchedRoles(String codingSchemeName, String Concept_in, boolean isCode)
		{
			return getFetchedAssociations(codingSchemeName, Concept_in, isCode, DLBWrapper.ROLE_RELATION);
		}


        private Role conceptAssociation2Role(gov.nih.nci.lexbig.ext.ConceptAssociation asso)
        {
			Role r = new Role(asso.getName(), asso.getValue());
			Vector v = asso.getQualifiers();
			for (int i=0; i<v.size(); i++)
			{
				Qualifier q = (Qualifier) v.elementAt(i);
				r.addQualifier(q);
			}
			return r;
		}

		public Vector<Role> getFetchedAssociations(String codingSchemeName, String Concept_in, boolean isCode, String relation)
		{
			String code = Concept_in;
			if (!isCode)
			{
				code = super.nameToCode(codingSchemeName, version, tag, Concept_in);
				if (code == null)
				{
					getLogger().warn("Invalid code: " + code);
					return null;
				}
			}
            //String is_a = IS_A;
            boolean direction = true;
            String is_a = getHasSubtype(codingSchemeName);
            if (is_a.compareTo(IS_A_OBO) == 0) direction = !direction;

			Vector<Role> v = new Vector<Role>();

    		Vector<ConceptAssociation> w = getConceptAssociations(
			                                   codingSchemeName,
	                                           version,
	                                           tag,
	                                           code,
	                                           true,
	                                           //DLBWrapper.ROLE_RELATION);
	                                           relation);


			if (w == null)
			{
				getLogger().error("ConvenienceMethods getConceptAssociationNameAndValues returns null.");
				return null;
			}


			Vector roles = getSupportedRoleNames();
            for (int i=0; i<w.size(); i++)
            {
				gov.nih.nci.lexbig.ext.ConceptAssociation nv = (gov.nih.nci.lexbig.ext.ConceptAssociation) w.elementAt(i);
				String assoname = nv.getName();
				if (roles.contains(assoname))
				{
					if (nv.getName().compareToIgnoreCase(is_a) != 0)
					{
						v.add(conceptAssociation2Role(nv));
					}
			    }
            }
            return v;
		}


		public Vector getFetchedRoles(String Concept_in)
		{
			return getFetchedRoles(Concept_in, false);
	    }


		/**
		 * Get the number of roles that are available of the types specified in the
		 * ConceptAttributeSetDescriptor used to obtain this Concept.
		 *
		 * @since DLBExt 1.0
		 * @param Concept_in the name of an ontylog concept
		 * @return number of roles of the specified types Returns null, if there is
		 *     an exception.
		 */

		//public Integer getNumberOfSpecifiedRoles(String Concept_in)
		public int getNumberOfSpecifiedRoles(String Concept_in)
		{
		     Vector v = getFetchedRoles(Concept_in);
		     if (v == null) return -1;
		     //return new Integer(v.size());
		     return v.size();
		}


		/**
		 * Gets the inverse roles which have been retrieved for this concept, i.e. the
		 * roles of other concepts which have this concept as a value.
		 *
		 * @since DLBExt 1.0
		 * @param Concept_in the name of an ontylog concept
		 * @return a vector of the inverse roles Returns null, if there is an
		 *     exception.
		 */

		public Vector getFetchedInverseRoles(String Concept_in)
		{
			return getFetchedInverseRoles(Concept_in, false);
		}


		public Vector getFetchedInverseRoles(String Concept_in, boolean isCode)
		{
             return getFetchedInverseRoles(codingScheme, Concept_in, isCode);
        }


		public Vector<Role> getFetchedInverseRoles(String codingSchemeName, String Concept_in, boolean isCode)
		{
			String code = Concept_in;
			if (!isCode)
			{
				code = nameToCode(codingSchemeName, version, tag, Concept_in);
				if (code == null)
				{
					getLogger().error("Invalid concept name: " + Concept_in);
					return null;
				}
			}
            //String is_a = IS_A;
            String is_a = getHasSubtype(codingSchemeName);
            //boolean direction = false;
            //if (is_a.compareTo(IS_A_OBO) == 0) direction = !direction;


			Vector<Role> v = new Vector<Role>();
	        Vector<ConceptAssociation> w = getConceptAssociations(codingSchemeName,
	                                       version,
	                                       tag,
	                                       code,
	                                       false);

			if (w == null)
			{
				getLogger().warn("ConvenienceMethods getConceptAssociationNameAndValues returns null.");
				return null;
			}


			Vector<String> roles = getSupportedRoleNames();
            for (int i=0; i<w.size(); i++)
            {
				//NameAndValue nv = (NameAndValue) w.elementAt(i);
				//String assoname = nv.getName();
				/*
				if (roles.contains(assoname))
				{
					if (nv.getName().compareToIgnoreCase(is_a) != 0)
					{
						//v.add(nv.getName() + "$" + nv.getContent());
						v.add(new Role(nv.getName(), nv.getContent()));
					}
			    }
			    */
				gov.nih.nci.lexbig.ext.ConceptAssociation nv = (gov.nih.nci.lexbig.ext.ConceptAssociation) w.elementAt(i);
				String assoname = nv.getName();
				if (roles.contains(assoname))
				{
					if (nv.getName().compareToIgnoreCase(is_a) != 0)
					{
						v.add(conceptAssociation2Role(nv));
					}
			    }
            }
            return v;
		}



		/**
		 * Get the number of inverse roles that are available of the types specified in
		 * the ConceptAttributeSetDescriptor used to obtain this
		 * Concept.
		 *
		 * @since DLBExt 1.0
		 * @param Concept_in the name of an ontylog concept
		 * @return number of inverse roles of the specified types Returns null, if
		 *     there is an exception.
		 */
		//public Integer getNumberOfSpecifiedInverseRoles(String Concept_in)
		public int getNumberOfSpecifiedInverseRoles(String Concept_in)
		{
		     Vector v = getFetchedInverseRoles(Concept_in);
		     if (v == null) return -1;
		     return v.size();
		}


		/**
		 * Checks if the concept has subconcepts.
		 *
		 * @since DLBExt 1.0
		 * @param name concept name
		 * @return If the concept has subconcepts, return true. Otherwise, return
		 *     false. Returns null, if there is an exception.
		 */
		private Boolean getFetchedHasSubs(String name)
		{
		    String code = nameToCode(codingScheme, version, tag, name);
		    if (code == null)
		    {
				return Boolean.FALSE;
			}

            //String associationName = IS_A;
			boolean direction = true;
			String is_a = getHasSubtype(codingScheme);

			if (is_a.compareTo(IS_A_OBO) == 0) direction = !direction;

            Boolean bVal = hasAssociatedConcepts(codingScheme,
                                         code,
										 direction,
										 is_a);

			return bVal;
		}


		/**
		 * Checks if the concept has superconcepts.
		 *
		 * @since DLBExt 1.0
		 * @param name concept name
		 * @return If the concept has superconcepts, return true. Otherwise, return
		 *     false. Returns null, if there is an exception.
		 */
		private Boolean getFetchedHasSups(String name)
		{
		    String code = nameToCode(codingScheme, version, tag, name);
		    if (code == null) return Boolean.FALSE;
            //String associationName = IS_A;
            //String associationName = getHasSubtype(codingScheme);
 			boolean direction = false;
			String is_a = getHasSubtype(codingScheme);
			if (is_a.compareTo(IS_A_OBO) == 0) direction = !direction;


            Boolean bVal = hasAssociatedConcepts(codingScheme,
                                         code,
										 direction,
										 is_a);

			if (bVal == null)
			{
				getLogger().warn("WARNING: ConvenienceMethods getFetchedHasSups returns null.");
			}

			return bVal;
		}



		private Boolean getFetchedHasSups(String value, boolean isCode)
		{
			String code = value;
			if (!isCode)
			{
		    	code = nameToCode(codingScheme, version, tag, value);
		    }
		    if (code == null) return Boolean.FALSE;
            //String associationName = getHasSubtype(codingScheme);
  			boolean direction = false;
			String is_a = getHasSubtype(codingScheme);
			if (is_a.compareTo(IS_A_OBO) == 0) direction = !direction;

            Boolean bVal = hasAssociatedConcepts(codingScheme,
                                         code,
										 direction,
										 is_a);

			if (bVal == null)
			{
				getLogger().warn("WARNING: ConvenienceMethods getFetchedHasSups returns null.");
			}

			return bVal;
		}


		/**
		 * Retrieves each concepts with a name (string identifier) that matches the argument pattern,
		 * based on the specified search options. Searches only concept names.
		 *
		 * @since DLBExt 1.0
		 * @param matchText search target
		 * @param maxToReturn maximum number of matched concepts to return
		 * @param synonyms true, if synonyms are returned; false, otherwise
		 * @return a vector of Concept objects with names that match the search
		 *     pattern. An empty vector is returned when there are no matches.
		 *     Returns null, if there is an exception.
		 */

		public Vector findConceptsWithNameMatching(String matchText,
						int maxToReturn,
						boolean synonyms)
		{
		     return findConceptsWithNameMatching(matchText,
						maxToReturn,
						synonyms,
						defaultAlgorithm);

		}

		public Vector findConceptsWithNameMatching(String matchText,
						int maxToReturn,
						String matchAlgorithm)
		{
		     return findConceptsWithNameMatching(matchText,
						maxToReturn,
						true,
						// defaultAlgorithm); //#2823
						matchAlgorithm);
		}


		public Vector findConceptsWithNameMatching(String matchText,
						int maxToReturn,
						boolean synonyms,
						String matchAlgorithm)
		{
			boolean preferredOnly = true;
			if (synonyms) preferredOnly = false;


            Vector sortByProperty_vec = new Vector();
            Vector source_vec = new Vector();
			Vector w = findCodedEntriesWithDesignationMatching(
												codingScheme,
												version,
												matchText,
												preferredOnly,
												sortByProperty_vec,
												source_vec,
												matchAlgorithm,
												maxToReturn);

            return w;
		}




//////////////////////////////////////////////////////////////////////////////////////////

    public CodedEntry[] findConceptWithSourceCodeMatching(String source, String code, int limit)
    {
        int matchOption = 9;
        int ASDIndex = 3;

        CodedEntry[] concepts = searchConcepts(code,
					 limit,
					 matchOption,
					 source,
				 	 ASDIndex);
        return concepts;
	}

	/**
	 * Retrieves each concept with a property value that matches the argument pattern,
	 * based on the specified search options. Searches only concept names.
	 *
	 * @since DLBExt 1.0
	 * @param propType_in
	 * @param pattern - the concept name pattern for searching
	 * @param searchLimit - the maximum numbe of matched concepts
	 * @return a vector of Concept objects with names that match the search
	 *     pattern. An empty vector is returned when there are no matches.
	 *     Returns null, if there is an exception.
	 */
//  used
	public Vector<CodedEntry> findConceptsWithPropertyMatching(String propType_in,
		String pattern,
		int searchLimit)
	{
        return findConceptsWithPropertyMatching(propType_in, pattern, searchLimit, defaultAlgorithm);
	}



	public Vector<CodedEntry> findConceptsWithPropertyMatching(String propType_in,
		String pattern,
		int searchLimit,
		String matchAlgorithm)

	{
		Vector<String> property_vec = new Vector<String>();
		property_vec.add(propType_in);
		Vector<String> restrictToProperties_vec = new Vector<String>();
		String displayPropertyName = "";
	    return findConceptsWithPropertyMatching(
	                                            pattern,
	                                            property_vec,
                                                restrictToProperties_vec,
                                                displayPropertyName,
                                                matchAlgorithm,
                                                searchLimit);

	}



	public Vector<CodedEntry> findConceptsWithPropertyMatching(String codingSchemeName, String propType_in,
		String pattern,
		int searchLimit,
		String matchAlgorithm)

	{
		Vector<String> property_vec = new Vector<String>();
		property_vec.add(propType_in);
		Vector<String> restrictToProperties_vec = new Vector<String>();
		String displayPropertyName = "";
	    return findConceptsWithPropertyMatching(codingSchemeName,
	                                            pattern,
	                                            property_vec,
                                                restrictToProperties_vec,
                                                displayPropertyName,
                                                matchAlgorithm,
                                                searchLimit);

	}



	/**
	 * Retrieves each concept with a property value that matches the argument pattern,
	 * based on the specified search options. Searches only concept names.
	 *
	 * @since DLBExt 1.0
	 * @param matchText search target
	 * @param property_vec
	 * @param restrictToProperties_vec
	 * @param displayPropertyName
	 * @param matchAlgorithm matching algorithm used
	 * @return a vector of Concept objects with names that match the search
	 *     pattern. An empty vector is returned when there are no matches.
	 *     Returns null, if there is an exception.
	 */


	public Vector<CodedEntry> findConceptsWithPropertyMatching(
	                                            String matchText,
	                                            Vector property_vec, // target property
                                                Vector restrictToProperties_vec,
                                                String displayPropertyName,
                                                String matchAlgorithm,
	                                            int maxToReturn)
	{
	    return findConceptsWithPropertyMatching(codingScheme,
	                                            matchText,
	                                            property_vec, // target property
                                                restrictToProperties_vec,
                                                displayPropertyName,
                                                matchAlgorithm,
	                                            maxToReturn);

	}


	public Vector<CodedEntry> findConceptsWithPropertyMatching(String codingSchemeName,
	                                            String matchText,
	                                            Vector property_vec, // target property
                                                Vector restrictToProperties_vec,
                                                String displayPropertyName,
                                                String matchAlgorithm,
	                                            int maxToReturn)
	{
		if (property_vec.size() == 0)
		{
			property_vec = getSupportedPropertyNames(codingSchemeName, version, null);
		}


		Vector<CodedEntry> w = findCodedEntriesWithPropertyMatching(
											codingSchemeName,
											version,
											matchText,
											property_vec,
											restrictToProperties_vec,
											matchAlgorithm,
											maxToReturn);
        return w;
	}

	/**
	 * Retrieves each concept with a designation value that matches the argument pattern,
	 * based on the specified search options. Searches only concept names.
	 *
	 * @since DLBExt 1.0
	 * @param matchText search target
	 * @param maxToReturn maximum number of matched concepts to return
	 * @param displayPropertyName the name of the display property
	 * @param includeSynonyms false, if match preferred name only; false,
	 *     otherwise
	 * @param matchAlgorithm matching algorithm used
	 * @return a vector of Concept objects with names that match the search
	 *     pattern. An empty vector is returned when there are no matches.
	 *     Returns null, if there is an exception.
	 */


	public Vector findConceptsWithDesignationMatching(String matchText,
												int maxToReturn,
												String displayPropertyName,
												boolean includeSynonyms,
												String matchAlgorithm)
	{
		boolean preferredOnly = true;
		if (includeSynonyms) preferredOnly = false;
		Vector sortByProperty_vec = new Vector();
		Vector restrictToProperties_vec = new Vector();


	    return findConceptsWithDesignationMatching(
	                                            matchText,
	                                            displayPropertyName,
												preferredOnly,
	                                            sortByProperty_vec,
                                                restrictToProperties_vec,
                                                matchAlgorithm,
	                                            maxToReturn);
    }


	/**
	 * Retrieves each concept with a designation value that matches the argument pattern,
	 * based on the specified search options. Searches only concept names.
	 *
	 * @since DLBExt 1.0
	 * @param matchText search target
	 * @param displayPropertyName the name of the display property
	 * @param preferredOnly
	 * @param sortByProperty_vec
	 * @param source_vec the set of sources to search within
	 * @param matchAlgorithm matching algorithm used
	 * @param maxToReturn maximum number of matched concepts to return
	 * @return a vector of Concept objects with names that match the search
	 *     pattern. An empty vector is returned when there are no matches.
	 *     Returns null, if there is an exception.
	 */

	public Vector findConceptsWithDesignationMatching(
	                                            String matchText,
	                                            String displayPropertyName,
												boolean preferredOnly,
	                                            Vector sortByProperty_vec,
                                                Vector source_vec,
                                                String matchAlgorithm,
	                                            int maxToReturn)
	{
		if (source_vec != null && source_vec.size() > 0)
		{
			Vector sources = getSupportedSources();
        	for (int i=0; i<source_vec.size(); i++)
        	{
				String source = (String) source_vec.elementAt(i);
				if (!sources.contains(source))
				{
					getLogger().error("Source " + source + " is not identifiable.");
					return null;
				}
			}
 	    }


    	Vector w = findCodedEntriesWithDesignationMatching(
			codingScheme, version, matchText, preferredOnly, sortByProperty_vec,
			source_vec, matchAlgorithm, maxToReturn);

		return w;
	}



//////////////////////////////////////////////////////////////////////////////////////////

	//public byte[] findConceptsWithAssociationMatching(
	/** "@since DLBExt 1.0
	 *
	 * @param associationName
	 * @param matchText
	 * @param maxToReturn
	 */
	//public CodedEntry[] findConceptsWithAssociationMatching(

	public Vector<CodedEntry> findConceptsWithAssociationMatching(
		                                           String associationName,
												   String matchText,
												   int maxToReturn)
	{
		return findConceptsWithAssociationMatching(associationName,
												   matchText,
												   maxToReturn,
												   defaultAlgorithm);

	}


	public Vector<CodedEntry> findConceptsWithAssociationMatching(
		                                           String associationName,
												   String matchText,
												   int maxToReturn,
												   String matchAlgorithm)
    {
		return findConceptsWithAssociationMatching(codingScheme,
		                                           associationName,
												   matchText,
												   maxToReturn,
												   matchAlgorithm,
												   null,
												   getRelation(codingScheme, associationName));
												   //null);
    }



	public Vector<CodedEntry> findConceptsWithAssociationMatching(
		                                           String associationName,
												   String matchText,
												   int maxToReturn,
												   String matchAlgorithm,
												   Vector restricted_property_vec)
    {
		return findConceptsWithAssociationMatching(codingScheme,
		                                           associationName,
												   matchText,
												   maxToReturn,
												   matchAlgorithm,
												   restricted_property_vec);


    }

    public String getRelation(String codingSchemeName, String associationName)
    {
		if (associationName == null) return null;//"relations";
		CodingScheme cs = getCodingScheme(codingSchemeName, null, null);
		Relations[] relations = cs.getRelations();
		for (int i=0; i<relations.length; i++)
		{
			Relations r = relations[i];
			org.LexGrid.relations.Association[] associations = r.getAssociation();
			for (int j=0; j<associations.length; j++)
			{
				org.LexGrid.relations.Association association = associations[j];
				String name = association.getAssociation();
				if (name.compareToIgnoreCase(associationName) == 0) return r.getDc();
				String forwardName = association.getForwardName();
				if (forwardName.compareToIgnoreCase(associationName) == 0) return r.getDc();
				String inverseName = association.getInverse();
				if (inverseName != null)
				{
					if (inverseName.compareToIgnoreCase(associationName) == 0) return r.getDc();
				}
		    }
		}
		return null;
	}


	public Vector<CodedEntry> findConceptsWithAssociationMatching(String codingSchemeName,
		                                           String associationName,
												   String matchText,
												   int maxToReturn,
												   String matchAlgorithm,
												   Vector restricted_property_vec)
    {
	     return findConceptsWithAssociationMatching(codingSchemeName,
		                                           associationName,
												   matchText,
												   maxToReturn,
												   matchAlgorithm,
												   restricted_property_vec,
												   getRelation(codingSchemeName, associationName));
												   //null);

    }


	public Vector<CodedEntry> findConceptsWithAssociationMatching(
		                                           String associationName,
												   String matchText,
												   int maxToReturn,
												   String matchAlgorithm,
												   Vector restricted_property_vec,
												   String relationName)
    {
	     return findConceptsWithAssociationMatching(codingScheme,
		                                           associationName,
												   matchText,
												   maxToReturn,
												   matchAlgorithm,
												   restricted_property_vec,
												   relationName);

    }



    // Use this method if the association is known to be contained in a partition called "roles", or "associations".
	public Vector<CodedEntry> findConceptsWithAssociationMatching(String codingSchemeName,
		                                           String associationName,
												   String matchText,
												   int maxToReturn,
												   String matchAlgorithm,
												   Vector restricted_property_vec,
												   String relationName)
	{
		Vector<CodedEntry> w = findCodedEntriesWithAssociationMatching(
			codingSchemeName, version, matchText, associationName, matchAlgorithm, maxToReturn, RESTRICT_TO_TARGET,
			restricted_property_vec, relationName);

		return w;
	}

//////////////////////////////////////////////////////////////////////////////////////////


		/**
		 * Retrieves each concept with a selected DTSPropertyType, if the concept DTSPropertyType
		 * value contains the argument pattern.
		 *
		 * @since DLBExt 1.0
		 * @param propertyName
		 * @param matchText
		 * @param searchLimit the maximum number of matches to be returned
		 * @return a vector of concept names with a property value that contains the
		 *      search pattern. An empty vector is returned when there are no
		 *     property values that contain the pattern. Returns null, if there is
		 *     an exception.
		 */
		public Vector<CodedEntry> findConceptsWithPropertyContaining(String propertyName,
						String matchText,
						int searchLimit)
		{
 			String matchAlgorithm = "contains";
 			return findConceptsWithPropertyMatching(
				                                propertyName,
	                                            matchText,
                                                searchLimit,
                                                matchAlgorithm);

		}


		/**
		 * Retrieves each concept with a selected DTSRoleType, if the concept DTSRoleType value
		 * matches the argument pattern.
		 *
		 * @since DLBExt 1.0
		 * @param roleType_in the input role type
		 * @param pattern - the concept role value pattern for searching
		 * @param searchLimit the maximum number of matches to be returned
		 * @return a vector of concept names with a role value that matches the
		 *     pattern. An empty vector is returned when there are no matches.
		 *     Returns null, if there is an exception.
		 */

/*
		public Vector findConceptWithRoleMatching(String roleType_in,
						String pattern,
						int searchLimit)
        {
		    return findConceptsWithRoleMatching(roleType_in,
						pattern,
						searchLimit);
		}
*/


		public Vector<CodedEntry> findConceptsWithRoleMatching(String roleType_in,
						String pattern,
						int searchLimit)
		{
		    return findConceptsWithRoleMatching(roleType_in,
						pattern,
						searchLimit,
						defaultAlgorithm);
		}



		public Vector<CodedEntry> findConceptsWithRoleMatching(String roleType_in,
						String pattern,
						int searchLimit,
						String matchAlgorithm)
		{
			return findConceptsWithAssociationMatching(
				                                   roleType_in,
												   pattern,
												   searchLimit,
												   matchAlgorithm);
		}


		/**
		 * Retrieves each concept which is a value of a DTSRole with a selected DTSRoleType, if the
		 * concept name having the role matches the argument pattern.
		 *
		 * @since DLBExt 1.0
		 * @param roleType_in the input role type
		 * @param pattern - the concept name pattern for searching
		 * @param searchLimit the maximum number of matches to be returned
		 * @return a vector of concept names which are the value of a role of the
		 *     specified DTSRoleType, and the concept name having the role matches
		 *     the search pattern. An empty vector is returned when there are no
		 *     matches. Returns null, if there is an exception.
		 */

/*
		public Vector findConceptsWithInverseConceptAssociationMatching(String roleType_in,
						String pattern,
						int searchLimit)
        {

		     return findConceptsWithInverseAssociationMatching(roleType_in,
						pattern,
						searchLimit);
		}
*/


		public Vector<CodedEntry> findConceptsWithInverseAssociationMatching(String roleType_in,
						String pattern,
						int searchLimit)
		{
		    return findConceptsWithInverseAssociationMatching(roleType_in,
						pattern,
						searchLimit,
						defaultAlgorithm);
		}

		public Vector<CodedEntry> findConceptsWithInverseAssociationMatching(
			            String roleType_in,
						String pattern,
						int searchLimit,
						String matchAlgorithm)
		{
			return findConceptsWithInverseAssociationMatching(roleType_in,
						pattern,
						searchLimit,
						matchAlgorithm,
						null);
		}


		public Vector<CodedEntry> findConceptsWithInverseAssociationMatching(
			            String roleType_in,
						String pattern,
						int searchLimit,
						String matchAlgorithm,
						Vector restricted_property_vec)
		{
		    return findConceptsWithInverseAssociationMatching(
				        codingScheme,
			            roleType_in,
						pattern,
						searchLimit,
						matchAlgorithm,
						restricted_property_vec);

		}


		public Vector<CodedEntry> findConceptsWithInverseAssociationMatching(
			            String codingSchemeName,
			            String roleType_in,
						String pattern,
						int searchLimit,
						String matchAlgorithm,
						Vector restricted_property_vec)
		{
		     return findConceptsWithInverseAssociationMatching(
			            codingSchemeName,
			            roleType_in,
						pattern,
						searchLimit,
						matchAlgorithm,
						restricted_property_vec,
						null);
		}



		public Vector<CodedEntry> findConceptsWithInverseAssociationMatching(
			            String roleType_in,
						String pattern,
						int searchLimit,
						String matchAlgorithm,
						Vector restricted_property_vec,
						String relationName)
		{
			return findConceptsWithInverseAssociationMatching(
							codingScheme,
							roleType_in,
							pattern,
							searchLimit,
							matchAlgorithm,
							restricted_property_vec,
							relationName);
		}


        // Use this method if the association is known to be contained in a partition called "roles", or "associations".
        // set relationName to "roles", or "associations" accordingly.

		public Vector<CodedEntry> findConceptsWithInverseAssociationMatching(
			            String codingSchemeName,
			            String roleType_in,
						String pattern,
						int searchLimit,
						String matchAlgorithm,
						Vector restricted_property_vec,
						String relationName)
		{
			Vector<CodedEntry> w = findCodedEntriesWithAssociationMatching(
				 codingSchemeName, version, pattern, roleType_in, matchAlgorithm, searchLimit,
				 RESTRICT_TO_SOURCE, restricted_property_vec, relationName);

			return w;
		}



		/**
		 * Gets all concepts with associations of the specified type whose target concept name is
		 * matched, based on the specified search options.
		 *
		 * @since DLBExt 1.0
		 * @param conceptAssocType_in the concept association type
		 * @param toConceptName - the to concept name
		 * @param searchLimit the maximum number of matches to be returned
		 * @return a vector of names of concepts with an association of the
		 *     specified type where the to concept matches the passed concept name
		 *     Returns null, if there is an exception.
		 */

		 /*
		public Vector findConceptsWithConceptAssociationMatching(String conceptAssocType_in,
						String toConceptName,
						int searchLimit)
		{
		     return findConceptsWithRoleMatching(conceptAssocType_in,
						toConceptName,
						searchLimit);
		}
		*/


/////////////////////////////////////////////////////
//// API for Supporting Term Browser Development
/////////////////////////////////////////////////////

	/** Gets all PropertyType in a given namespace
	 *
	 * @since DLBExt 1.0
	 * @return all PropertyType objects
	 */
//		used
	public PropertyType[] getAllPropertyTypes()
	{
		//byte[] byteNode= null;
		Vector w = getSupportedProperties(codingScheme, version, tag);
		if (w == null) return null;

        PropertyType[] a = new PropertyType[w.size()];
		//Vector v = new Vector();
		for (int i=0; i<w.size(); i++)
		{
			SupportedProperty sp = (SupportedProperty) w.elementAt(i);
			int j = i+1;

			PropertyType propertytype
			   = new PropertyType(
				        sp.getLocalId(),
						//sp.getContent(),
						j, // property id (integer valued) is not supported by LexBIG
						"");
			//v.add(propertytype);
			a[i] = propertytype;
		}
		return a;//objectToByteArray(v);
	}


	/** Gets all RoleType
	 *
	 * @since DLBExt 1.0
	 * @return all RoleType objects
	 */
	public RoleType[] getAllRoleTypes()
	{
		//101606
		Vector roles = getSupportedRoleNames();
	    try {
			Vector<RoleType> v = new Vector<RoleType>();
			Vector<SupportedAssociation> assos = getSupportedAssociations(codingScheme);
			for (int j=0; j<assos.size(); j++)
			{
				SupportedAssociation sa = (SupportedAssociation) assos.elementAt(j);
				String assoname = sa.getLocalId();

				if (roles.contains(assoname) && assoname.compareTo(IS_A) != 0
				    && assoname.compareTo(IS_A_OBO) != 0)
				{
					RoleType roletype = new RoleType(
						        sa.getLocalId(),
								//sa.getContent(),
								j+1,//-1,
								"");
					v.add(roletype);
			    }
			}
			if (v.size() == 0) return null;
			RoleType[] a = new RoleType[v.size()];

			for (int i=0; i<v.size(); i++)
			{
				a[i] = (RoleType) v.elementAt(i);
			}
			return a;

		} catch (Exception e) {
		    e.printStackTrace();
			getLogger().error("getAllRoleTypes failed.");
			return null;
		}
	}


	/** Gets concepts matching a given sets of concept names in a byte array
	 *
	 * @since DLBExt 1.0
	 * @param conceptNames_in concept names
	 * @param ASD_index attribute set descriptor index
	 * @return concepts matching a given sets of concept names in a byte array
	 */
	//public byte[] getConcepts(Vector conceptNames_in,
	public CodedEntry[] getConcepts(Vector<String> conceptNames_in, int ASD_index)
	{
        if (conceptNames_in == null) return null;
        if (conceptNames_in.size() == 0) return new CodedEntry[0];

//        Vector properties = null;
//        if (ASD_index == 0)
//        {
//			properties = new Vector();
//			properties.add(NONE);
//		}

		try {
			Vector<CodedEntry> v = new Vector<CodedEntry>();
			for (int i=0; i<conceptNames_in.size(); i++)
			{
				String conceptname = (String) conceptNames_in.elementAt(i);
				CodedEntry ce = getConceptByName(conceptname, ASD_index);
				v.add(ce);
			}

			return toArray(v);

		} catch ( Exception e ) {
			e.printStackTrace();
			return null;
		}
	}


	/** Gets RoleType by name
	 * .
	 *
	 * @since DLBExt 1.0
	 * @param rolename role name
	 * @return RoleType matches with a given role type name
	 */
	public RoleType getRoleTypeByName(String rolename)
	{
		Vector v = getSupportedRoleNames();
		for (int i=0; i<v.size(); i++)
		{
			String assoname = (String) v.elementAt(i);
			if (assoname.compareTo(rolename) == 0)
			{
				RoleType rt = new RoleType(
						 rolename,
						 -1,
						 "");

				return rt;//objectToByteArray(rt);
			}
		}
		return  null;
	}


	/** Gets PropertyType by name
	 *
	 * @since DLBExt 1.0
	 * @param propertyName property name
	 * @return PropertyType matches with a given property type name
	 */
	public PropertyType getPropertyTypeByName(String propertyName)
	{
		Vector v = getSupportedPropertyNames(codingScheme, version, null);
		for (int i=0; i<v.size(); i++)
		{
			String propname = (String) v.elementAt(i);
			if (propname.compareTo(propertyName) == 0)
			{
				 PropertyType pt = new PropertyType(
						 propertyName,
						 -1,
						 "");
				return pt;//objectToByteArray(pt);
			}
		}
		return  null;
	}


    /** Finds concept by name
     *
     * @since DLBExt 1.0
     * @param conceptname concept name
     * @return a Concept instance in a byte array
     */
//	used
    public CodedEntry findConcept(String conceptname)
    {
		return findConceptByName(conceptname, 1);
	}


    public CodedEntry findConcept(String conceptname, boolean allAttributes)
    {
		int ASD_index = 0;
		if (allAttributes) ASD_index = 1;
		return findConceptByName(conceptname, ASD_index);
	}


    public CodedEntry findConceptByName(String conceptname, int ASD_index)
	{
		String code = nameToCode(codingScheme, version, tag, conceptname);

		if (code == null)
		{
			getLogger().error("Unable to find code for name " + conceptname);
			return null;
		}
		return findConceptByCode(code, ASD_index);
	}


    /** "@since DLBExt 1.0
     *
     * @param conceptname
     * @param displayPropertyName
     */
    public CodedEntry findConceptByName(String conceptname, String displayPropertyName)
	{
		String code = nameToCode(codingScheme, version, tag, conceptname);
		if (code == null) return null;
		return findConceptByCode(code, displayPropertyName);
	}


    /** "@since DLBExt 1.0
     *
     * @param conceptname
     * @param ASD_index
     */
    private CodedEntry getConceptByName(String conceptname, int ASD_index)
	{
		String code = nameToCode(codingScheme, version, tag, conceptname);
		if (code == null) return null;
		CodedEntry ce = getCodedEntry(codingScheme, version, tag, code, ASD_index);
		return ce;
	}

//////////////////////////////////////////////////////////////////////


	 /** getFetched Concept Associations of an Concept instance
	  *
	  * @since DLBExt 1.0
	  * @param code
	  * @return a gov.nih.nci.lexrpc.client.Concept instance
	  */

	 public Vector getFetchedConceptAssociations(String code)
     {
          return getFetchedConceptAssociations(codingScheme, code);
     }



	 public Vector<ConceptAssociation> getFetchedConceptAssociations(
										  String codingSchemeName,
										  String code)
	 {
		//Vector associations = getSupportedAssociationNames(codingSchemeName);
		//if (associations.size() == 0) return new Vector();

		//String is_a = IS_A;
		String is_a = getHasSubtype(codingSchemeName);
		Vector<ConceptAssociation> v = new Vector<ConceptAssociation>();

		Vector<ConceptAssociation> w = getConceptAssociations(codingSchemeName,
									   version,
									   tag,
									   code,
									   true);

		if (w == null)
		{
			getLogger().warn("ConvenienceMethods getConceptAssociationNameAndValues returns null.");
			return null;
		}

        Vector<String> roles = getSupportedRoleNames(codingSchemeName);
		if (w == null) return null;
		for (int i=0; i<w.size(); i++)
		{
			gov.nih.nci.lexbig.ext.ConceptAssociation nv = (gov.nih.nci.lexbig.ext.ConceptAssociation) w.elementAt(i);
			String assoname = nv.getName();
			if (!roles.contains(assoname))
			{
				if (nv.getName().compareToIgnoreCase(is_a) != 0)
				{
					v.add(nv);
				}
			}
		}
		return v;
	 }


    /** "@since DLBExt 1.0
     *
     * @param propertynames
     */
    public Vector<String> getValidPropertyNames(Vector<String> propertynames)
	{
        Vector<String> v = new Vector<String>();
		if (propertynames == null) return null;
		if (propertynames.size() == 0) return v;
		Vector<String> propnames = getSupportedPropertyNames(codingScheme, version, null);
		for (int i=0; i<propertynames.size(); i++)
		{
			String s = (String) propertynames.elementAt(i);
			boolean valid = false;
			for (int j=0; j<propnames.size(); j++)
            {
				String t = (String) propnames.elementAt(j);
				if (s.compareTo(t) == 0)
				{
					valid = true;
					break;
				}
			}
			if (valid)
			{
				v.add(s);
			}
		}
		return v;
	}


    /** Searches concepts that match with a specific search criteria (complex search).
     *
     * @since DLBExt 1.0
     * @param inputvalue
     * @param inputcode
     * @param ASD_index AttributeSetDescriptor index
     * @return concepts that match with a specific search criteria in a byte
     *     array
     */

//  used
    public CodedEntry getConcept(String inputvalue,
                             boolean inputcode,
                             int ASD_index)
	{
		if (inputcode)
		{
			return findConceptByCode(inputvalue, ASD_index);
		}
		return findConceptByName(inputvalue, ASD_index);
	}



    /** getConcept
     *
     * @since DLBExt 1.0
     * @param inputvalue
     * @param inputcode
     * @param displayPropertyName
     */
    public CodedEntry getConcept(String inputvalue,
                             boolean inputcode,
                             String displayPropertyName)
	{
		if (inputcode)
		{
			return findConceptByCode(inputvalue, displayPropertyName);
		}
		return findConceptByName(inputvalue, displayPropertyName);
	}



     /** Gets concepts matching a given sets of concept names in a byte array
      *
      * @since DLBExt 1.0
      * @param conceptNames_in concept names
      * @return concepts matching a given sets of concept names in a byte
      *     array .1
      */



     public CodedEntry[] getConcepts(Vector<String> conceptNames_in)
     {
          return getConcepts(conceptNames_in, true);
	 }


     public CodedEntry[] getConcepts(Vector<String> conceptNames_in, boolean allAttributes)
     {
		  if (allAttributes)
		  {
          	return getConcepts(conceptNames_in,
                             false, // is name, not code
                             1); // all attributes
		  }
		  else
		  {
          	return getConcepts(conceptNames_in,
                             false, // is name, not code
                             0); // no attributes

		  }
	 }



     public CodedEntry[] getConcepts(Vector<String> inputvalues,
                              boolean inputcode,
                              int ASD_index)
	{
        if (!inputcode)
        {
			return getConcepts(inputvalues, ASD_index);
	    }

		Vector<CodedEntry> v = new Vector<CodedEntry>();
		for (int i=0; i<inputvalues.size(); i++)
		{
			String conceptcode = (String) inputvalues.elementAt(i);
			// to be modified (ASD_index);

			CodedEntry ce = getCodedEntryByCode(codingScheme, version, tag, conceptcode);
			v.add(ce);
		}
		return toArray(v);
	}


//////////////////////////////////////////////////////////////////////////////////////
// 051405

    /** Gets subconcepts of a given DescLogicConcept
     *
     * @since DLBExt 1.0
     * @param conceptname
     * @param allAttributes
     * @return concepts matching a given sets of concept names in a byte array
     *      .1
     */

    //public byte[] getDirectSubs(String inputvalue,
//   used
    public CodedEntry[] getDirectSubs(String conceptname, boolean allAttributes)
    {
		int ASD_index = 0;
		if (allAttributes) ASD_index = 1;
		return getDirectSubs(conceptname, false, ASD_index);

	}


    public CodedEntry[] getDirectSubs(String conceptname)
    {
		return getDirectSubs(conceptname, false, 1);
	}


    public CodedEntry[] getDirectSubs(String conceptname, int ASD_index)
    {
		return getDirectSubs(conceptname, false, ASD_index);
    }


    public CodedEntry[] getDirectSubs(String inputvalue,
                                boolean inputcode,
                                int ASD_index)
	{
		try {
			String code = inputvalue;
			if (!inputcode)
			{
				code = nameToCode(codingScheme, version, tag, inputvalue);
			}
			if (code == null) return null;

			//String relation = IS_A;
			String relation = getHasSubtype(codingScheme);

			boolean direction = true;
			// deal with ASD_Index later

			//Vector v = new Vector();
			Vector<String> codes = findRelatedCodes(codingScheme,
	                               version,
	                               tag,
	                               code,
	                               relation,
	                               direction);
	        if (codes == null)
	        {
			    getLogger().warn("findRelatedCodes returns null.");
			    return null;//objectToByteArray(v);
			}

            CodedEntry[] a = new CodedEntry[codes.size()];
			for (int i=0; i<codes.size(); i++)
			{
				String concept_code = (String) codes.elementAt(i);
		        CodedEntry c = getConcept(concept_code, ASD_index);
		        //v.add(c);
		        a[i] = c;
			}
			return a;//objectToByteArray(v);

		} catch ( Exception e ) {
			e.printStackTrace();
			return null;
		}

	}


    /** Gets direct superconcepts.
     *
     *
     * @since DLBExt 1.0
     * @param conceptname
     * @param allAttributes
     * @return concepts matching a given sets of concept names in a byte array
     *      .1
     */

    public CodedEntry[] getDirectSups(String conceptname, boolean allAttributes)
	{
		if (allAttributes)
		{
        	return getDirectSups(conceptname, false, 1);
		}
		else
		{
			return getDirectSups(conceptname, false, 0);
		}
	}


    public CodedEntry[] getDirectSups(String conceptname)
	{
        return getDirectSups(conceptname, false, 1);
	}


    public CodedEntry[] getDirectSups(String conceptname, int ASD_index)
	{
        return getDirectSups(conceptname, false, ASD_index);
	}


    public CodedEntry[] getDirectSups(String inputvalue,
                                boolean inputcode,
                                int ASD_index)
	{
		try {
			String code = inputvalue;
			if (!inputcode)
			{
				code = nameToCode(codingScheme, version, tag, inputvalue);
			}
			if (code == null) return null;

			//String relation = IS_A;
			String relation = getHasSubtype(codingScheme);

			boolean direction = false;
			// deal with ASD_Index later

            //Vector v = new Vector();
			Vector codes = findRelatedCodes(codingScheme,
	                               version,
	                               tag,
	                               code,
	                               relation,
	                               direction);

	        if (codes == null)
	        {
			    getLogger().warn("findRelatedCodes returns null.");
			    return null;//objectToByteArray(v);
			}

            CodedEntry[] a = new CodedEntry[codes.size()];
			for (int i=0; i<codes.size(); i++)
			{
				String concept_code = (String) codes.elementAt(i);
		        CodedEntry c = getConcept(concept_code, ASD_index);
		        //v.add(c);
		        a[i] = c;
			}
			return a;//objectToByteArray(v);

		} catch ( Exception e ) {
			e.printStackTrace();
			return null;
		}
	}


///////////////////////////////////////////////////////////////////////////////////////////////////

	/** sortVisitedNodes
	 *
	 * @param visited_node
	 */
	private CodedEntry[] sortVisitedNodes(Hashtable visited_node)
    {
		CodedEntry[] concepts = new CodedEntry[visited_node.size()];
		int lcv = 0;
		for (Enumeration e = visited_node.keys(); e.hasMoreElements();)
		{
			CodedEntry concept = (CodedEntry) e.nextElement();
			concepts[lcv] = concept;
			lcv++;
		}
		return concepts;//sortConcepts(concepts);
	}


	/** Checks if a concept has any subconcepts.
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme coding scheme name
	 * @param code coding scheme code
	 * @return Boolean variable, set to TRUE if it is; to FALSE, otherwise
	 */
	public Boolean hasChildren(String codingScheme, String code)
	{
		boolean direction = true;
		String is_a = getHasSubtype(codingScheme);
		if (is_a.compareTo(IS_A_OBO) == 0) direction = !direction;

        return hasAssociatedConcepts(codingScheme,
	                                     code,
	                                     direction,
										 is_a);
	}

	/** Checks if a concept has any superconcepts.
	 *
	 * @param codingScheme coding scheme name
	 * @param code coding scheme code
	 * @return Boolean variable, set to TRUE if it is; to FALSE, otherwise
	 */

	public Boolean hasParents(String codingScheme, String code)
	{
		boolean direction = false;
		String is_a = getHasSubtype(codingScheme);
		if (is_a.compareTo(IS_A_OBO) == 0) direction = !direction;

        return hasAssociatedConcepts(codingScheme,
	                                     code,
										 direction,
										 is_a);
	}


     Vector<PropertyQualifier> getPropertyQualifiers(org.LexGrid.commonTypes.Property p)
     {
		 Vector<PropertyQualifier> v = new Vector<PropertyQualifier>();
		 PropertyQualifier[] qualifiers = p.getPropertyQualifier();
		 for (int i=0; i<qualifiers.length; i++)
		 {
			 v.add(qualifiers[i]);
		 }
		 return v;
	 }


	 /** Finds the Concept instance for a given concept code.
	  *
	  * @since DLBExt 1.0
	  * @param conceptcode
	  * @param allAttributes
	  * @return the Concept instance for a given concept code
	  */

	 public CodedEntry findConceptByCode(String conceptcode, boolean allAttributes)
	 {
		 int ASD_index = 0;
		 if (allAttributes) ASD_index = 1;
		 return findConceptByCode(conceptcode, ASD_index);
	 }

//used
	 public CodedEntry findConceptByCode(String code)
	 {
          return findConceptByCode(code, 1);
     }

	 /** findConceptByCode
	  *
	  * @param code concept code
	  * @param ASD_index
	  */
	 public CodedEntry findConceptByCode(String code, int ASD_index)
	 {
		 return getConcept(code, ASD_index);
	 }


	 /** findConceptByCode
	  *
	  * @param code concept code
	  * @param displayPropertyName
	  */
	 public CodedEntry findConceptByCode(String code, String displayPropertyName)
	 {
		 return getConcept(code, displayPropertyName);
	 }


	 /** getConcept
	  *
	  * @param codingSchemeName
	  * @param code concept code
	  * @param isCode
	  * @param ASD_index
	  */

     // 041207
	 public CodedEntry getConcept(String codingSchemeName, String code, boolean isCode, int ASD_index)
	 {
		 CodedEntry ce = getCodedEntry(codingSchemeName, version, tag, code, ASD_index);
		 if (ce == null)
		 {
			 getLogger().warn("Cannot find CodedEntry for code " + code);
			 return null;
		 }
	     return ce;
	 }



	 /** getConcept
	  *
	  * @since DLBExt 1.0
	  * @param code concept code
	  * @param displayPropertyName
	  */
	 private CodedEntry getConcept(String code, String displayPropertyName)
	 {

         CodedEntry ce = getCodedEntry(codingScheme,
	                                version,
									tag,
									code,
									3);

		 if (ce == null)
		 {
			 getLogger().warn("Cannot find CodedEntry for code " + code);
			 return null;
		 }
	     return ce;
	 }




	 /** getSupportedConceptStatus
	  */
	 public Vector getSupportedConceptStatus()
	 {
		 return getSupportedConceptStatus(codingScheme);
	 }


	/** getTreeNodes
	 *
	 * @since DLBExt 1.0
	 * @param root_code concept code
	 * @param isaflag
	 * @param rolenames
	 * @param direction
	 */
	private HashMap<String,TreeNode> getTreeNodes(
								 String root_code,
								 boolean isaflag,
								 Vector rolenames,
								 boolean direction)
	{
	//  code to vector of role names mapping
		HashMap<String,TreeNode> code2RolenamesMap = new HashMap<String,TreeNode>();
        Vector<String> links = new Vector<String>();
        for (int i=0; i<rolenames.size(); i++)
        {
			String rolename = (String) rolenames.elementAt(i);
			links.add(rolename);
		}
		if (links.size() == 0 && !isaflag)
		{
			getLogger().error("links.size() == 0 && !isaflag" + " nothing done");
			return code2RolenamesMap;
		}

        Vector<NameAndValue> w = new Vector<NameAndValue>();

        if (isaflag)
        {
			//String associationName = IS_A;
			String associationName = getHasSubtype(codingScheme);
			Vector concepts = getRelatedConcepts(root_code, direction, associationName, 0);
			for (int i=0; i<concepts.size(); i++)
			{
				CodedEntry c = (CodedEntry) concepts.elementAt(i);
				NameAndValue nv = createNameAndValue(associationName, c.getConceptCode());
				w.add(nv);
			}
		}

		if (rolenames != null)
		{
			for (int i=0; i<rolenames.size(); i++)
			{
				String associationName = (String) rolenames.elementAt(i);
				Vector concepts = getRelatedConcepts(root_code, direction, associationName, 0);
				for (int j=0; j<concepts.size(); j++)
				{
					CodedEntry c = (CodedEntry) concepts.elementAt(j);
					NameAndValue nv = createNameAndValue(associationName, c.getConceptCode());
					w.add(nv);
				}
			}
		}

		if (w.size() == 0) // an orphan, no needs to continue
		{
			return code2RolenamesMap;
		}

		for (int i=0; i<w.size(); i++)
		{
			NameAndValue nv = (NameAndValue) w.elementAt(i);
			String rolename = nv.getName();

			if (links.contains(rolename) || (isaflag && rolename.equals(IS_A)) )
			{
				String ref_code = nv.getContent();
				if (code2RolenamesMap.containsKey(ref_code))
				{
					TreeNode node = (TreeNode) code2RolenamesMap.get(ref_code);
					node.setIsA(isaflag);
					node.setTraverseDown(direction);
					if (!node.getLinks().contains(ref_code))
					{
						node.addLink(rolename);
					}
			    }
				else
				{
					TreeNode node
					   = new TreeNode();
						node.setIsA(isaflag);
						node.setTraverseDown(direction);
					    node.addLink(rolename);
					code2RolenamesMap.put(ref_code, node);
				}
			}
		}
		return code2RolenamesMap;
	}



	private HashMap<String,TreeNode> getTreeNodes(
								 String root_code,
								 String isaName,
								 Vector rolenames,
								 boolean direction)
	{

	//  code to vector of role names mapping
		HashMap<String,TreeNode> code2RolenamesMap = new HashMap<String,TreeNode>();
        Vector<String> links = new Vector<String>();
        for (int i=0; i<rolenames.size(); i++)
        {
			String rolename = (String) rolenames.elementAt(i);
			links.add(rolename);
		}
		if (links.size() == 0 && isaName == null)
		{
			getLogger().error("links.size() == 0 && isaName == null" + " nothing done");
			return code2RolenamesMap;
		}

        Vector<NameAndValue> w = new Vector<NameAndValue>();

        if (isaName != null)
        {
			String associationName = isaName;
			Vector concepts = getRelatedConcepts(root_code, direction, associationName, 0);
			for (int i=0; i<concepts.size(); i++)
			{
				CodedEntry c = (CodedEntry) concepts.elementAt(i);
				NameAndValue nv = createNameAndValue(associationName, c.getConceptCode());
				w.add(nv);
			}
		}

		if (rolenames != null)
		{
			for (int i=0; i<rolenames.size(); i++)
			{
				String associationName = (String) rolenames.elementAt(i);
				Vector<CodedEntry> concepts = getRelatedConcepts(root_code, direction, associationName, 0);
				for (int j=0; j<concepts.size(); j++)
				{
					CodedEntry c = (CodedEntry) concepts.elementAt(j);
					NameAndValue nv = createNameAndValue(associationName, c.getConceptCode());
					w.add(nv);
				}
			}
		}

		if (w.size() == 0) // an orphan, no needs to continue
		{
			return code2RolenamesMap;
		}

		for (int i=0; i<w.size(); i++)
		{
			NameAndValue nv = (NameAndValue) w.elementAt(i);
			String rolename = nv.getName();

			if (links.contains(rolename) || (isaName != null && rolename.equals(isaName)) )
			{
				String ref_code = nv.getContent();
				if (code2RolenamesMap.containsKey(ref_code))
				{
					TreeNode node
					   = (TreeNode) code2RolenamesMap.get(ref_code);
					if (isaName != null)
					{
						node.setIsA(true);
				    }
				    else
				    {
						node.setIsA(false);
					}
					node.setTraverseDown(direction);
					if (!node.getLinks().contains(ref_code))
					{
						node.addLink(rolename);
					}
			    }
				else
				{
					TreeNode node = new TreeNode();
						if (isaName != null)
						{
							node.setIsA(true);
						}
						else
						{
							node.setIsA(false);
						}

						node.setTraverseDown(direction);
					    node.addLink(rolename);
					code2RolenamesMap.put(ref_code, node);
				}
			}
		}
		return code2RolenamesMap;
	}


	/** Returns a root byte array based on the concept provided and limits the number of levels of child concepts returned.
	 * The root byte array contains the child concepts to the level supplied.
	 *
	 * @since DLBExt 1.0
	 * @param rootname - the concept name to use as the root of the tree.
	 * @param direction - Indicates the direction of the tree to create,
	 *     either to the root or to the leaf from the concept provided.
	 * @param asdIndex
	 * @return the root node byte array of a tree containing a collection of
	 *     children
	 * @since dlbExt 1.0
	 */

	public DefaultMutableTreeNode getTree(String rootname,
					boolean direction,
					int asdIndex)
	{

		return getTree(rootname,
					direction,
					-1,
					asdIndex,
					true,
					new Vector());
	}


	public DefaultMutableTreeNode getTree(String rootname,
					boolean direction,
					int level,
					int asdIndex)
	{

		return getTree(rootname,
					direction,
					level,
					asdIndex,
					true,
					new Vector());
	}

//	used
	public DefaultMutableTreeNode getTree(String rootname,
	                      boolean direction,
	                      int levels,
	                      int asdIndex,
		                  boolean isaflag,
		                  Vector rolenames)
	{
		if (isaflag)
		{
			//boolean direction = true;
			String is_a = getHasSubtype(codingScheme);
			if (is_a.compareTo(IS_A_OBO) == 0)
			{
				direction = !direction;
				return getTree(rootname, direction, levels, asdIndex, IS_A_OBO, rolenames);
			}
		}

		String code = nameToCode(codingScheme, version, tag, rootname);
		if (code == null) return null;
		int ASD_index = 0;
		CodedEntry c = getConcept(code, ASD_index);

		if (c == null)
		{
			getLogger().error("Root concept " + rootname + " does not exist.");
			return null;
		}

		TreeNode treenode = new TreeNode();
		treenode.setTraverseDown(direction);
		treenode.setIsA(isaflag);
		//c.setTreeNode(treenode);
		treenode.setConcept(c);

		DefaultMutableTreeNode dmtn = new DefaultMutableTreeNode(treenode);
		dmtn = populateTreeNodes(dmtn, isaflag, rolenames, direction);

		return dmtn;
	}


//	used
    // New getTree method allowing the user to enter is-a association name
	public DefaultMutableTreeNode getTree(String rootname,
	                      boolean direction,
	                      int levels,
	                      int ASD_index,
		                  String isaName,
		                  Vector rolenames)
	{
		String code = nameToCode(codingScheme, version, tag, rootname);
		if (code == null)
		{
			getLogger().error("getTree Code for concept " + rootname + " not found.");
			return null;
		}

		CodedEntry c = getConcept(code, ASD_index);

		if (c == null)
		{
			getLogger().error("Root concept " + rootname + " does not exist.");
			return null;
		}

		TreeNode treenode = new TreeNode();
		treenode.setTraverseDown(direction);

		if (isaName != null)
		{
			treenode.setIsA(true);
	    }
	    else
	    {
			treenode.setIsA(false);
		}
		treenode.setConcept(c);

		DefaultMutableTreeNode dmtn = new DefaultMutableTreeNode(treenode);
		dmtn = populateTreeNodes(dmtn, isaName, rolenames, direction);

		return dmtn;//objectToByteArray(dmtn);
	}




	/**
	 *
	 * @since DLBExt 1.0
	 * @param dmtn
	 * @param isaflag
	 * @param rolenames
	 * @param direction
	 */


	private DefaultMutableTreeNode populateTreeNodes(DefaultMutableTreeNode dmtn,
	    	    boolean isaflag, Vector rolenames, boolean direction)
	{

        TreeNode node = (TreeNode) dmtn.getUserObject();
		CodedEntry c = node.getConcept();
		String code = c.getConceptCode();
	    HashMap hashMap = getTreeNodes(
									code,
									isaflag,
									rolenames,
									direction);

        Set ref_codes = hashMap.keySet();

        Iterator it = ref_codes.iterator();

        int num_childnodes = 0;
        while (it.hasNext())
        {
			String subconcept_code = (String) it.next();
			TreeNode treenode = (TreeNode) hashMap.get(subconcept_code);

			CodedEntry child_concept = new CodedEntry();
			child_concept.setConceptCode(subconcept_code);
            treenode.setConcept(child_concept);

            treenode.setName(getName(getConcept(subconcept_code, 0)));

			DefaultMutableTreeNode child_dmtd = new DefaultMutableTreeNode(treenode);
			child_dmtd = populateTreeNodes(child_dmtd, isaflag, rolenames, direction);
			dmtn.add(child_dmtd);
			num_childnodes++;
		}
		return dmtn;
	}



	private DefaultMutableTreeNode populateTreeNodes(DefaultMutableTreeNode dmtn,
	    	    String isaName, Vector rolenames, boolean direction)
	{
        TreeNode node = (TreeNode) dmtn.getUserObject();
		CodedEntry c = node.getConcept();
		String code = c.getConceptCode();
	    HashMap hashMap = getTreeNodes(
									code,
									isaName,
									rolenames,
									direction);

        Set ref_codes = hashMap.keySet();
        Iterator it = ref_codes.iterator();

        int num_childnodes = 0;
        while (it.hasNext())
        {
			String subconcept_code = (String) it.next();

//			TreeNode treenode = (TreeNode) hashMap.get(subconcept_code);
			String subconcept_name = codeToName(codingScheme, version, subconcept_code);

			CodedEntry child_concept = new CodedEntry();
			//child_concept.setName(subconcept_name);
			child_concept.setConceptCode(subconcept_code);
			//child_concept.setTreeNode(treenode);

			TreeNode childnode = new TreeNode();
			childnode.setConcept(child_concept);
			childnode.setName(subconcept_name);

			DefaultMutableTreeNode child_dmtd = new DefaultMutableTreeNode(childnode);
			child_dmtd = populateTreeNodes(child_dmtd, isaName, rolenames, direction);
			dmtn.add(child_dmtd);
			num_childnodes++;
		}
		return dmtn;
	}



    /**
     *
     * @since DLBExt 1.0
     * @param code concept code
     */
    public Boolean codeExists(String code)
    {
	    return codeExists(codingScheme, code);
    }


    /**
     * Checks if two codes are related through a particular association.
     *
     * @since DLBExt 1.0
     * @param code1 concept code
     * @param code2 concept code
     * @param associationName association name
     * @return Boolean variable, sets to TRUE if related; to FALSE, otherwise
     */
    public Boolean areCodesRelated(
								 String code1, // source
								 String code2, // target
								 String associationName)
	{
		/*
    	return areCodesRelated(codingScheme,
								 version,
								 null,
								 code1, // source
								 code2, // target
								 associationName);
		*/
		Vector v = getRelatedConcepts(code1, associationName);
		if (v == null) return Boolean.FALSE;
		for (int i=0; i<v.size(); i++)
		{
			CodedEntry ce = (CodedEntry) v.elementAt(i);
			if (ce.getConceptCode().compareTo(code2) == 0) return Boolean.TRUE;
		}
		return Boolean.FALSE;
    }


///////////////////////
//  History
///////////////////////

     /**
      * Dumps dumpNCIChangeEvent (for debug use only).
      *
      * @since DLBExt 1.0
      * @param pw
      * @param e
      */
     private void dumpNCIChangeEvent(PrintWriter pw, NCIChangeEvent e)
     {
		 String code = e.getConceptcode();
		 String name = e.getConceptName();
		 int actionId = e.getEditaction().getType();
		 String action = "";
		 if (actionId == ChangeType.CREATE_TYPE)
		 {
			 action = "Create";
		 }
		 else if (actionId == ChangeType.MERGE_TYPE)
		 {
			 action = "Merge";
		 }
		 else if (actionId == ChangeType.MODIFY_TYPE)
		 {
			 action = "Modify";
		 }
		 else if (actionId == ChangeType.RETIRE_TYPE)
		 {
			 action = "Retire";
		 }
		 else if (actionId == ChangeType.SPLIT_TYPE)
		 {
			 action = "Split";
		 }

		 java.util.Date d = e.getEditDate();
		 String referenceCode = e.getReferencecode();
		 String referenceName = e.getReferencename();
		 pw.println("\n\tCode: " + code);
		 pw.println("\tName: " + name);
		 pw.println("\tAction: " + action);
		 pw.println("\tEdit Date: " + d.toString());
		 pw.println("\tReference Code: " + referenceCode);
		 pw.println("\tReference Name: " + referenceName);
 	 }


    /**
     * Converts a NCICHangeEvent instance to a HistoryRecord instance.
     *
     * @since DLBExt 1.0
     * @param namespaceId namespace id
     * @param e a NCICHangeEvent instance
     * @return a HistoryRecord instance
     */
    private HistoryRecord NCIChangeEvent2HistoryRecord(
		int namespaceId, NCIChangeEvent e)
    {
		if (e == null) return null;
		HistoryRecord rec = new HistoryRecord();
		rec.setCode(e.getConceptcode());
		rec.setNamespaceId(namespaceId);
//		int editaction = e.getEditaction().getType();
		String action = "";
		int actionId = e.getEditaction().getType();
		if (actionId == ChangeType.CREATE_TYPE)
		{
		    action = "Create";
		}
		else if (actionId == ChangeType.MERGE_TYPE)
		{
		    action = "Merge";
		}
		else if (actionId == ChangeType.MODIFY_TYPE)
		{
		    action = "Modify";
		}
		else if (actionId == ChangeType.RETIRE_TYPE)
		{
		    action = "Retire";
		}
		else if (actionId == ChangeType.SPLIT_TYPE)
		{
		    action = "Split";
		}
		rec.setEditAction(action);
		rec.setEditDate(e.getEditDate());
		rec.setReference(e.getReferencecode());
		return rec;
	}


//  used
    //public byte[] getConceptHistory(
	/**
	 * Gets HistoryRecord corresponding to a given concept code.
	 *
	 * @since DLBExt 1.0
	 * @param code concept code
	 * @param beginDate begin date
	 * @param endDate end date
	 */
	public HistoryRecord[] getConceptHistory(
                                    String code,
                                    java.util.Date beginDate,
                                    java.util.Date endDate)
    {
        Vector<NCIChangeEvent> w = getConceptHistory(codingScheme,
                                    code,
                                    beginDate,
                                    endDate);

        Vector<HistoryRecord> v = new Vector<HistoryRecord>();

		for (int i=0; i<w.size(); i++)
		{
			NCIChangeEvent nce = (NCIChangeEvent) w.elementAt(i);
			if (nce != null)
			{
				HistoryRecord rec = NCIChangeEvent2HistoryRecord(namespaceId, nce);
				v.add(rec);
		    }
		}
		HistoryRecord[] a = new HistoryRecord[v.size()];
		for (int i=0; i<v.size(); i++)
		{
			a[i] = (HistoryRecord) v.elementAt(i);
		}
		return a;
    }


    /**
     * Gets concept change version Dates.
     *
     * @since DLBExt 1.0
     * @param code concept code
     * @return a vector containing all concept change version Dates
     */
    private Vector<Date> getConceptChangeVersionDates(String code)
	{
		Vector<Date> v = new Vector<Date>();
        Vector csvs = getConceptChangeVersions(codingScheme, code);
		for (int i=0;i<csvs.size(); i++)
		{
			CodingSchemeVersion csv = (CodingSchemeVersion) csvs.elementAt(i);
			java.util.Date d = csv.getVersionDate();
            v.add(d);
		}
		return v;
    }


    /**
     * Gets concept create version Dates.
     *
     * @since DLBExt 1.0
     * @param code concept code
     * @return a vector containing all concept create version Dates
     */
    public java.util.Date getConceptCreateVersionDate(String code)
    {
		CodingSchemeVersion csv = getConceptCreateVersion(codingScheme, code);
		if (csv == null) return null;
		return csv.getVersionDate();

    }

/////////////////////////////////////////////////////////////////////////////////////////
	/**
	 * getHistoryStart
	 *
	 * @since DLBExt 1.0
	 */

	public java.util.Date getHistoryStart()
    {
		return getHistoryStartBaselineDate(codingScheme);
    }

    /**
     * getHistoryStartDate
     *
     * @since DLBExt 1.0
     */
    public java.util.Date getHistoryStartDate()
    {
		return getHistoryStartBaselineDate(codingScheme);
    }

    /**
     * getHistoryStartBaselineDate
     *
     * @since DLBExt 1.0
     */
    public java.util.Date getHistoryStartBaselineDate()
    {
		return getHistoryStartBaselineDate(codingScheme);
    }

    /**
     * getHistoryEnd
     *
     * @since DLBExt 1.0
     */

    public java.util.Date getHistoryEnd()
    {
		return getHistoryEndBaselineDate(codingScheme);
    }

    /**
     * getHistoryEndDate
     *
     * @since DLBExt 1.0
     */
    public java.util.Date getHistoryEndDate()
    {
		return getHistoryEndBaselineDate(codingScheme);
    }

    /**
     *
     * @since DLBExt 1.0
     */

    public java.util.Date getHistoryEndBaselineDate()
    {
		return getHistoryEndBaselineDate(codingScheme);
    }




/////////////////////////////////////////////////////////////////////////////////////////


	/**
	 *
	 * @since DLBExt 1.0
	 * @param vocabularyname
	 */
	public Vector getVocabularyVersions(String vocabularyname)
	{
        return getAvailableCodingSchemeVersions(vocabularyname);
	}

	/**
	 * Gets available coding scheme tags for a given coding scheme name and version.
	 *
	 * @since DLBExt 1.0
	 * @param vocabularyname coding scheme name
	 * @param version coding scheme version
	 */
	public Vector getVocabularyTags(String vocabularyname, String version)
	{
        return getAvailableCodingSchemeTags(vocabularyname, version);
	}

	/** Gets available coding scheme names.
	 *
	 * @since DLBExt 1.0
	 * @return a Vector containing all available coding scheme names.
	 */
	private Vector getAvailableCodingSchemes()
	{
		 return getSupportedCodingSchemes();
	}
//	used
	/** Gets available vocabulary (i.e., coding scheme) names.
	 *
	 * @since DLBExt 1.0
	 * @return a Vector containing all available vocabulary (i.e., coding scheme)
	 *      names.
	 */
	//080307
	public Vector<String> getVocabularyNames()
	{
        return getSupportedCodingSchemeNames();//getAvailableCodingSchemes();
	}

	/** Checks if a coding scheme exists.
	 *
	 * @since DLBExt 1.0
	 * @param codingSchemeName
	 * @return true, if the coding scheme exists; false, otherwise
	 */
	//080307
	public boolean codingSchemeExists(String codingSchemeName)
	{
		//Vector v = getSupportedCodingSchemes();
		Vector v = getSupportedCodingSchemeNames();
		return (v.contains(codingSchemeName));
	}


     /** dump history records (for debugging use)
      *
      * @since DLBExt 1.0
      * @param pw an instance of PrintWriter
      * @param e an instance of HistoryRecord
      */
     public void dumpHistoryRecord(PrintWriter pw, HistoryRecord e)
     {
		 String code = e.getCode();
		 int namespaceId = e.getNamespaceId();
		 String editaction = e.getEditAction();
		 java.util.Date editdate = e.getEditDate();
		 String reference = e.getReference();
         pw.println("\n");
		 pw.println("\tCode: " + code);
		 pw.println("\tNamespaceId: " + namespaceId);
		 pw.println("\tAction: " + editaction);
		 pw.println("\tEdit Date: " + editdate.toString());
		 pw.println("\tReference Code: " + reference);
 	 }

 	 //////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////

	/** Print a tree (UserObject is a Concept instance).
	 *
	 * @since DLBExt 1.0
	 * @param treeWriter an instance of PrintWriter
	 * @param root The root (DefaultMutableTreeNode) of the tree
	 */

	public void printTree(PrintWriter treeWriter, DefaultMutableTreeNode root)
	{
		if (treeWriter == null)
		{
		    getLogger().warn("PrintWriter is null.");
			return;
		}

		if (root == null)
		{

		    getLogger().warn("printTree -- root is null.");
			return;
		}

        CodedEntry con = null;
		try {
			TreeNode treenode = (TreeNode) root.getUserObject();
			con = (CodedEntry) treenode.getConcept();
	    } catch (Exception e) {
			e.printStackTrace();
			getLogger().warn("ERROR: printTree con == null " );
			return;
		}

        treeWriter.println(getName(con) + " (" + con.getConceptCode() + ")");
		treeWriter.flush();
        printTree(treeWriter, root, 1);
	}

	/** Print a tree (UserObject is a Concept instance).
	 *
	 * @since DLBExt 1.0
	 * @param treeWriter an instance of PrintWriter
	 * @param root the root (DefaultMutableTreeNode) of the tree
	 * @param level tree level
	 */

	private void printTree(PrintWriter treeWriter,
	                       DefaultMutableTreeNode root,
						   int level)
	{
		int length = root.getChildCount();
		if (length ==0) return;

		for (int i=0; i<length; i++) {
			DefaultMutableTreeNode node = (DefaultMutableTreeNode)root.getChildAt(i);
			printNode(treeWriter, node, level);
			treeWriter.flush();
			printTree(treeWriter, node, level+1);
		}
	}

	/** Print a tree node (UserObject is a Concept instance).
	 *
	 * @since DLBExt 1.0
	 * @param treeWriter an instance of PrintWriter
	 * @param node tree node
	 * @param level tree level
	 * @param root the root (DefaultMutableTreeNode) of the tree
	 */

	private void printNode(PrintWriter treeWriter,
	                       DefaultMutableTreeNode node,
						   int level)
	{
		TreeNode treenode = (TreeNode) node.getUserObject();
		CodedEntry concept = (CodedEntry) treenode.getConcept();
		String conceptName = treenode.getName();
		boolean traverseDown = treenode.getTraverseDown();
		boolean isA = treenode.getIsA();
		HashSet links = treenode.getLinks();

		if (isA && (links.contains(IS_A)))
		{
   		    for (int j=0; j<level; j++)
			{
				treeWriter.print("\t");
				treeWriter.flush();
			}

			if (traverseDown)
				treeWriter.println("[inverse is-a] "+conceptName + " (" + concept.getConceptCode() + ")");
			else
				treeWriter.println("[is-a] "+conceptName + " (" + concept.getConceptCode() + ")");

            treeWriter.flush();
		}
		else if (isA && (links.contains(IS_A_OBO)))
		{
   		    for (int j=0; j<level; j++)
			{
				treeWriter.print("\t");
				treeWriter.flush();
			}

			if (traverseDown)
				treeWriter.println("[is-a] "+conceptName + " (" + concept.getConceptCode() + ")");
			else
				treeWriter.println("[inverse is-a] "+conceptName + " (" + concept.getConceptCode() + ")");

            treeWriter.flush();
		}

		Iterator it = links.iterator();
		while (it.hasNext())
		{
			String linkname = (String) it.next();
			if (linkname.compareTo(IS_A) != 0 && linkname.compareTo(IS_A_OBO) != 0)
            {
				for (int j=0; j<level; j++)
				{
					treeWriter.print("\t");
					treeWriter.flush();
				}

				if (!traverseDown)
					treeWriter.println("[inverse "+linkname+"] "+conceptName + " (" + concept.getConceptCode() + ")");
				else
					treeWriter.println("["+linkname+"] "+conceptName + " (" + concept.getConceptCode() + ")");

				treeWriter.flush();
		    }
		}
	}


	/** Ouptut a tree
	 *
	 * @since DLBExt 1.0
	 * @param pw an instance of PrintWriter
	 * @param dmtn an instance of DefaultMutableTreeNode
	 */
	public void outputTree(PrintWriter pw, DefaultMutableTreeNode dmtn)
	{
         outputTree(pw, dmtn, 0);
    }


	/** Ouptut a tree
	 *
	 * @since DLBExt 1.0
	 * @param pw an instance of PrintWriter
	 * @param dmtn an instance of DefaultMutableTreeNode
	 * @param level level
	 */
	public void outputTree(PrintWriter pw, DefaultMutableTreeNode dmtn, int level)
	{
		if (dmtn == null) return;
		if (pw == null) return;

		TreeNode node = (TreeNode) dmtn.getUserObject();
		CodedEntry c = node.getConcept();

        StringBuffer sb = new StringBuffer();
			for (int i = 0; i < level; i++)
				sb.append('\t');
			String prefix = sb.toString();

		pw.println(prefix +
			c.getConceptCode() +
				"/" + node.getName());

        pw.flush();
		Enumeration enumeration = dmtn.children();
		while (enumeration.hasMoreElements())
		{
			DefaultMutableTreeNode child_node = (DefaultMutableTreeNode) enumeration.nextElement();
			outputTree(pw, child_node, level+1);
		}
	}


	public void outputProperties(PrintWriter pw, CodedEntry ce)
	{
        if (ce == null)
        {
 			getLogger().error("wrapper.getProperties invalid input -- ce is null.");
 			return;
		}

		Definition[] definitions = ce.getDefinition();
		for (int i=0; i<definitions.length; i++)
		{
		    Definition d = definitions[i];
		    pw.println(d.getProperty() + ": " + d.getText().getContent());
		}

		Comment[] comments = ce.getComment();
		for (int i=0; i<comments.length; i++)
		{
		    Comment c = comments[i];
		    pw.println(c.getProperty() + ": " + c.getText().getContent());
		}

		Instruction[] instructions = ce.getInstruction();
		for (int i=0; i<instructions.length; i++)
		{
		    Instruction inst = instructions[i];
		    pw.println(inst.getProperty() + ": " + inst.getText().getContent());

		}

		Presentation[] presentations = ce.getPresentation();
		if (presentations != null)
		{
			for (int i=0; i<presentations.length; i++)
			{
				Presentation p = presentations[i];
				Vector source_vec = getPropertySources(p);
				if (source_vec.size() == 0)
				{
//					java.lang.String prop_name = p.getProperty();
//					java.lang.String prop_value = p.getText().getContent();

				    pw.println(p.getProperty() + ": " + p.getText().getContent());
				}
			}
		}


 	    org.LexGrid.commonTypes.Property[] properties = ce.getConceptProperty();
		for (int i=0; i<properties.length; i++)
		{
			org.LexGrid.commonTypes.Property p = properties[i];
			if (p != null)
			{
				pw.println(p.getProperty() + ": " + p.getText().getContent());
		    }
		}
	}


	/** Gets root concepts
	 *
	 * @since DLBExt 1.0
	 * @return a Vector containing root concept instances
	 */
	public CodedEntry[] getRootConcepts()
	{
		boolean allAttributes = true;
		return getRootConcepts(allAttributes);
	}


//	used
	public CodedEntry[] getRootConcepts(boolean allAttributes)
	{
	    if (allAttributes)
		{
			return getRootConcepts(1);
		}
		return getRootConcepts(0);
	}



    public CodedEntry[] getRootConcepts(int ASD_index)
    {
		return getConceptChildRoots(ASD_index);
	}



	private Vector<CodedEntry> toVector(CodedEntry[] a)
	{
		if (a == null) return null;
		Vector<CodedEntry> v = new Vector<CodedEntry>();
		for (int i=0; i<a.length; i++)
		{
		    v.add(a[i]);
		}
		return v;
	}


	public Vector<CodedEntry> getMatches(
		String pattern,
		int matchLimit)
	{
		return findConceptsWithNameMatching(pattern, matchLimit);
	}

	 public String getConceptStatus(String conceptname)
	 {
		 String code = nameToCode(codingScheme, version, tag, conceptname);
		 String status = getConceptStatus(codingScheme, version, tag, code);
		 if (status == null) return "Unknown";
		 return status;
	 }


    public Boolean hasStatus(String conceptname)
    {
		String code = nameToCode(codingScheme, version, tag, conceptname);
		String status = getConceptStatus(codingScheme, version, tag, code);
		if (status == null) return Boolean.FALSE;
		return Boolean.TRUE;
	}


	public boolean isValidHistoryDate(Date historydate)
	{
		return isValidHistoryDate(codingScheme, historydate);
	}


    public Vector getPropertyNames(String conceptname)
    {
        return getPropertyNames(codingScheme,
	                            version,
								version,
								conceptname);

	}


	/**
	 * Find relationship (e.g., role) sources and targets
	 *
	 * @since DLBExt 1.0
	 * @param code concept code
	 * @param relation relationship name
	 * @param direction true: code is source; false: code is target
	 */


	public Vector<String> findRelatedCodes(String code,
                                   String relation,
	                               boolean direction)
	{
		return findRelatedCodes(codingScheme,
	                                    version,
	                                    tag,
	                                    code,
	                                    relation,
	                                    direction);
	}

//	used
	public Vector<String> getAllQualifierTypes()
	{
		return getSupportedPropertyQualifier(codingScheme);
	}
//	used
	public Vector<String> getAllConceptPropertyQualifierTypes()
	{
		return getSupportedPropertyQualifier(codingScheme);
	}

	public Vector<String> getConceptPropertyQualifierTypes()
	{
		return getSupportedPropertyQualifier(codingScheme);
	}

//	used
	public Vector<String> getAllConceptAssociationQualifierTypes()
	{
		return getSupportedAssociationQualifier(codingScheme);
	}

	public Vector<String> getConceptAssociationQualifierTypes()
	{
		return getSupportedAssociationQualifier(codingScheme);
	}


    public RoleType[] getAllRoleTypesInNamespace()
    {
		Vector v = getSupportedRoleNames();

		RoleType[] roletypes = new RoleType[v.size()];
		for (int i=0; i<v.size(); i++)
		{
			String name = (String) v.elementAt(i);
			RoleType rt = new RoleType(name, -1, "");
			roletypes[i] = rt;
		}
		return roletypes;
	}

    private EditActionDate NCIChangeEvent2EditActionDate(NCIChangeEvent nce)
    {
		if (nce == null) return null;
		EditActionDate ead = new EditActionDate();
		ead.setEditDate(nce.getEditDate());
		ChangeType ct =	nce.getEditaction();
		ead.setAction(ct.getType());
		return ead;
	}

//  used
    public ConceptHistory getConceptHistory(String conceptcode)
    {
		ConceptHistory history = new ConceptHistory();
		history.setCode(conceptcode);
		history.setName(getConceptNameByCode(conceptcode));
		history.setNamespaceId(getNamespaceId(codingScheme));

        Vector v = getConceptHistory(codingScheme, conceptcode);
        for (int i=0; i<v.size(); i++)
        {
			NCIChangeEvent nce = (NCIChangeEvent) v.elementAt(i);
			history.addEditActionDate(NCIChangeEvent2EditActionDate(nce));
		}

		return history;
	}

////////////////////////////////////////////////////////////////////////////////////////

	public CodedEntry[] searchConcepts(String pattern, int searchLimit)
	{
		return searchConcepts(pattern, searchLimit, defaultAlgorithm);
	}


	public CodedEntry[] searchConcepts(String pattern, int max, String matchAlgorithm)
	{
		Vector w = findCodedEntriesWithDesignationMatching(
					  codingScheme,
					  version,
					  pattern,
					  false,
					  null,
					  null,
					  matchAlgorithm,
					  max);

// to be implemented (asdIndex)??? default 0

		return toArray(w);
	}


	/** Searches concepts that match with a specific search criteria (complex search).
	 *
	 * @param namespaceId namespace id
	 * @param pattern search pattern
	 * @param limit maximum matched concepts
	 * @param matchOption match option
	 * @param matchType match type (property name or role name, if search by
	 *     property or role)
	 * @param ASDIndex AttributeSetDescriptor index
	 * @return concepts that match with a specific search criteria in a byte
	 *     array
	 */


	private CodedEntry[] searchConcepts(
	             int namespaceId,
				 String pattern,
				 int limit,
				 int matchOption,
				 String matchType, // role/property name, or source name (if search by name)
				 int ASDIndex
				 )
	{
        String codingSchemeName = "";
        if (namespaceId <= 0)
        {
			codingSchemeName = codingScheme;
		}
		else
		{
			Integer nsid_obj = new Integer(namespaceId);
			String nsid_str = nsid_obj.toString();
			codingSchemeName = localName2FormalName(nsid_str);
	    }

	    return searchConcepts(
	             codingSchemeName,
				 pattern,
				 limit,
				 matchOption,
				 matchType,
				 ASDIndex
				 );
	}


    public CodedEntry[] searchConcepts(
				 String pattern,
				 int limit,
				 int matchOption,
				 String matchType)
	{
        return searchConcepts(
			     codingScheme,
				 pattern,
				 limit,
				 matchOption,
				 matchType,
				 0);
	}


//  used
    public CodedEntry[] searchConcepts(
				 String pattern,
				 int limit,
				 int matchOption,
				 String matchType,
				 int ASDIndex)
	{
		return searchConcepts(
	             codingScheme,
				 pattern,
				 limit,
				 matchOption,
				 matchType,
				 ASDIndex,
				 defaultAlgorithm,
				 null);
	}

	public CodedEntry[] searchConcepts(
	             String codingSchemeName,
				 String pattern,
				 int limit,
				 int matchOption,
				 String matchType,
				 int ASDIndex)
	{
		return searchConcepts(
	             codingSchemeName,
				 pattern,
				 limit,
				 matchOption,
				 matchType,
				 ASDIndex,
				 defaultAlgorithm,
				 null);
	}

//used
	public CodedEntry[] searchConcepts(
				 String pattern,
				 int limit,
				 int matchOption,
				 String matchType,
				 int ASDIndex,
				 String matchAlgorithm)
    {
		return searchConcepts(
	             codingScheme,
				 pattern,
				 limit,
				 matchOption,
				 matchType,
				 ASDIndex,
				 matchAlgorithm,
				 null);
    }


	public CodedEntry[] searchConcepts(
	             String codingSchemeName,
				 String pattern,
				 int limit,
				 int matchOption,
				 String matchType,
				 int ASDIndex,
				 String matchAlgorithm)
	{

		return searchConcepts(
	             codingSchemeName,
				 pattern,
				 limit,
				 matchOption,
				 matchType,
				 ASDIndex,
				 matchAlgorithm,
				 null); // restricted_property_vec

	}

	public CodedEntry[] searchConcepts(
				 String pattern,
				 int limit,
				 int matchOption,
				 String matchType,
				 int ASDIndex,
				 String matchAlgorithm,
				 Vector restricted_property_vec)
	{
		return searchConcepts(
	             codingScheme,
				 pattern,
				 limit,
				 matchOption,
				 matchType,
				 ASDIndex,
				 matchAlgorithm,
				 restricted_property_vec);

	}


	public CodedEntry[] searchConcepts(
	             String codingSchemeName,
				 String pattern,
				 int limit,
				 int matchOption,
				 String matchType,
				 int ASDIndex,
				 String matchAlgorithm,
				 Vector restricted_property_vec)
	{
		Vector<CodedEntry> v = new Vector<CodedEntry>();
		if (matchAlgorithm == null || matchAlgorithm.compareTo("") == 0)
		{
			matchAlgorithm = defaultAlgorithm;
		}
	    try {

			if (matchOption == MATCH_BY_NAME)
			{
                Vector<String> source_vec = null;
                if (matchType != null && matchType.compareTo("") != 0)
                {
					source_vec = new Vector<String>();
					source_vec.add(matchType);
				}

				v = findCodedEntriesWithDesignationMatching(
							  codingSchemeName,
							  version,
							  pattern,
							  false, // preferredOnly
							  null, //  sortByProperty_vec
							  source_vec, // source_vec
							  matchAlgorithm,
							  limit);

		        return toArray(v);
			}

			else if (matchOption == MATCH_BY_ROLE)
			{
		         v = findConceptsWithAssociationMatching(codingSchemeName, matchType, pattern, limit, matchAlgorithm, restricted_property_vec);
			}

			else if (matchOption == MATCH_BY_PROPERTY)
			{
				 v = findConceptsWithPropertyMatching(codingSchemeName, matchType, pattern, limit, matchAlgorithm);
			}

			else if (matchOption == MATCH_BY_SILO)
			{
                 // not supported by LexBIG
			}

			else if (matchOption == MATCH_BY_ASSOCIATION)
			{
		         v = findConceptsWithAssociationMatching(codingSchemeName, matchType, pattern, limit, matchAlgorithm, restricted_property_vec);
			}

			else if (matchOption == MATCH_BY_INVERSE_ROLE)
			{
			     v = findConceptsWithInverseAssociationMatching(codingSchemeName, matchType, pattern, limit, matchAlgorithm, restricted_property_vec);
			}

			else if (matchOption == MATCH_BY_INVERSE_ASSOCIATION)
			{

			     v = findConceptsWithInverseAssociationMatching(codingSchemeName, matchType, pattern, limit, matchAlgorithm, restricted_property_vec);
			}

			else if (matchOption == MATCH_BY_SOURCE_CODE)
			{
				// search by an source code
				// if no code is specified, then all concepts have the specified source will be retrieved
				v = findConceptsWithSourceCodeMatching(codingSchemeName, matchType, pattern, limit);
			}

		} catch (Exception e) {
	        e.printStackTrace();
		}

		return toArray(v);
	}


	public int getMatchedConceptCount(
				 String pattern,
				 int matchOption,
				 String matchType,
				 String matchAlgorithm,
				 Vector restricted_property_vec)
	{
		return getMatchedConceptCount(
	             codingScheme,
				 pattern,
				 matchOption,
				 matchType,
				 matchAlgorithm,
				 restricted_property_vec);
	}




	public int getMatchedConceptCount(
	             String codingSchemeName,
				 String pattern,
				 int matchOption,
				 String matchType,
				 String matchAlgorithm,
				 Vector restricted_property_vec)
	{
		CodedEntry[] concepts = searchConcepts(
	             codingSchemeName,
				 pattern,
				 100000,
				 matchOption,
				 matchType,
				 0,//ASDIndex,
				 matchAlgorithm,
				 restricted_property_vec);
		return concepts.length;
	}


    public Vector<Role> getRoles(CodedEntry ce)
    {
		return getRoles(codingScheme, version, tag, ce);
	}

    public Vector<Role> getRoles(String codingScheme, String version, String tag, CodedEntry ce)
    {
		if (ce == null) return null;
		return getRoles(codingScheme, version, tag, ce.getConceptCode());
	}


    public Vector<Role> getRoles(String codingScheme, String version, String tag, String code)
    {
		Vector<Role> roles = new Vector<Role>();
		Vector<ConceptAssociation> w1 = null;
		try {
			w1 = getConceptAssociations(
										   codingScheme,
										   version,
										   tag,
										   code,
										   true,
										   ROLE_RELATION);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}


		if (w1 != null)
		{
			for (int i=0; i<w1.size(); i++)
			{
				gov.nih.nci.lexbig.ext.ConceptAssociation nv = (gov.nih.nci.lexbig.ext.ConceptAssociation) w1.elementAt(i);
				{
					if (!nv.getName().equals(IS_A) && !nv.getName().equals(IS_A_OBO))// && hset.contains(nv.getName()))
					{
						roles.add(conceptAssociation2Role(nv));
					}
				}
			}

		}
		return roles;
	}


    public Vector getInverseRoles(CodedEntry ce)
    {
		return getInverseRoles(codingScheme, version, tag, ce);
	}

    public Vector getInverseRoles(String codingScheme, String version, String tag,CodedEntry ce)
    {
		if (ce == null) return null;
		return getInverseRoles(codingScheme, version, tag, ce.getConceptCode());
	}


    public Vector<Role> getInverseRoles(String codingScheme, String version, String tag, String code)
    {
		Vector<Role> roles = new Vector<Role>();
		Vector<ConceptAssociation> w2 = null;
		try {
		     w2 = getConceptAssociations(
										   codingScheme,
										   version,
										   tag,
										   code,
										   false,
										   ROLE_RELATION);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}

		if (w2 != null)
		{
			for (int i=0; i<w2.size(); i++)
			{
				gov.nih.nci.lexbig.ext.ConceptAssociation nv = (gov.nih.nci.lexbig.ext.ConceptAssociation) w2.elementAt(i);
				{
					if (!nv.getName().equals(IS_A) && !nv.getName().equals(IS_A_OBO))// && hset.contains(nv.getName()))
					{
						roles.add(conceptAssociation2Role(nv));
					}
				}
			}
		}
		return roles;
	}


    public Vector getConceptAssociations(CodedEntry ce)
    {
		return getConceptAssociations(codingScheme, version, tag, ce);
	}

    public Vector getConceptAssociations(String codingScheme, String version, String tag,CodedEntry ce)
    {
		if (ce == null) return null;
		return getConceptAssociations(codingScheme, version, tag, ce.getConceptCode());
	}


    public Vector<ConceptAssociation> getConceptAssociations(String codingScheme, String version, String tag, String code)
    {
		Vector<ConceptAssociation> associations = new Vector<ConceptAssociation>();
		Vector<ConceptAssociation> w1 = null;

		String relationName = null;
		if (hasRelation(codingScheme, version, tag, ASSOCIATION_RELATION)) relationName = ASSOCIATION_RELATION;
		try {
		    w1 = getConceptAssociations(
										   codingScheme,
										   version,
										   tag,
										   code,
										   true,
										   relationName);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}

		if (w1 != null)
		{
			for (int i=0; i<w1.size(); i++)
			{
				gov.nih.nci.lexbig.ext.ConceptAssociation nv = (gov.nih.nci.lexbig.ext.ConceptAssociation) w1.elementAt(i);
				{
					if (!nv.getName().equals(IS_A) && !nv.getName().equals(IS_A_OBO))// && hset.contains(nv.getName()))
					{
						associations.add(nv);
					}
				}
			}

		}
		return associations;
	}



    public Vector getInverseConceptAssociations(CodedEntry ce)
    {
		return getInverseConceptAssociations(codingScheme, version, version, ce);
	}

    public Vector getInverseConceptAssociations(String codingScheme, String version, String tag,CodedEntry ce)
    {
		if (ce == null) return null;
		return getInverseConceptAssociations(codingScheme, version, tag, ce.getConceptCode());
	}


    public Vector<ConceptAssociation> getInverseConceptAssociations(String codingScheme, String version, String tag, String code)
    {
		Vector<ConceptAssociation> associations = new Vector<ConceptAssociation>();
		Vector<ConceptAssociation> w1 = null;

		String relationName = null;
		if (hasRelation(codingScheme, version, tag, ASSOCIATION_RELATION)) relationName = ASSOCIATION_RELATION;
		try {
		    w1 = getConceptAssociations(
										   codingScheme,
										   version,
										   tag,
										   code,
										   false,
										   //ASSOCIATION_RELATION);
										   relationName);

		} catch (Exception ex) {
			//ex.printStackTrace();
			return null;
		}

		if (w1 != null)
		{
 		    for (int i=0; i<w1.size(); i++)
			{
				gov.nih.nci.lexbig.ext.ConceptAssociation nv = (gov.nih.nci.lexbig.ext.ConceptAssociation) w1.elementAt(i);
				{
					if (!nv.getName().equals(IS_A) && !nv.getName().equals(IS_A_OBO))// && hset.contains(nv.getName()))
					{
						associations.add(nv);
					}
				}
			}

		}
		return associations;
	}


    public void outputConcept(PrintWriter pw, CodedEntry ce)
	{
		if (pw == null)
		{
			System.out.println("outputConcept -- PrintWriter is null.");
			return;
		}
		if (ce == null)
		{
			System.out.println("outputConcept -- ce is null.");
			return;
		}

        String conceptName = getName(ce);
//        String code = ce.getConceptCode();
		pw.println("\nConcept: " + conceptName);
		pw.println("Code: " + ce.getConceptCode());
		//pw.println("Concept Status: " + ce.getConceptStatus());
		pw.println("Is Concept Active: " + ce.getIsActive());

		pw.println("\nProperties:");
		Vector properties = getProperties(ce);
		for (int i=0; i<properties.size(); i++)
		{
			gov.nih.nci.lexbig.ext.Property p = (gov.nih.nci.lexbig.ext.Property) properties.elementAt(i);
			pw.println("\t" + p.getName() + ": " + p.getValue());
			Vector qualifier_vec = p.getQualifiers();
			for (int j=0; j<qualifier_vec.size(); j++)
			{
				Qualifier q = (Qualifier) qualifier_vec.elementAt(j);
				pw.println("\t\t" + q.getName() + ": " + q.getValue());
			}
		}


		pw.println("\nRoles:");
		Vector roles = getRoles(ce);
		for (int i=0; i<roles.size(); i++)
		{
			Role role = (Role) roles.elementAt(i);
            CodedEntry c = getConcept(role.getValue(), 0);
			pw.println("\t["
				+  conceptName
				+  "]-->(" + role.getName()
				+")-->["
				+ getName(c) + " (" + role.getValue() + ")"
				+ "]");

			Vector qualifier_vec = role.getQualifiers();
			for (int j=0; j<qualifier_vec.size(); j++)
			{
				Qualifier q = (Qualifier) qualifier_vec.elementAt(j);
				pw.println("\t\t" + q.getName() + ": " + q.getValue());
			}

		}
		if (roles.size() == 0)
		{
			pw.println("\tNone");
		}

        pw.println("\nInverse Roles:");
		roles = getInverseRoles(ce);
		for (int i=0; i<roles.size(); i++)
		{
			Role role = (Role) roles.elementAt(i);

//			CodedEntry c = getConcept(role.getValue(), 0);

			pw.println("\t["
				+  role.getValue()
				+  "]-->(" + role.getName()
				+")-->["
				+ conceptName + " (" + ce.getConceptCode() + ")"
				+ "]");

			Vector qualifier_vec = role.getQualifiers();
			for (int j=0; j<qualifier_vec.size(); j++)
			{
				Qualifier q = (Qualifier) qualifier_vec.elementAt(j);
				pw.println("\t\t" + q.getName() + ": " + q.getValue());
			}

		}
		if (roles.size() == 0)
		{
			pw.println("\tNone");
		}

		pw.println("\nAssociations:");
		Vector associations = getConceptAssociations(ce);
		for (int i=0; i<associations.size(); i++)
		{
			gov.nih.nci.lexbig.ext.ConceptAssociation asso = (ConceptAssociation) associations.elementAt(i);
			CodedEntry c = getConcept(asso.getValue(), 0);
			pw.println("\t["
				+  conceptName
				+  "]-->(" + asso.getName()
				+")-->["
				+ getName(c) + " (" + asso.getValue() + ")"
				+ "]");

			Vector qualifier_vec = asso.getQualifiers();
			for (int j=0; j<qualifier_vec.size(); j++)
			{
				Qualifier q = (Qualifier) qualifier_vec.elementAt(j);
				pw.println("\t\t" + q.getName() + ": " + q.getValue());
			}

		}
		if (associations.size() == 0)
		{
			pw.println("\tNone");
		}

        pw.println("\nInverse Associations:");
		associations = getInverseConceptAssociations(ce);
		for (int i=0; i<associations.size(); i++)
		{
		    gov.nih.nci.lexbig.ext.ConceptAssociation asso = (ConceptAssociation) associations.elementAt(i);
//		    CodedEntry c = getConcept(asso.getValue(), 0);

			pw.println("\t["
				+  asso.getValue()
				+  "]-->(" + asso.getName()
				+")-->["
				+ conceptName + " (" + ce.getConceptCode() + ")"
				+ "]");

			Vector qualifier_vec = asso.getQualifiers();
			for (int j=0; j<qualifier_vec.size(); j++)
			{
				Qualifier q = (Qualifier) qualifier_vec.elementAt(j);
				pw.println("\t\t" + q.getName() + ": " + q.getValue());
			}
		}
		if (associations.size() == 0)
		{
			pw.println("\tNone");
		}
		pw.println("------------------\n\n");
	}


/*
(C) id
*/

	public RoleType getRoleTypeById(int roleId)
	{
		Vector v = getSupportedRoleNames();
		if (roleId <0 || roleId >= v.size()) return null;
		String rolename = (String) v.elementAt(roleId);
		return new RoleType(rolename, roleId, "");
	}


	public PropertyType getPropertyTypeById(int propertyId)
	{
		PropertyType[] properties = getAllPropertyTypes();
		if (propertyId >= 0 && propertyId<properties.length)
		{
			String name = properties[propertyId].getName();
			return new PropertyType(name, propertyId, "");
		}
		return null;
	}


/////////////////////////////////////////////////////////////////////////////////////////////////
// Methods for supporting metathesaurus queries
/////////////////////////////////////////////////////////////////////////////////////////////////
//used
	public Vector getSupportedSources()
	{
		return getSupportedSourceNames(codingScheme, version, tag);
	}


	/**
	 * Returns a vector of the names of org.LexGrid.naming.SupportedContext for the default coding scheme.
	 *
	 * @since DLBExt 1.0
	 * @return a vector of the names of org.LexGrid.naming.SupportedContext for
	 *     the default coding scheme.
	 */

	public Vector getSupportedContext()
	{
		return getSupportedContext(codingScheme, version, tag);
	}

	public int getSupportedContextCount()
	{
		return getSupportedContextCount(codingScheme, version, tag);
	}
//used
    public Vector<CodedEntry> findConceptsWithSourceCodeMatching(String sourceAbbr, String code, int limit)
    {
         return findConceptsWithSourceCodeMatching(codingScheme, sourceAbbr, code, limit);
	}


    public Vector<CodedEntry> findConceptsWithSourceCodeMatching(String codingSchemeName, String sourceAbbr, String code, int limit)
    {
		/*
        Vector sources = getSupportedSourceNames(codingSchemeName, null, null);
        if (!sources.contains(sourceAbbr))
        {
			getLogger().warn("Source " + sourceAbbr + " is not identifiable.");
			return null;
		}
		*/

		Vector<CodedEntry> v = findCodedEntriesWithSourceCodeMatching(codingSchemeName, sourceAbbr, code, limit);
		return v;
	}


    public Vector getRelatedConcepts(String conceptCode, String associationName)
    {
		return getRelatedConceptsBySource(
		 								 conceptCode,
		 								 associationName,
		 								 null); // sourceAbbr
	}

//  used
    public Vector getRelatedConcepts(String conceptCode, boolean direction, String associationName)
    {
		return getRelatedConceptsBySource(
		 								 conceptCode,
		 								 direction,
		 								 associationName,
		 								 null); // sourceAbbr
	}


    // 031407
    public Vector<CodedEntry> getRelatedConcepts(String conceptCode, boolean direction, String associationName, int ASD_Index)
    {
        Vector<CodedEntry> v = getRelatedCodedEntries(codingScheme,
 								 version,
 								 tag,
 								 conceptCode,
 								 direction,
 								 associationName);

		return v;
	}



    public Vector<CodedEntry> getRelatedConceptsBySource(
 								 String conceptCode,
 								 String associationName,
 								 String sourceAbbr)
	{
         return getRelatedConceptsBySource(
 								 conceptCode,
 								 associationName,
 								 sourceAbbr,
 								 0);

	}

    // No direction is specified == both direction applied:
    public Vector<CodedEntry> getRelatedConceptsBySource(
 								 String conceptCode,
 								 String associationName,
 								 String sourceAbbr,
 								 int ASD_Index)
   {
	    if (sourceAbbr != null)
	    {
			Vector sources = getSupportedSources();
			if (!sources.contains(sourceAbbr))
			{
				getLogger().error("Source " + sourceAbbr + " is not identifiable.");
				return null;
			}
	    }

	    /*
        Vector v1 = getAssociatedCodedEntriesBySource(codingScheme,
										 null,//String version,
										 null,//String tag,
	                                     conceptCode,
										 new Vector(),//Vector properties,
										 true,//boolean direction,
										 associationName,
										 1,//int resolveAssociationDepth,
										 1,//int resolveCodedEntryDepth,
										 sourceAbbr
										 );
        Vector v2 = getAssociatedCodedEntriesBySource(codingScheme,
										 null,//String version,
										 null,//String tag,
	                                     conceptCode,
										 new Vector(),//Vector properties,
										 false,//boolean direction,
										 associationName,
										 1,//int resolveAssociationDepth,
										 1,//int resolveCodedEntryDepth,
										 sourceAbbr
										 );
        */

         Vector<CodedEntry> v1 = getAssociatedCodedEntriesBySource(codingScheme,
 										 version,
 										 tag,
 	                                     conceptCode,
 										 new Vector(),//Vector properties,
 										 true,//boolean direction,
 										 associationName,
 										 1,//int resolveAssociationDepth,
 										 1,//int resolveCodedEntryDepth,
 										 sourceAbbr
 										 );
         Vector<CodedEntry> v2 = getAssociatedCodedEntriesBySource(codingScheme,
 										 version,
 										 tag,
 	                                     conceptCode,
 										 new Vector(),//Vector properties,
 										 false,//boolean direction,
 										 associationName,
 										 1,//int resolveAssociationDepth,
 										 1,//int resolveCodedEntryDepth,
 										 sourceAbbr
										 );


        // 031507
        if (v1 == null && v2 == null) return new Vector<CodedEntry>();
        else if (v1 != null && v2 == null) return v1;
        else if (v1 == null && v2 != null) return v2;
        else
        {
			for (int i=0; i<v2.size(); i++)
			{
				CodedEntry ce = (CodedEntry) v2.elementAt(i);
				v1.add(ce);
			}
			return v1;
		}
	}

//  used
    public Vector<CodedEntry> getRelatedConceptsBySource(
 								 String conceptCode,
 								 boolean direction,
 								 String associationName,
 								 String sourceAbbr)
	{
          return getRelatedConceptsBySource(
 								 conceptCode,
 								 direction,
 								 associationName,
 								 sourceAbbr,
 								 0);

	}

    public Vector<CodedEntry> getRelatedConceptsBySource(
 								 String conceptCode,
 								 boolean direction,
 								 String associationName,
 								 String sourceAbbr,
 								 int ASD_Index)
    {
		if (sourceAbbr != null)
		{
			Vector sources = getSupportedSources();

			if (!sources.contains(sourceAbbr))
			{
				getLogger().error("Source " + sourceAbbr + " is not identifiable.");
				return null;
			}
	    }
/*
        Vector v = getAssociatedCodedEntriesBySource(codingScheme,
										 null,//String version,
										 null,//String tag,
	                                     conceptCode,
										 new Vector(),//Vector properties,
										 //true,//boolean direction, 050807
										 direction,
										 associationName,
										 1,//int resolveAssociationDepth,
										 1,//int resolveCodedEntryDepth,
										 sourceAbbr
										 );
*/

        Vector<CodedEntry> v = getAssociatedCodedEntriesBySource(codingScheme,
										 version,
										 tag,
	                                     conceptCode,
										 new Vector(),//Vector properties,
										 //true,//boolean direction, 050807
										 direction,
										 associationName,
										 1,//int resolveAssociationDepth,
										 1,//int resolveCodedEntryDepth,
										 sourceAbbr
										 );

        // 031507
        if (v == null) return new Vector<CodedEntry>();
		return v;
	}


    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    //static String ROOT_ASSOCIATION = "owl:AnnotationProperty";
    //static String ROOT_ROLE = "owl:ObjectProperty";

	public Vector getSupportedAssociationNames()
	{
         return getSupportedAssociationNames(codingScheme);
	}

	public Vector getSupportedConceptAssociationNames()
	{
         return getConceptAssociationTypes();
	}

	public Vector getSupportedConceptAssociationNames(String codingSchemeName)
	{
         return getAssociationNamesByRelation(codingSchemeName, version, DLBWrapper.ASSOCIATION_RELATION);
	}

	public Vector<String> getSupportedRoleNames()
	{
         return getSupportedRoleNames(codingScheme);
	}
    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


    public Vector getConceptAssociationTypes()
    {
		if (hasRelation(codingScheme, ASSOCIATION_RELATION))
		{
			return getAssociationNamesByRelation(codingScheme, version, ASSOCIATION_RELATION);
		}
		else
		{
			return getAssociationNamesByRelation(codingScheme, version, null);
		}
	}

//used
	public Vector getAllAssociationTypes()
	{
		return getSupportedAssociationNames();
	}
//	used
	public Vector getAllConceptAssociationTypes()
	{
		return getConceptAssociationTypes();
	}



	public Vector getSupportedPropertyQualifiers()
	{
         return getSupportedPropertyQualifiers(codingScheme);
	}


	public Vector getDefinitionCollectionbyCode(String vocabularyName, String code) {
		return getConceptProperties(vocabularyName, code, DLBWrapper.TYPE_DEFINITION, null);
	}

	public Vector getSemanticTypeCollectionbyCode(String vocabularyName, String code) {
		return getConceptProperties(vocabularyName, code, DLBWrapper.TYPE_PROPERTY, "Semantic_Type");
	}

	public Vector getPropertyCollectionbyCode(String vocabularyName, String code) {
		return getConceptProperties(vocabularyName, code, DLBWrapper.TYPE_PROPERTY, null);
	}

	public Vector getAtomCollectionbyCode(String vocabularyName, String code) {
		return getConceptProperties(vocabularyName, code, DLBWrapper.TYPE_PRESENTATION, null);
	}


	public Vector getRoleCollectionbyCode(String vocabularyName, String code) {
		CodedEntry ce = getCodedEntry(vocabularyName, version, tag, code);
		return getRoles(ce);
	}

	public Vector getInverseRoleCollectionbyCode(String vocabularyName, String code) {
		CodedEntry ce = getCodedEntry(vocabularyName, version, tag, code);
		return getInverseRoles(ce);
	}

	public Vector getAssociationCollectionbyCode(String vocabularyName, String code) {
		CodedEntry ce = getCodedEntry(vocabularyName, version, tag, code);
		return getConceptAssociations(ce);
    }

	public Vector getInverseAssociationCollectionbyCode(String vocabularyName, String code) {
		CodedEntry ce = getCodedEntry(vocabularyName, version, tag, code);
	    return getInverseConceptAssociations(ce);
    }


	public Vector<EditActionDate> getHistoryCollectionbyCode(String vocabularyName, String code) {
		// 072807
	    Vector<NCIChangeEvent> v = getConceptHistory(vocabularyName, code);
	    Vector<EditActionDate> w = new Vector<EditActionDate>();
		for (int i=0; i<v.size(); i++)
		{
			NCIChangeEvent nce = (NCIChangeEvent) v.elementAt(i);
			EditActionDate e = new EditActionDate();
			e.setEditDate(nce.getEditDate());
			e.setAction(nce.getEditaction().getType());
			w.add(e);
		}
		return w;

	}

    public boolean isAssociationSupported(String name)
	{
		Vector v = getSupportedAssociationNames(codingScheme);
		return v.contains(name);
	}
/*
    public boolean isAssociationSupported(String codingSchemeName, String name)
	{
		Vector v = getSupportedAssociationNames(codingSchemeName, version, null);
		return v.contains(name);
	}
*/

 // boolean shortResult, boolean score not used.

	public CodedEntry[] searchConcepts(String scheme, String searchTerm, int limit, String source,
						 boolean cui, boolean shortResult, boolean score, int ASDIndex)
    {
		Vector<CodedEntry> v = new Vector<CodedEntry>();
		if (cui)
		{
			CodedEntry concept  = findConceptByCode(searchTerm, ASDIndex);
			if (concept != null)
			{
				v.add(concept);
			}
		}
		else
		{
			if (source != null && source.compareTo("*") != 0) // search by source
			{
			    int matchOption = MATCH_BY_SOURCE_CODE;
			    String matchType = source;
			    return searchConcepts(
				              scheme,
							  searchTerm,
							  limit,
							  matchOption,
							  matchType,
				              ASDIndex);

			}
			else
			{
			    int matchOption = MATCH_BY_NAME;
			    String matchType = null;
			    return searchConcepts(
				              scheme,
							  searchTerm,
							  limit,
							  matchOption,
							  matchType,
				              ASDIndex);
				/*
				v = searchCodedEntries(scheme, version, tag, searchTerm, limit, source,
										 cui, shortResult, score); // lazy load
				*/
 		    }
	    }
	    return toArray(v);
	}

	public CodedEntry[] searchConcepts(String scheme, String searchTerm, int limit, String source,
						 boolean cui, boolean shortResult, boolean score)
    {
	   try {
	       return searchConcepts(scheme, searchTerm, limit, source, cui, shortResult, score, 0);
	   } catch (Exception ex) {
	      ex.printStackTrace();
	      return null;
	   }
	}


    public Vector getAvailableMetaDataURNs()
    {
		Vector<String> v = new Vector<String>();
		try {
			AbsoluteCodingSchemeVersionReferenceList acsvrl = metadataService.listCodingSchemes();
			AbsoluteCodingSchemeVersionReference[] acdvra =	acsvrl.getAbsoluteCodingSchemeVersionReference();
			for (int i=0; i<acdvra.length; i++)
			{
				AbsoluteCodingSchemeVersionReference acsvr = acdvra[i];
				String urn = acsvr.getCodingSchemeURN();
//				String version = acsvr.getCodingSchemeVersion();
				v.add(urn);
			}
	    } catch (Exception e) {
			e.printStackTrace();

		}
		return v;
	}


	public NameAndValue[] getMetadataProperties(String codingSchemeURN)
	{
        String formalName = localName2FormalName(codingSchemeURN);
		Vector<NameAndValue> v = new Vector<NameAndValue>();
		try {
			LexBIGServiceMetadata svc = lbSvc.getServiceMetadata();

			AbsoluteCodingSchemeVersionReferenceList acsvrl = svc.listCodingSchemes();
			AbsoluteCodingSchemeVersionReference[] acdvra =	acsvrl.getAbsoluteCodingSchemeVersionReference();
			for (int i=0; i<acdvra.length; i++)
			{
				AbsoluteCodingSchemeVersionReference acsvr = acdvra[i];
				String urn = acsvr.getCodingSchemeURN();
				if (urn.equals(formalName))
				{
					//100807 KLO
					svc = svc.restrictToCodingScheme(acsvr);
					MetadataPropertyList mdpl = svc.resolve();
					MetadataProperty[] properties = mdpl.getMetadataProperty();
					for (int j=0; j<properties.length; j++)
					{
						MetadataProperty property = properties[j];
						NameAndValue nv = createNameAndValue(property.getName(), property.getValue());
						v.add(nv);
					}
				}
			}

			if (v.size() > 0)
			{
				NameAndValue[] nv_array = new NameAndValue[v.size()];
				for (int i=0; i<v.size(); i++)
				{
					NameAndValue nv = (NameAndValue) v.elementAt(i);
					nv_array[i] = nv;
			    }
			    return nv_array;
			}

	    } catch (Exception e) {
			e.printStackTrace();

		}
		return new NameAndValue[0];

	}


	public static Date stringToDate(String aString)
    {
		return stringToDate(aString, "MM/dd/yyyy");
	}

	public static Date stringToDate(String aString, String format)
	{
		try {
			SimpleDateFormat sdf = new SimpleDateFormat(format);
			ParsePosition pos = new ParsePosition(0);
 			Date theDate = sdf.parse(aString, pos);
 			return theDate;
 		}catch(Exception e){
 			e.printStackTrace();
 			return null;
 		}
	}
//	used??
	public Vector findConceptsWithSiloMatching(String searchTerm, int limit)
	{
		return new Vector();
	}

	//used
	public Vector fetchTermAssociations(String code)
	{
		return getConceptAssociations(codingScheme, version, tag, code);
	}


    public Vector getRelatedConceptCodes(String code, String associationName, String sourceAbbr)
    {
		return getRelatedConceptCodes(code, null, associationName, sourceAbbr);
	}

    public Vector getRelatedConceptCodes(String code,  Boolean direction, String associationName, String sourceAbbr)
    {
		Vector<String> v = new Vector<String>();
		Vector<AssociatedConcept> v1 = new Vector<AssociatedConcept>();
		Vector<AssociatedConcept> v2 = new Vector<AssociatedConcept>();
		try {
			if (direction == null)
			{
				v1 = getSources(codingScheme, version, tag, code, associationName, sourceAbbr);
				v2 = getTargets(codingScheme, version, tag, code, associationName, sourceAbbr);
			}
			else if (direction.equals(Boolean.TRUE))
			{
				v2 = getTargets(codingScheme, version, tag, code, associationName, sourceAbbr);
			}
			else if (direction.equals(Boolean.FALSE))
			{
				v2 = getSources(codingScheme, version, tag, code, associationName, sourceAbbr);
			}

			for (int i=0; i<v1.size(); i++)
			{
				AssociatedConcept ac = (AssociatedConcept) v1.elementAt(i);
				v.add(ac.getConceptCode());
			}

			for (int i=0; i<v2.size(); i++)
			{
				AssociatedConcept ac = (AssociatedConcept) v2.elementAt(i);
				v.add(ac.getConceptCode());
			}
	    } catch (Exception e) {
			e.printStackTrace();
			return v;
		}
		return v;
	}



}

