package gov.nih.nci.lexbig.ext;


import java.util.*;
import java.io.*;
import java.sql.Date;
import java.text.SimpleDateFormat;

import org.w3c.dom.*;
import org.apache.xerces.dom.DocumentImpl;
import org.apache.xerces.dom.DOMImplementationImpl;
import org.w3c.dom.Document;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.Serializer;
import org.apache.xml.serialize.SerializerFactory;
import org.apache.xml.serialize.XMLSerializer;

/**
  * <!-- LICENSE_TEXT_START -->
* Copyright 2007 NGIT. This software was developed in conjunction with the National Cancer Institute,
* and so to the extent government employees are co-authors, any rights in such works shall be subject to Title 17 of the United States Code, section 105.
* Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
* 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the disclaimer of Article 3, below. Redistributions
* in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
* 2. The end-user documentation included with the redistribution, if any, must include the following acknowledgment:
* "This product includes software developed by NGIT and the National Cancer Institute."
* If no such end-user documentation is to be included, this acknowledgment shall appear in the software itself,
* wherever such third-party acknowledgments normally appear.
* 3. The names "The National Cancer Institute", "NCI" and "NGIT" must not be used to endorse or promote products derived from this software.
* 4. This license does not authorize the incorporation of this software into any third party proprietary programs. This license does not authorize
* the recipient to use any trademarks owned by either NCI or NGIT
* 5. THIS SOFTWARE IS PROVIDED "AS IS," AND ANY EXPRESSED OR IMPLIED WARRANTIES, (INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE) ARE DISCLAIMED. IN NO EVENT SHALL THE NATIONAL CANCER INSTITUTE,
* NGIT, OR THEIR AFFILIATES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * <!-- LICENSE_TEXT_END -->
  */

/** PropertyType
 *
 * @since DLBExt 1.0
 * @author Northrop Grumman Information Technology
 * @version 1.0
 */

public class PropertyType implements Serializable {

	/** identifier
	 */
	int id;
	/** code
	 */
	String code;
	/** name
	 */
	String name;

	/** constructor
	 *
	 * @since DLBExt 1.0
	 */
	public PropertyType() {
		this.id = 0;
		this.code = "";
		this.name = "";
	}

	/** Constructs a PropertyType with the given name, id, and code values.
	 *
	 * @since DLBExt 1.0
	 * @param name name of the PropertyType.
	 * @param id id of the PropertyType.
	 * @param code code of the PropertyType.
	 */
	public PropertyType(String name, int id, String code) {
		this.name = name;
		this.id = id;
		this.code = code;
	}

	/** Assigns a value to id
	 *
	 * @since DLBExt 1.0
	 * @param id identifier
	 */
	public void setId(int id) {
		this.id = id;
	}

	/** Assigns a value to code
	 *
	 * @since DLBExt 1.0
	 * @param code code
	 */
	public void setCode(String code) {
		this.code = code;
	}

	/** Assigns a value to name
	 *
	 * @since DLBExt 1.0
	 * @param name name
	 */
	public void setName(String name) {
		this.name = name;
	}

	/** Gets id
	 *
	 * @since DLBExt 1.0
	 * @return id
	 */
	public int getId() {
		return this.id;
	}

	/** Gets code
	 *
	 * @since DLBExt 1.0
	 * @return code
	 */
	public String getCode() {
		return this.code;
	}

	/** Gets name
	 *
	 * @since DLBExt 1.0
	 * @return name
	 */
	public String getName() {
		return this.name;
	}

	/** Gets a XML representation of a PropertyType object in String
	 *
	 * @since DLBExt 1.0
	 * @return a XML representation of a PropertyType object in String
	 */
	public String toXML() {
		return writeDocument(toDOM());
	}

	/** Converts a XML Document to a String
	 *
	 * @since DLBExt 1.0
	 * @param doc a XML Document
	 * @return a String representation of a XML Document
	 */
	private String writeDocument(Document doc)
	{
	    try {
	        OutputFormat format = new OutputFormat(doc);
			StringWriter stringOut = new StringWriter();
	        XMLSerializer serial = new XMLSerializer(stringOut, format);
	        serial.asDOMSerializer();
	        serial.serialize(doc.getDocumentElement());
	        return stringOut.toString();
	    } catch (Exception ex) {
	        ex.printStackTrace();
	    }
	    return null;
	}

	/** Gets a String representation of a PropertyType object.
	 *
	 * @since DLBExt 1.0
	 * @return a String representation of a PropertyType object.
	 */
	public String toString() {
		return this.name;
	}

	/** Gets a XML Document representation of a PropertyType object.
	 *
	 * @since DLBExt 1.0
	 * @return a XML Document representation of a PropertyType object.
	 */
	public Document toDOM()
	{
		Document doc= new DocumentImpl();
		try {
			Element root = doc.createElement("PropertyType");
			Element element;
			element = doc.createElement("id");
			element.appendChild(doc.createTextNode(intToString(id)));
			root.appendChild(element);

			element = doc.createElement("code");
			element.appendChild(doc.createTextNode(code));
			root.appendChild(element);

			element = doc.createElement("name");
			element.appendChild(doc.createTextNode(name));
			root.appendChild(element);

			doc.appendChild(root);
	    } catch (Exception ex) {
	        ex.printStackTrace();
	    }
		return doc;
	}

	/** Converts an int value to a String
	 *
	 * @since DLBExt 1.0
	 * @param i an int value
	 * @return a String representation of an int value
	 */
	private String intToString(int i)
	{
		return "" + i;
	}

} // end of PropertyType
