package gov.nih.nci.lexbig.ext;

import java.util.*;
import java.io.*;
import java.sql.Date;
import java.text.SimpleDateFormat;

import org.w3c.dom.*;
import org.apache.xerces.dom.DocumentImpl;
import org.apache.xerces.dom.DOMImplementationImpl;
import org.w3c.dom.Document;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.Serializer;
import org.apache.xml.serialize.SerializerFactory;
import org.apache.xml.serialize.XMLSerializer;


import org.LexGrid.concepts.CodedEntry;

/**
  * <!-- LICENSE_TEXT_START -->
* Copyright 2007 NGIT. This software was developed in conjunction with the National Cancer Institute,
* and so to the extent government employees are co-authors, any rights in such works shall be subject to Title 17 of the United States Code, section 105.
* Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
* 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the disclaimer of Article 3, below. Redistributions
* in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
* 2. The end-user documentation included with the redistribution, if any, must include the following acknowledgment:
* "This product includes software developed by NGIT and the National Cancer Institute."
* If no such end-user documentation is to be included, this acknowledgment shall appear in the software itself,
* wherever such third-party acknowledgments normally appear.
* 3. The names "The National Cancer Institute", "NCI" and "NGIT" must not be used to endorse or promote products derived from this software.
* 4. This license does not authorize the incorporation of this software into any third party proprietary programs. This license does not authorize
* the recipient to use any trademarks owned by either NCI or NGIT
* 5. THIS SOFTWARE IS PROVIDED "AS IS," AND ANY EXPRESSED OR IMPLIED WARRANTIES, (INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE) ARE DISCLAIMED. IN NO EVENT SHALL THE NATIONAL CANCER INSTITUTE,
* NGIT, OR THEIR AFFILIATES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * <!-- LICENSE_TEXT_END -->
  */


/** TreeNode
 *
 * @since DLBExt 1.0
 * @author Northrop Grumman Information Technology
 * @version DLBExt 1.0
 */

public class TreeNode implements Serializable {

    /** Instance of CodedEntry
     */
    CodedEntry concept = null;

	/** a name
	 */
	String name;

	/** a HashSet for storing role relationships
	 */
	HashSet links;

	/** a boolean variable indicating whether the tree contains the is-a relationship (i.e., the taxonomy)
	 */
	boolean isA;

	/** traverse Down or Up (an boolean variable)
	 */

	boolean traverseDown;

	/** a default constructor
	 *
	 * @since DLBExt 1.0
	 */

	public TreeNode() {
		isA = false;
		links = new HashSet();
		traverseDown = true;
	}

	/** Assigns a value to name
	 *
	 * @since DLBExt 1.0
	 * @param name node name
	 */
	public void setName(String name) {
		this.name = name;
	}


	/** Adds a role name to links Sets value to link
	 *
	 * @since DLBExt 1.0
	 * @param rolename a role name
	 */
	public void addLink(String rolename) {
		links.add(rolename);
	}

	/** Assign value to links
	 *
	 * @since DLBExt 1.0
	 * @param links an HashSet instance
	 */
	public void setLinks(HashSet links) {
		this.links = links;
	}


	/** Assigns a value to Sets value to isA
	 *
	 * @since DLBExt 1.0
	 * @param isA a boolean variable (true, if the isA relationship is included;
	 *      false, otherwise)
	 */
	public void setIsA(boolean isA) {
		this.isA = isA;
	}

	/** Assigns a value to traverseDown
	 *
	 * @since DLBExt 1.0
	 * @param traverseDown a boolean variable, set to true if traverse down;
	 *     false, if traverse up
	 */
	public void setTraverseDown(boolean traverseDown) {
		this.traverseDown = traverseDown;
	}

	/** Gets name
	 *
	 * @since DLBExt 1.0
	 * @return return the textual presentation of the coded entry (preferred, if
	 *      it exists)
	 */
	public String getName() {
		return this.name;
	}

	/** Gets link
	 *
	 * @since DLBExt 1.0
	 * @return link
	 */
	public HashSet getLinks() {
		return links;
	}

	/** Gets isA
	 *
	 * @since DLBExt 1.0
	 * @return isA
	 */
	public boolean getIsA() {
		return isA;
	}

	/** Gets traverseDown
	 *
	 * @since DLBExt 1.0
	 * @return traverseDown
	 */
	public boolean getTraverseDown() {
		return traverseDown;
	}

	/** Sets concept to a given instance of CodedEntry.
	 *
	 * @since DLBExt 1.0
	 * @param ce an instance of CodedEntry
	 */
	public void setConcept(CodedEntry ce) {
		this.concept = ce;
	}

	/** Gets the instance of CodedEntry.
	 *
	 * @return The instance of CodedEntry
	 */
	public CodedEntry getConcept() {
		return this.concept;
	}


	/** Gets the name of the TreeNode instance.
	 *
	 * @return The name of the TreeNode instance.
	 */
	public String toString() {
		return this.name;
	}


} // end of TreeNode
