package gov.nih.nci.lexbig.ext;

import edu.mayo.informatics.indexer.api.exceptions.InternalErrorException;

import gov.nih.nci.system.applicationservice.EVSApplicationService;


import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;
import java.util.Vector;

import org.LexGrid.LexBIG.DataModel.Collections.AssociatedConceptList;
import org.LexGrid.LexBIG.DataModel.Collections.AssociationList;
import org.LexGrid.LexBIG.DataModel.Collections.CodingSchemeRenderingList;
import org.LexGrid.LexBIG.DataModel.Collections.CodingSchemeTagList;
import org.LexGrid.LexBIG.DataModel.Collections.CodingSchemeVersionList;
import org.LexGrid.LexBIG.DataModel.Collections.ConceptReferenceList;
import org.LexGrid.LexBIG.DataModel.Collections.ExtensionDescriptionList;
import org.LexGrid.LexBIG.DataModel.Collections.LocalNameList;
import org.LexGrid.LexBIG.DataModel.Collections.ModuleDescriptionList;
import org.LexGrid.LexBIG.DataModel.Collections.NCIChangeEventList;
import org.LexGrid.LexBIG.DataModel.Collections.NameAndValueList;
import org.LexGrid.LexBIG.DataModel.Collections.ResolvedConceptReferenceList;
import org.LexGrid.LexBIG.DataModel.Collections.SortOptionList;
import org.LexGrid.LexBIG.DataModel.Collections.SystemReleaseList;
import org.LexGrid.LexBIG.DataModel.Core.AssociatedConcept;
import org.LexGrid.LexBIG.DataModel.Core.Association;
import org.LexGrid.LexBIG.DataModel.Core.CodingSchemeSummary;
import org.LexGrid.LexBIG.DataModel.Core.CodingSchemeVersionOrTag;
import org.LexGrid.LexBIG.DataModel.Core.ConceptReference;
import org.LexGrid.LexBIG.DataModel.Core.NameAndValue;
import org.LexGrid.LexBIG.DataModel.Core.ResolvedConceptReference;
import org.LexGrid.LexBIG.DataModel.InterfaceElements.CodingSchemeRendering;
import org.LexGrid.LexBIG.DataModel.InterfaceElements.ExtensionDescription;
import org.LexGrid.LexBIG.DataModel.InterfaceElements.ModuleDescription;
import org.LexGrid.LexBIG.DataModel.InterfaceElements.RenderingDetail;
import org.LexGrid.LexBIG.DataModel.InterfaceElements.SortOption;
import org.LexGrid.LexBIG.DataModel.NCIHistory.NCIChangeEvent;
import org.LexGrid.LexBIG.DataModel.NCIHistory.types.ChangeType;
import org.LexGrid.LexBIG.Exceptions.LBException;
import org.LexGrid.LexBIG.Exceptions.LBParameterException;
import org.LexGrid.LexBIG.History.HistoryService;
import org.LexGrid.LexBIG.Impl.LexBIGServiceImpl;
import org.LexGrid.LexBIG.LexBIGService.CodedNodeGraph;
import org.LexGrid.LexBIG.LexBIGService.CodedNodeSet;
import org.LexGrid.LexBIG.LexBIGService.LexBIGService;
import org.LexGrid.LexBIG.LexBIGService.LexBIGServiceMetadata;
import org.LexGrid.LexBIG.LexBIGService.CodedNodeSet.PropertyType;
import org.LexGrid.LexBIG.LexBIGService.CodedNodeSet.SearchDesignationOption;
import org.LexGrid.LexBIG.Utility.Constructors;
import org.LexGrid.LexBIG.Utility.ConvenienceMethods;
import org.LexGrid.LexBIG.Utility.Iterators.ResolvedConceptReferencesIterator;
import org.LexGrid.LexBIG.Utility.LBConstants.MatchAlgorithms;
import org.LexGrid.codingSchemes.CodingScheme;
import org.LexGrid.codingSchemes.CodingSchemeVersion;
import org.LexGrid.codingSchemes.Versions;
import org.LexGrid.commonTypes.PropertyQualifier;
import org.LexGrid.commonTypes.Source;
import org.LexGrid.concepts.CodedEntry;
import org.LexGrid.concepts.Comment;
import org.LexGrid.concepts.Definition;
import org.LexGrid.concepts.Instruction;
import org.LexGrid.concepts.Presentation;
import org.LexGrid.naming.SupportedAssociation;
import org.LexGrid.naming.SupportedAssociationQualifier;
import org.LexGrid.naming.SupportedConceptStatus;
import org.LexGrid.naming.SupportedContext;
import org.LexGrid.naming.SupportedProperty;
import org.LexGrid.naming.SupportedPropertyQualifier;
import org.LexGrid.naming.SupportedRepresentationalForm;
import org.LexGrid.naming.SupportedSource;
import org.LexGrid.relations.Relations;
import org.LexGrid.versions.SystemRelease;
import org.apache.log4j.Category;
import org.apache.log4j.FileAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;

import org.LexGrid.LexBIG.DataModel.Core.types.CodingSchemeVersionStatus;

/**
 * DLBWrapper
 *
 * @since DLBExt 1.0
 * @author Northrop Grumman Information Technology
 * @version 1.0
 */
public class DLBWrapper {

// to be activated later
	//public EVSApplicationService lbSvc = null;

    protected org.LexGrid.LexBIG.LexBIGService.LexBIGService lbSvc = null;


	/** an instance of org.LexGrid.LexBIG.Utility.ConvenienceMethods
	 */

 	public org.LexGrid.LexBIG.Utility.ConvenienceMethods lbConvMethods = null;

 	//LexBIGCM lbConvMethods = null;

    /** an instance of CodingSchemeRenderingList
     */
    public CodingSchemeRenderingList csrl = null;

    /** A vector containing the name of all supported coding schemes.
     */
    protected Vector<CodingScheme> supportedCodingSchemes = null;

//TODO This is ridiculous
	// NCI Coding scheme name
	/** A static string "NCI_Thesaurus".
	 */
//	static final String NCI = "NCI_Thesaurus";

	// NCI Unique name for concept code
	//static final String NC_NAME = "Display_Name";
	/** A static string "KIND".
	 */
	static final String KIND = "KIND";
	/** A static string "CONCEPT_NAME".
	 */
	static final String CONCEPT_NAME = "CONCEPT_NAME";
	/** A constant String "Preferred_Name".
	 */
	static final String PREFERRED_NAME = "Preferred_Name";
	/** A constant String "hasSubtype".
	 */
	static final String IS_A = "hasSubtype";

	/** A constant String "is_a".
	 */
	static final String IS_A_OBO = "is_a";

	/** A constant String "is_a".
	 */
	static final String HAS_SUBCLASS = "hasSubclass";


	/** A constant String "NONE".
	 */
	static final String NONE = "NONE";

	/** A constant String "exactMatch".
	 */
	static final String EXACT_MATCH = "exactMatch";

    /** A constant equaling 1.
     */
    public static final int RESTRICT_TO_TARGET = 1;
	/** A constant equaling 2.
	 */
	public static final int RESTRICT_TO_SOURCE = 2;

    /** A constant String with a value "owl:AnnotationProperty"
     */
    public static String ROOT_ASSOCIATION = "owl:AnnotationProperty";
    /** A constant String with a value "owl:ObjectProperty"
     */
    public static String ROOT_ROLE = "owl:ObjectProperty";

    /** A constant String with a value "associations"
     */
    public static String ASSOCIATION_RELATION = "associations";
    /** A constant String with a value "roles"
     */
    public static String ROLE_RELATION = "roles";


    //static final String SEMANTIC_NET = "SemanticNet";

    /** A constant String with a value "urn:lsid:nlm.nih.gov:semnet"
     */
    static final String SEMANTIC_NET = "urn:lsid:nlm.nih.gov:semnet";

    /** A static vector containing all available semantic types.
     */
    static Vector<String> semanticType_vec;
    /** A static hashmap mapping semantic type to code.
     */
    static HashMap<String,String> semanticType_hashmap;
    static HashMap<String,String> semanticTypeName2Code_hashmap;

    /** A static variable.
     */
    static Category cat;

    // property type:
    /** A static String with a value "definition"
     */
    public static String TYPE_DEFINITION = "definition";
    /** A static String with a value "comment"
     */
    public static String TYPE_COMMENT = "comment";
    /** A static String with a value "property"
     */
    public static String TYPE_PROPERTY = "property";
    /** A static String with a value "instruction"
     */
    public static String TYPE_INSTRUCTION = "instruction";
    /** A static String with a value "presentation"
     */
    public static String TYPE_PRESENTATION = "presentation";

    private boolean isRemote = false;

    protected String version = null;
    protected String tag = null;

	/** active coding scheme name
	 */
	protected String codingScheme;// = NSID2codingSchemeName(namespaceId);

    protected CodingScheme defaultCodingScheme = null;

	final static String[] BROADENING_HIER_ASSOC = new String[] {"CHD", "hasSubtype"};
	final static String[] NARROWING_HIER_ASSOC = new String[] {"PAR", "isa", "branch_of", "part_of", "tributary_of"};

	final static String[] HIER_ASSOC = new String[] {"hasSubtype", "hasSubclass", "is_a", "isa", "PAR", "CHD", "branch_of", "part_of", "tributary_of"};

	/** An instance of LocalNameList.
	 */
	static LocalNameList noPropertiesList = new LocalNameList();
	{
		// temporarily commented out
		//ncNameList.addEntry(NONE);
	}

	/** A static LocalNameList.
	 */
	static LocalNameList kindList = new LocalNameList();
	{
		//ncNameList.addEntry(KIND);
		kindList.addEntry(KIND);
	}


    LexBIGServiceMetadata metadataService = null;

///////////////////////
//  Constructors
///////////////////////

	/** Creates a new DLBWrapper instance from org.LexGrid.service.Service and LexBIGService parameters
	 *
	 * @since DLBExt 1.0
	 */

    // Local
	public DLBWrapper()
    {
		try {
			this.lbSvc = new LexBIGServiceImpl();
			lbConvMethods = new ConvenienceMethods(this.lbSvc);
			metadataService = this.lbSvc.getServiceMetadata();
			csrl = this.lbSvc.getSupportedCodingSchemes();
			supportedCodingSchemes = getSupportedCodingSchemes();

			this.codingScheme = null;
			this.version = null;
			this.tag = null;

		} catch (Exception e) {
			e.printStackTrace();
		}
    }


    // Remote
    public DLBWrapper(EVSApplicationService lbSvc)
    {
		this.lbSvc = lbSvc;

		isRemote = true;

		try {
			lbConvMethods = new ConvenienceMethods(this.lbSvc);
			metadataService = this.lbSvc.getServiceMetadata();

			this.codingScheme = null;
			this.version = null;
			this.tag = null;

			csrl = this.lbSvc.getSupportedCodingSchemes();
			supportedCodingSchemes = getSupportedCodingSchemes();
		} catch (Exception e) {
			e.printStackTrace();
		}
    }



    public Logger getLogger()
    {
//		if (cat == null)
//		{
			// 012607 add appender programmatically
			Logger logger = Logger.getLogger(this.getClass());
			PatternLayout layout = new PatternLayout("%d [%t] %-5p %c - %m%n");

			FileAppender appender = null;
			try {
				appender = new FileAppender(layout,"c:/logfile",true);
			}catch(Exception e){}

			Logger.getRootLogger().addAppender(appender);
			logger.setLevel(Level.WARN);

//			Logger.getLogger("DLBWrapper");
//			cat = Category.getInstance("DLBWrapper");
//		}
		return logger;
	}


///////////////////////
//  Utility
///////////////////////

	/** Converts an array of objects to a vector of objects
	 *
	 * @since DLBExt 1.0
	 * @param objects an array of objects
	 * @return a vector of objects; return null, if connection fails or an
	 *     exception is thrown
	 */
	protected Vector arrayToVector(Object[] objects)
	{
		Vector v = new Vector();
		if (objects == null)
			return v;

		for (int i=0; i<objects.length; i++)
			v.add(objects[i]);
		return v;
	}

	/** Converts a HashSet to a Vector
	 *
	 * @param hset a HashSet instance
	 * @return a Vector instance
	 */
	protected Vector<String> hashSet2Vector(HashSet hset)
	{
	    Vector<String> v = new Vector<String>();
	    Iterator it = hset.iterator();
	    while (it.hasNext())
		{
		    v.add((String) it.next());
		}
		return v;
	}

	/** Converts a LocalNameList instance to a Vector
	 *
	 * @since DLBExt 1.0
	 * @param v a Vector
	 * @return a LocalNameList
	 */
	public LocalNameList vector2LocalNameList(Vector v)
	{
	    if (v == null) return null;
	    LocalNameList list = new LocalNameList();
		for (int i=0; i<v.size(); i++)
		{
		    String vEntry = (String) v.elementAt(i);
	        list.addEntry(vEntry);
		}
        return list;
	}


	/** Converts a LocalNameList instance to a HashSet
	 *
	 * @since DLBExt 1.0
	 * @param list an instance of LocalNameList
	 * @return a LocalNameList
	 */
	public HashSet<String> LocalNameList2HashSet(LocalNameList list)
	{
	    if (list == null) return null;
	    HashSet<String> hset = new HashSet<String>();
		java.lang.String[] entries = list.getEntry();


		for (int i=0; i<entries.length; i++)
		{
		    String vEntry = (String) entries[i];
	        if (!hset.contains(vEntry))
			{
                hset.add(vEntry);
			}
		}
        return hset;
	}





	/** Creates a ConceptReference instance corresponding to a given coding scheme name and a concept code.
	 *
	 * @since DLBExt 1.0
	 * @param codingSchemeName a coding scheme name
	 * @param code a concept code
	 * @return a ConceptReference instance corresponding to a given coding scheme
	 *      name and a concept code.
	 */

    public static ConceptReference createConceptReference(String code, String codingSchemeName)
    {
        ConceptReference cr = new ConceptReference();
        cr.setCodingScheme(codingSchemeName);
        cr.setConceptCode(code);
        return cr;
    }


	/** Creates a CodingSchemeVersionOrTag instance for a given version and a tag
	 *
	 * @since DLBExt 1.0
	 * @param version coding scheme version
	 * @param tag coding scheme tag
	 * @return a CodingSchemeVersionOrTag instance for a given version and a tag
	 */
	public CodingSchemeVersionOrTag createCodingSchemeVersionOrTag(
	                                     String version, String tag)
    {
        CodingSchemeVersionOrTag versionOrTag = new CodingSchemeVersionOrTag();
        if (version != null && version.compareTo("") != 0) versionOrTag.setVersion(version);
		if (tag != null && tag.compareTo("") != 0) versionOrTag.setTag(tag);
		return versionOrTag;
    }


	/** Creates a NameAndValue instance for a given name and value pair.
	 *
	 * @since DLBExt 1.0
	 * @param name a name
	 * @param value a value
	 * @return a NameAndValue instance for a given name and value pair
	 */
	public NameAndValue createNameAndValue(String name, String value)
	{
        NameAndValue nv = new NameAndValue();
		nv.setName(name);
		nv.setContent(value);
		return nv;
	}




///////////////////////
//  Meta Data
///////////////////////

    /*
     * Discover Vocabularies (and Versions)
     */

    /** Gets all supported coding schemes
     *
     * @since DLBExt 1.0
     * @return a Vector containing all supported CodingScheme instances
     */
    public Vector<CodingScheme> getSupportedCodingSchemes()
	{
		if (supportedCodingSchemes != null)
		{
		    return supportedCodingSchemes;
		}

		Vector<CodingScheme> v = new Vector<CodingScheme>();

        try {
			if (lbSvc == null)
			{
			   getLogger().error("ERROR: lbSvc is null.");
			   return null;
			}
			CodingSchemeRenderingList csrl = lbSvc.getSupportedCodingSchemes();
			CodingSchemeRendering[] csrs = csrl.getCodingSchemeRendering();
			for (int i=0; i<csrs.length; i++)
			{
				CodingSchemeRendering csr = csrs[i];
				CodingSchemeSummary css = csr.getCodingSchemeSummary();
				String formalname = css.getFormalName();
//				String localname = css.getLocalName();
				String representsVersion = css.getRepresentsVersion();
				CodingSchemeVersionOrTag vt = new CodingSchemeVersionOrTag();
				vt.setVersion(representsVersion);
				CodingScheme scheme = lbSvc.resolveCodingScheme(formalname, vt);
				v.add(scheme);

			}
	    } catch (Exception e) {
			e.printStackTrace();
			return null;
		}

		return v;
	}


	/** Gets all supported coding scheme names
	 *
	 * @since DLBExt 1.0
	 * @return a Vector containing all supported coding scheme names
	 */
	public Vector<String> getSupportedCodingSchemeNames()
	{
		Vector<String> v = new Vector<String>();
        CodingSchemeRendering[] csrs = csrl.getCodingSchemeRendering();
		for (int i=0; i<csrs.length; i++)
		{
		    CodingSchemeRendering csr = csrs[i];
		    CodingSchemeSummary css = csr.getCodingSchemeSummary();
			v.add(css.getFormalName());
		}
		return v;
	}

    /** Gets CodingScheme instance for a given coding scheme name.
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @return a CodingScheme instance for a given coding scheme name
     */
    public CodingScheme getCodingScheme(String codingScheme)
	{
		/*
		if (!Name2CodingSchemeMap.containsKey(codingScheme)) return null;
		return (CodingScheme) Name2CodingSchemeMap.get(codingScheme);
		*/
		return getCodingScheme(codingScheme, version, tag);
	}


    /** Gets CodingScheme instance for a given coding scheme name and version.
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param version a coding scheme version
     * @return a CodingScheme instance for a given coding scheme name and
     *     version.
     */
    public CodingScheme getCodingScheme(String codingScheme, String version)
	{
		/*
		if (!Name2CodingSchemeMap.containsKey(codingScheme)) return null;
		return (CodingScheme) Name2CodingSchemeMap.get(codingScheme);
		*/
		return getCodingScheme(codingScheme, version, tag);
	}


    /** Gets CodingScheme instance for given coding scheme name, version, and tag
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param version a coding scheme version
     * @param tag a coding scheme tag
     * @return a CodingScheme instance for given coding scheme name, version,
     *     and tag
     */
    public CodingScheme getCodingScheme(String codingScheme, String version, String tag)
	{
		CodingSchemeVersionOrTag vt = null;
		if (version != null || tag != null)
		{
			vt = createCodingSchemeVersionOrTag(version, tag);
		}
		CodingScheme scheme = null;
		try {
			scheme = lbSvc.resolveCodingScheme(codingScheme, vt);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return scheme;
	}


    /** Gets all supported search techniques
     *
     * @since DLBExt 1.0
     * @return a Vector containing all supported search techniques
     */
    public Vector<String> getSupportedSearchTechniques()
	{
		Vector<String> v = new Vector<String>();
		try {

			//ExtensionDescriptionList list = lbSvc.getMatchAlgorithms();
			ModuleDescriptionList list = lbSvc.getMatchAlgorithms();

			if (list == null)
			{
				getLogger().warn("WARNING: DLBWrapper getSupportedSearchTechnique lbSvc.getIndexExtensions returns null.");
			    return v;
			}

			//ExtensionDescription[] eda = list.getExtensionDescription();
			ModuleDescription[] eda = list.getModuleDescription();
			if (eda == null)
			{
				getLogger().warn("WARNING: DLBWrapper getSupportedSearchTechnique list.getIndexgetExtensionDescription returns null.");
			    return v;
			}

			for (int i=0; i<eda.length; i++)
			{
				//ExtensionDescription ed = (ExtensionDescription) eda[i];
				ModuleDescription ed = (ModuleDescription) eda[i];

				//v.add(ed.getExtensionName());
				v.add(ed.getName());
			}
	    } catch (Exception e) {
			e.printStackTrace();
			return null;
		}

		return v;
    }

    /** Gets the description of search technique for a given codeing scheme name.
     *
     * @since DLBExt 1.0
     * @param name search technique name
     * @return the description of search technique for a given codeing scheme
     *     name
     */
    public String getSearchTechniqueDescription(String name)
	{
		try {
			//ExtensionDescriptionList list = lbSvc.getMatchAlgorithms(); //lbSvc.getIndexExtensions();
			ModuleDescriptionList list = lbSvc.getMatchAlgorithms();

			if (list == null)
			{
				getLogger().warn("WARNING: DLBWrapper getSearchTechniqueDescription lbSvc.getMatchAlgorithms returns null.");
			    return null;
			}

			//ExtensionDescription[] eda = list.getExtensionDescription();
			ModuleDescription[] eda = list.getModuleDescription();
			if (eda == null)
			{
				getLogger().warn("WARNING: DLBWrapper getSearchTechniqueDescription list.getIndexgetExtensionDescription returns null.");
			    return null;
			}

			for (int i=0; i<eda.length; i++)
			{
				//ExtensionDescription ed = (ExtensionDescription) eda[i];
				ModuleDescription ed = (ModuleDescription) eda[i];
				//if (ed.getExtensionName().compareTo(name) == 0)
				if (ed.getName().compareTo(name) == 0)
				{
					//String desc = ed.getExtensionDescription();
					String desc = ed.getDescription();
					return desc;
				}
			}
			getLogger().error("Algorithm " + name + " not found.");
	    } catch (Exception e) {
			e.printStackTrace();
		}

		return null;
    }


    /*
     * Discover Vocabulary Versions
     */

	/** Gets available coding scheme versions for a given CodingScheme
	 *
	 * @since DLBExt 1.0
	 * @param codingSchemeName
	 * @return a Vector containing all available coding scheme versions for a
	 *     given CodingScheme
	 */
 	public Vector<String> getAvailableCodingSchemeVersions(String codingSchemeName) //CodingScheme cs)
	{
		Vector<String> v = new Vector<String>();
		try {
			CodingSchemeRenderingList csrl = lbSvc.getSupportedCodingSchemes();
			CodingSchemeRendering[] csrs = csrl.getCodingSchemeRendering();
			for (int i=0; i<csrs.length; i++)
			{
				CodingSchemeRendering csr = (CodingSchemeRendering) csrs[i];
				CodingSchemeSummary css = csr.getCodingSchemeSummary();
				String formalName = css.getFormalName();
				String localName = css.getLocalName();
				if (formalName.compareTo(codingSchemeName) == 0 || localName.compareTo(codingSchemeName) == 0)
				{
					v.add(css.getRepresentsVersion());
				}
			}
	    } catch (Exception e) {
			return new Vector<String>();
		}
		return v;
	}


	/** Gets copy right statement for a given CodingScheme
	 *
	 * @since DLBExt 1.0
	 * @param cs an instance of CodingScheme
	 * @return copy right statement for a given CodingScheme
	 */
	public String getCopyRight(CodingScheme cs)
	{
		 if (cs == null) return null;
		 return cs.getCopyright().getContent();
	}


	/** Gets copy right statement for a given coding scheme name
	 *
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @return copy right statement for a given coding scheme name
	 */
	public String getCopyRight(String codingScheme)
	{
         CodingScheme cs = getCodingScheme(codingScheme);
		 if (cs == null)
		 {
			 return null;
		 }

		 if (cs.getCopyright() == null)
		 {
			getLogger().warn("cs.getCopyright() returns null" );
		 }

		 return cs.getCopyright().getContent();
	}

	/** Gets sources of a given CodingScheme.
	 *
	 * @since DLBExt 1.0
	 * @param cs an instance of CodingScheme
	 * @return a Vector containing sources of a given CodingScheme
	 */
	public Vector getSources(CodingScheme cs)
	{
		 if (cs == null) return null;
		 return arrayToVector(cs.getSource());
    }


	/** Gets sources of a given CodingScheme.
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @return Gets sources of a given coding scheme
	 */
	public Vector getSources(String codingScheme)
	{
         CodingScheme cs = getCodingScheme(codingScheme);
		 if (cs == null) return null;
		 return arrayToVector(cs.getSource());
	}

	/** Gets versions of a given CodingScheme.
	 *
	 * @since DLBExt 1.0
	 * @param cs an instance of CodingScheme
	 * @return versions of a given CodingScheme
	 */
	public Vector<String> getVersions(CodingScheme cs)
    {
		 if (cs == null) return null;
		 Vector<String> v = new Vector<String>();
         Versions versions = cs.getVersions();
		 CodingSchemeVersion[] csvs = versions.getVersion();
		 for (int i=0; i<csvs.length; i++)
		 {
		      CodingSchemeVersion csv = (CodingSchemeVersion) csvs[i];
			  v.add(csv.getVersion());
		 }
		 return v;
	}

	/** Gets versions of a given CodingScheme.
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @return versions of a given CodingScheme
	 */
	public Vector<String> getVersions(String codingScheme)
    {
         CodingScheme cs = getCodingScheme(codingScheme);
		 if (cs == null) return null;
		 Vector<String> v = new Vector<String>();
         Versions versions = cs.getVersions();
		 CodingSchemeVersion[] csvs = versions.getVersion();
		 for (int i=0; i<csvs.length; i++)
		 {
		      CodingSchemeVersion csv = (CodingSchemeVersion) csvs[i];
			  v.add(csv.getVersion());
		 }
		 return v;
	}


	/** Gets the instance of CodingSchemeVersion given an instance of CodingScheme and a version number.
	 *
	 * @since DLBExt 1.0
	 * @param cs an instance of CodingScheme
	 * @param version a coding scheme version
	 * @return the instance of CodingSchemeVersion given an instance of
	 *     CodingScheme and a version number
	 */
	protected CodingSchemeVersion getCodingSchemeVersion(CodingScheme cs, String version)
    {
		 if (cs == null) return null;

		 // this method fails:
         Versions versions = cs.getVersions();

		 CodingSchemeVersion[] csvs = versions.getVersion();
		 for (int i=0; i<csvs.length; i++)
		 {
		      CodingSchemeVersion csv = (CodingSchemeVersion) csvs[i];
			  if (csv.getVersion().compareToIgnoreCase(version) == 0)
			  {
			      return csv;
			  }
		 }
		 return null;
	}

	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param version a coding scheme version
	 */
	protected CodingSchemeVersion getCodingSchemeVersion(String codingScheme, String version)
    {
         CodingScheme cs = getCodingScheme(codingScheme);
		 if (cs == null) return null;
         Versions versions = cs.getVersions();
		 CodingSchemeVersion[] csvs = versions.getVersion();
		 for (int i=0; i<csvs.length; i++)
		 {
		      CodingSchemeVersion csv = (CodingSchemeVersion) csvs[i];
			  if (csv.getVersion().compareToIgnoreCase(version) == 0)
			  {
			      return csv;
			  }
		 }
		 return null;
	}

	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param version a coding scheme version
	 */
	public java.util.Date getVersionDate(String codingScheme, String version)
    {
         CodingSchemeVersion csv = getCodingSchemeVersion(codingScheme, version);
		 if (csv == null)  return null;
		 return csv.getVersionDate();
	}

	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param version a coding scheme version
	 */
	public Boolean isVersionComplete(String codingScheme, String version)
    {
         CodingSchemeVersion csv = getCodingSchemeVersion(codingScheme, version);
		 return csv.getIsComplete();
	}


	/**
	 *
	 * @since DLBExt 1.0
	 * @param cs
	 */


	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param version a coding scheme version
	 */
	public Vector<String> getAvailableCodingSchemeTags(String codingScheme, String version)
	{
		 Vector<String> v = new Vector<String>();
		 try {
			 CodingSchemeRenderingList csrl = lbSvc.getSupportedCodingSchemes();
			 CodingSchemeRendering[] csra = csrl.getCodingSchemeRendering();
			 for (int i=0; i<csra.length; i++)
			 {
				CodingSchemeRendering csr = csra[i];
				CodingSchemeSummary css = csr.getCodingSchemeSummary();

				if ((css.getFormalName() != null && css.getFormalName().compareTo(codingScheme) == 0) ||
				    (css.getLocalName() != null && css.getLocalName().compareTo(codingScheme) == 0))
				{
					if (version == null)
					{
						RenderingDetail rd = csr.getRenderingDetail();
						CodingSchemeTagList cstl = rd.getVersionTags();
						java.lang.String[] tags = cstl.getTag();
						for (int j=0; j<tags.length; j++)
						{
							String tag = (String) tags[j];
							v.add(tag);
						}
					}
					else if (version.compareTo(css.getRepresentsVersion()) == 0)
					{
						RenderingDetail rd = csr.getRenderingDetail();
						CodingSchemeTagList cstl = rd.getVersionTags();
						java.lang.String[] tags = cstl.getTag();
						for (int j=0; j<tags.length; j++)
						{
							String tag = (String) tags[j];
							v.add(tag);
						}
					}
				}
			 }
	     } catch (Exception e) {
			 e.printStackTrace();
		 }
		 return v;
	 }

	public String getVocabularyVersionByTag(String codingScheme, String tag)
	{
		 if (codingScheme == null) return null;
		 try {
			 CodingSchemeRenderingList csrl = lbSvc.getSupportedCodingSchemes();
			 CodingSchemeRendering[] csra = csrl.getCodingSchemeRendering();
			 for (int i=0; i<csra.length; i++)
			 {
				CodingSchemeRendering csr = csra[i];
				CodingSchemeSummary css = csr.getCodingSchemeSummary();
				if (css.getFormalName().compareTo(codingScheme) == 0 || css.getLocalName().compareTo(codingScheme) == 0)
				{
					if (tag == null) return css.getRepresentsVersion();
					RenderingDetail rd = csr.getRenderingDetail();
					CodingSchemeTagList cstl = rd.getVersionTags();
					java.lang.String[] tags = cstl.getTag();
					for (int j=0; j<tags.length; j++)
					{
						String version_tag = (String) tags[j];
						if (version_tag.compareToIgnoreCase(tag) == 0)
						{
							return css.getRepresentsVersion();
						}
					}
				}
			 }
	     } catch (Exception e) {
			 e.printStackTrace();
		 }
		 getLogger().warn("Version corresponding to tag " + tag + " is not found " + " in " + codingScheme);
		 return null;
	 }

    /**
     *
     * @since DLBExt 1.0
     * @param cs
     */
    public Vector<SupportedProperty> getSupportedProperties(CodingScheme cs)
	{
		if (cs == null) return null;
        Vector<SupportedProperty> v = new Vector<SupportedProperty>();
	    SupportedProperty[] properties = cs.getMappings().getSupportedProperty();
		for (int i=0; i<properties.length; i++)
		{
		     SupportedProperty sp = (SupportedProperty) properties[i];
		     v.add(sp);
		}
        return v;
	}


    public Vector<SupportedSource> getSupportedSources(CodingScheme cs)
	{
		if (cs == null) return null;
        Vector<SupportedSource> v = new Vector<SupportedSource>();
	    SupportedSource[] sources = cs.getMappings().getSupportedSource();


		for (int i=0; i<sources.length; i++)
		{
		     SupportedSource src = (SupportedSource) sources[i];
		     v.add(src);
		}
        return v;
	}



    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param version a coding scheme version
     * @param tag
     */
    public Vector<SupportedProperty> getSupportedProperties(String codingScheme, String version, String tag)
	{
        CodingScheme cs = getCodingScheme(codingScheme, version, tag);
        if (cs == null) return null;
        return getSupportedProperties(cs);
	}


    /**
     *
     * @since DLBExt 1.0
     * @param cs
     */
    public Vector<String> getSupportedPropertyNames(CodingScheme cs)
	{
        Vector w = getSupportedProperties(cs);
		if (w == null) return null;

        Vector<String> v = new Vector<String>();
		for (int i=0; i<w.size(); i++)
		{
		     SupportedProperty sp = (SupportedProperty) w.elementAt(i);

		     if (sp == null)
		     {
				 getLogger().warn("DLBWrapper getSupportedPropertyNames SupportedProperty == null");
			 }
			 else if (sp.getLocalId() == null)
			 {
			     getLogger().warn("DLBWrapper getSupportedPropertyNames SupportedProperty.getLocalId() == null");
			 }


		     //v.add(sp.getContent());
		     v.add(sp.getLocalId());
		}
        return v;
	}


    /**
     *
     * @since DLBExt 1.0
     */
    public Vector<String> getSupportedPropertyNames()
	{
		Vector<String> v = new Vector<String>();
		Vector cs_vec = getSupportedCodingSchemes();
		for (int i=0; i<cs_vec.size(); i++)
		{
		    CodingScheme cs = (CodingScheme) cs_vec.elementAt(i);
		    Vector w = getSupportedProperties(cs);

			if (w == null)
			{
			   getLogger().warn("WARNING: getSupportedProperties(cs) returns null");
			   return null;
			}

			for (int j=0; j<w.size(); j++)
			{
				 SupportedProperty sp = (SupportedProperty) w.elementAt(j);
				 String propertyname = sp.getLocalId();

				 if (!v.contains(propertyname))
				 {
					 v.add(propertyname);
				 }
			}
		}

        return v;
	}




    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param version a coding scheme version
     * @param tag
     */
    public Vector<String> getSupportedPropertyNames(String codingScheme, String version, String tag)
	{
        CodingScheme cs = getCodingScheme(codingScheme, version, tag);
        if (cs == null) return null;
        return getSupportedPropertyNames(cs);
	}


    /**
     *
     * @since DLBExt 1.0
     * @param cs
     */
    public Vector<String> getSupportedRepresentationalForms(CodingScheme cs)
	{
		if (cs == null) return null;

        Vector<String> v = new Vector<String>();
	    SupportedRepresentationalForm[] forms = cs.getMappings().getSupportedRepresentationalForm();
		for (int i=0; i<forms.length; i++)
		{
		     SupportedRepresentationalForm sf = (SupportedRepresentationalForm) forms[i];
		     v.add(sf.getContent());
		}
        return v;
	}

    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param version a coding scheme version
     * @param tag
     */
    public Vector<String> getSupportedRepresentationalForms(String codingScheme, String version, String tag)
	{
        CodingScheme cs = getCodingScheme(codingScheme, version, tag);
        if (cs == null) return null;
        return getSupportedRepresentationalForms(cs);
	}

    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme
     */
    public Vector<String> getSupportedConceptStatus(String codingScheme)
	{
	    CodingScheme cs = getCodingScheme(codingScheme, version, tag);
	    return getSupportedConceptStatus(cs);
    }

    public Vector<String> getSupportedConceptStatus(CodingScheme cs)
	{
		if (cs == null) return null;

        Vector<String> v = new Vector<String>();
	    SupportedConceptStatus[] scs = cs.getMappings().getSupportedConceptStatus();
	    if (scs == null)
	    {
			getLogger().warn("WARNING: DLBWrapper getSupportedConceptStatus returns null.");
			return null;
		}
		else if (scs.length == 0)
		{
			getLogger().warn("WARNING: DLBWrapper getSupportedConceptStatus returns 0 entries.");
		}

		for (int i=0; i<scs.length; i++)
		{
		     SupportedConceptStatus status = (SupportedConceptStatus) scs[i];
		     //v.add(status.getContent());
		     v.add(status.getLocalId());
		}
		return v;
	}


    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param version a coding scheme version
     * @param tag
     */
    public Vector<String> getSupportedConceptStatus(String codingScheme, String version, String tag)
	{
        CodingScheme cs = getCodingScheme(codingScheme, version, tag);
        if (cs == null) return null;
        return getSupportedConceptStatus(cs);
	}




///////////////////////////////////////////////////////////////////////////////////

    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param version a coding scheme version
     * @param tag
     * @param code
     */
    protected ResolvedConceptReference resolveConceptReference(
		                            String codingScheme,
	                                String version,
		                            String tag,
									String code)
	{
        //return resolveConceptReference(codingScheme, version, null, code, null, 1, 1);
        return resolveConceptReference(codingScheme, version, tag, code, null, 1, 1);
	}


    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param version a coding scheme version
     * @param tag
     * @param code
     * @param properties
     * @param resolveAssociationDepth
     * @param resolveCodedEntryDepth
     */
    protected ResolvedConceptReference resolveConceptReference(
		                            String codingScheme,
	                                String version,
	                                String tag,
									String code,
									Vector properties,
									int resolveAssociationDepth,
                                    int resolveCodedEntryDepth)
	{
    	return resolveConceptReference(
		                            codingScheme,
	                                version,
									tag,
									code,
									properties,
									resolveAssociationDepth,
                                    resolveCodedEntryDepth,
                                    true);
	}


    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param version a coding scheme version
     * @param tag
     * @param code
     * @param properties
     * @param resolveAssociationDepth
     * @param resolveCodedEntryDepth
     * @param direction
     */
    protected ResolvedConceptReference resolveConceptReference(
		                            String codingScheme,
	                                String version,
									String tag,
									String code,
									Vector properties,
									int resolveAssociationDepth,
                                    int resolveCodedEntryDepth,
                                    boolean direction
									)
	{
    	return resolveConceptReference(
		                            codingScheme,
	                                version,
									tag,
									code,
									properties,
									resolveAssociationDepth,
                                    resolveCodedEntryDepth,
                                    direction,
                                    null);

	}



    protected ResolvedConceptReference resolveConceptReference(
		                            String codingScheme,
	                                String version,
									String tag,
									String code,
									Vector properties,
									int resolveAssociationDepth,
                                    int resolveCodedEntryDepth,
                                    boolean direction,
                                    String relation
									)
	{
        String relationName = relation;
        // 031407
        if (relation != null)
        {
			if (!hasRelation(codingScheme, version, tag, relation))
			{
				relationName = null;
			}
	    }

		CodingSchemeVersionOrTag tagOrVersion = createCodingSchemeVersionOrTag(version, tag);
		ResolvedConceptReference rcr = null;
		ResolvedConceptReferenceList rcrl = null;

		boolean b1 = true;
		boolean b2 = false;

		if (!direction)
		{
			b1 = false;
			b2 = true;
		}

		try {

			ConceptReference ref = createConceptReference(code, codingScheme);
			if (ref == null)
			{
				getLogger().warn("WARNING: createConceptReference(" + code + ") returns null.");
				return null;
			}

			rcrl = lbSvc.getNodeGraph(codingScheme, tagOrVersion, null)
					.resolveAsList(
						ref,
						b1,
						b2,
						1,
						1,
						null,//LexBIGCM.createLocalNameList(CONCEPT_NAME),
						null,
						null,
						1024);


			if (rcrl == null)
			{
				getLogger().error("lbSvc.getNodeGraph returns null");
				return null;
			}

			// Analyze the result ...
			if (rcrl.getResolvedConceptReferenceCount() > 0) {
				ResolvedConceptReference[] rcrs = rcrl.getResolvedConceptReference();
				if (rcrs == null)
				{
					getLogger().error("rcrl.getResolvedConceptReference returns null");
					return null;
				}
				if (rcrs.length == 0)
				{
					getLogger().error("rcrs.length == 0");
					return null;
				}
				rcr = rcrs[0];
			}

	    } catch (Exception e) {
			getLogger().error("ResolvedConceptReference threw exception.");
			//e.printStackTrace();
		}
        return rcr;
	}

	/*
	 * Test Membership in Vocabulary
	 */

	 /**
	  *
	  * @since DLBExt 1.0
	  * @param codingScheme a coding scheme name
	  * @param code
	  */

	 public boolean conceptExists(String codingScheme, String code)
     {
	     Boolean Bool_val = conceptExists(codingScheme, version, tag, true, code);

	     if (Bool_val == null || Bool_val.equals(Boolean.FALSE)) return false;
	     return true;
     }


	 protected Boolean conceptExists(String codingScheme, String version, String tag,
	                               boolean activeOnly, String code)
	 {
	 	 CodingSchemeVersionOrTag tagOrVersion = createCodingSchemeVersionOrTag(version, tag);
	 	 try {
			 CodedNodeSet cns = lbSvc.getCodingSchemeConcepts(codingScheme, tagOrVersion);//, activeOnly);
			 //ConceptReference cr = createConceptReference(codingScheme, code);

			 ConceptReference cr = createConceptReference(code, codingScheme);


			 return cns.isCodeInSet(cr);
		 } catch (Exception e) {
			 e.printStackTrace();
			 return null;
		 }
	 }


/*
 CodedNodeGraph restrictToAssociations(ConceptReferenceList association,
                                       ConceptReferenceList associationQualifiers)
 */


     protected NameAndValueList createNameAndValueList(String[] names, String[] values)
     {
		 NameAndValueList nvList = new NameAndValueList();
		 for (int i=0; i<names.length; i++)
		 {
			 NameAndValue nv = new NameAndValue();
			 nv.setName(names[i]);
			 if (values != null)
			 {
			 	nv.setContent(values[i]);
		     }
			 nvList.addNameAndValue(nv);
		 }
		 return nvList;
	}



	 /**
	  *
	  * @since DLBExt 1.0
	  * @param codingScheme a coding scheme name
	  * @param version a coding scheme version
	  * @param tag
	  * @param associationName
	  * @param associationQualiferName
	  * @param code
	  */
	 protected Boolean isCodeInGraph(String codingScheme, String version, String tag,
	                               String associationName, String associationQualiferName, String code)
	 {
		 CodingSchemeVersionOrTag tagOrVersion = createCodingSchemeVersionOrTag(version, tag);
	 	 try {
			 CodedNodeGraph cng = lbSvc.getNodeGraph(codingScheme, tagOrVersion, null);

			 NameAndValueList arl_asso =
				createNameAndValueList(
					new String[] {associationName}, null);

			 NameAndValueList arl_qualifier =
				createNameAndValueList(
					new String[] {associationQualiferName}, null);

//			 LocalNameList qualifiers = null;//new LocalNameList();

			 cng = cng.restrictToAssociations(arl_asso, arl_qualifier);
			 //ConceptReference targetCode = createConceptReference(codingScheme, code);

			 ConceptReference targetCode = createConceptReference(code, codingScheme);

			 return cng.isCodeInGraph(targetCode);
		 } catch (Exception e) {
			 e.printStackTrace();
			 return null;
		 }
	 }


    /*
     * Enumerate All Concepts
     */

	 /**
	  *
	  * @since DLBExt 1.0
	  * @param codingScheme a coding scheme name
	  * @param version a coding scheme version
	  * @param tag
	  * @param activeOnly
	  * @param sortproperties
	  * @param properties
	  * @param maxToReturn
	  */
	 protected Vector<CodedEntry> getAllConcepts(String codingScheme, String version, String tag, boolean activeOnly,
	                               Vector sortproperties, Vector properties, int maxToReturn
								   )
     {
	     Vector<CodedEntry> v = new Vector<CodedEntry>();
	 	 CodingSchemeVersionOrTag tagOrVersion = createCodingSchemeVersionOrTag(version, tag);
         try {
			 CodedNodeSet cns = lbSvc.getCodingSchemeConcepts(codingScheme, tagOrVersion);//, activeOnly);

	//		 LocalNameList sortByProperty = vector2LocalNameList(sortproperties);
			 LocalNameList restrictToProperties = vector2LocalNameList(properties);

// temporary fix
			// Sort by search engine recommendation & code ...
			SortOptionList sortCriteria =
			    Constructors.createSortOptionList(new String[]{"matchToQuery", "code"});


//			 ResolvedConceptReferenceList rcrl = cns.resolveToList(sortByProperty, restrictToProperties, maxToReturn);
			 ResolvedConceptReferenceList rcrl = cns.resolveToList(sortCriteria, restrictToProperties, null, maxToReturn);

			 ResolvedConceptReference[] rcrs = rcrl.getResolvedConceptReference();

			 for (int i=0; i<rcrs.length; i++)
			 {
				 ResolvedConceptReference rcr = rcrs[i];
				 CodedEntry ce = rcr.getReferencedEntry();
				 v.add(ce);
			 }
	     } catch (Exception e) {
			 e.printStackTrace();
			 return null;
		 }
		 return v;
     }





    /*
     * Enumerate Properties
	 * Retrieve Concept and Attributes by Code
     */

	 /**
	  *
	  * @since DLBExt 1.0
	  * @param ce
	  */
	 protected Vector<NameAndValue> getConceptProperties(CodedEntry ce)
     {
	    Vector<NameAndValue> v = new Vector<NameAndValue>();
        java.util.Enumeration enumeration = ce.enumerateComment();
		while (enumeration.hasMoreElements())
		{
		    Comment property = (Comment) enumeration.nextElement();
			v.add(createNameAndValue(property.getProperty(), property.getText().getContent()));
		}

        enumeration = ce.enumerateDefinition();
		while (enumeration.hasMoreElements())
		{
		    Definition property = (Definition) enumeration.nextElement();
			v.add(createNameAndValue(property.getProperty(), property.getText().getContent()));
		}

        enumeration = ce.enumerateInstruction();
		while (enumeration.hasMoreElements())
		{
		    Instruction property = (Instruction) enumeration.nextElement();
			v.add(createNameAndValue(property.getProperty(), property.getText().getContent()));
		}

        enumeration = ce.enumeratePresentation();
		while (enumeration.hasMoreElements())
		{
		    Presentation property = (Presentation) enumeration.nextElement();
			v.add(createNameAndValue(property.getProperty(), property.getText().getContent()));
		}

        enumeration = ce.enumerateConceptProperty();
		while (enumeration.hasMoreElements())
		{
		    org.LexGrid.commonTypes.Property property = (org.LexGrid.commonTypes.Property) enumeration.nextElement();
			v.add(createNameAndValue(property.getProperty(), property.getText().getContent()));

		}
		return v;
    }



    protected CodedEntry findCodedEntryWithNameMatching(String codingScheme,
                                                String version,
	                                            String name)
	{

	    Vector v = findCodedEntriesWithDesignationMatching(codingScheme,
 	                                            version,
 	                                            name,
 												false,
 	                                            new Vector(),
                                                null,
 												"exactMatch",
												1);
		if (v != null && v.size() > 0)
		{
			CodedEntry ce = (CodedEntry) v.elementAt(0);
			return ce;
		}
		return null;

    }


    /*
     * Map Synonyms to Preferred Names
     */

    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param version a coding scheme version
     * @param matchText
     * @param preferredOnly
     * @param sortByProperty_vec
     * @param maxToReturn
     * @param matchAlgorithm
     */
    protected Vector<String> mapSynonymsToPreferredNames(String codingScheme,
                                                String version,
	                                            String matchText,
												boolean preferredOnly,
	                                            Vector sortByProperty_vec,
	                                            int maxToReturn,
												String matchAlgorithm)
    {
	    Vector<String> restrictToProperties_vec = new Vector<String>();
		restrictToProperties_vec.add("Synonym");


	    Vector<CodedEntry> v = findCodedEntriesWithDesignationMatching(codingScheme,
	                                            version,
	                                            matchText,
												preferredOnly,
	                                            sortByProperty_vec,
                                                restrictToProperties_vec,
												matchAlgorithm,
												maxToReturn);

        // Get perferred names for each element
		String propertyName = PREFERRED_NAME;
		Vector<String> names = new Vector<String>();
		for (int i=0; i<v.size(); i++)
		{
//             ResolvedConceptReference rcr = (ResolvedConceptReference) v.elementAt(i);
			 Vector<String> u = getPropertyValues(v.elementAt(i), propertyName);
			 for (int j=0; j<u.size(); j++)
			 {
			     names.add((String) u.elementAt(j));
			 }
		}
		return names;
	}




    /*
     * Test for Current/Obsolete Concept
     */


	 /**
	  *
	  * @since DLBExt 1.0
	  * @param codingScheme a coding scheme name
	  * @param version a coding scheme version
	  * @param tag
	  * @param code
	  */
	 public String getConceptStatus(String codingScheme, String version, String tag, String code)
	 {
         CodedEntry ce = getCodedEntry(codingScheme,
	                                version,
									tag,
									code);
         if (ce == null)
		 {
			  getLogger().warn("Concept with code " + code + " not found in " + codingScheme
			      + ".");
			  return null;
		 }
		 return ce.getConceptStatus();
	 }

     /**
      *
      * @since DLBExt 1.0
      * @param codingScheme a coding scheme name
      * @param code
      */
     public Boolean isConceptActive(String codingScheme,
									 String code)
	 {
          return isConceptActive(codingScheme, version, tag, code);
	 }



     /**
      *
      * @since DLBExt 1.0
      * @param codingScheme a coding scheme name
      * @param version a coding scheme version
      * @param tag
      * @param code
      */
     public Boolean isConceptActive(String codingScheme,
	                                 String version,
									 String tag,
									 String code)

     {
        CodedEntry ce = getCodedEntry(codingScheme,
	                                version,
									tag,
									code);
         if (ce == null)
		 {
			  getLogger().warn("Concept with code " + code + " not found in " + codingScheme
			      + ".");
			  return null;
		 }
         return new Boolean(ce.getIsActive());
	 }




     public Boolean isCodeRetired(String codingSchemeName, String code)
     {
		 try {
			 Vector v = getConceptEditActionDates(codingSchemeName, code, toActionIndex("retire"));
			 if (v != null && v.size() > 0) return Boolean.TRUE;


	     } catch (Exception e) {
			 return null;
		 }
		 return Boolean.FALSE;
	 }


    /*
     * Enumerate Kinds
     */

	 /**
	  *
	  * @since DLBExt 1.0
	  * @param codingScheme a coding scheme name
	  * @param version a coding scheme version
	  * @param tag
	  * @param activeOnly
	  * @param kind
	  * @param maxToReturn
	  */
	 protected Vector<CodedEntry> getConceptsByKind(String codingScheme, String version, String tag,
	                                  boolean activeOnly, String kind, int maxToReturn)
     {
		 /*
		 CodedNodeSet 	restrictToMatchingProperties(LocalNameList propertyNames,
		                                             CodedNodeSet.PropertyType[] propertyTypes,
		                                             java.lang.String matchText,
		                                             java.lang.String matchAlgorithm,
		                                             java.lang.String language)
		 */

		 Vector<CodedEntry> v = new Vector<CodedEntry>();
		 try {
			 CodedNodeSet cns =	lbSvc.getCodingSchemeConcepts(codingScheme, createCodingSchemeVersionOrTag(version, tag));//, activeOnly);
			 cns = cns.restrictToMatchingProperties(kindList, null, kind, "ExactMatch", null);
			 ResolvedConceptReferenceList rcrl = cns.resolveToList(null,kindList,null,maxToReturn);
			 ResolvedConceptReference[] rcrs = rcrl.getResolvedConceptReference();

			 for (int i=0; i<rcrs.length; i++)
			 {
				 ResolvedConceptReference rcr = rcrs[i];
				 CodedEntry ce = rcr.getReferencedEntry();
				 v.add(ce);
			 }
	     }
	     catch (Exception e) {
			 e.printStackTrace();
			 return null;
		 }
		 return v;
     }


///////////////////////
//  Relationships
///////////////////////

    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param version a coding scheme version
     */


    public Vector<String> getRelationNames(String codingScheme, String version)
	{
	    Vector<String> v = new Vector<String>();
		CodingScheme cs = getCodingScheme(codingScheme, version, null);

		Relations[] rels = cs.getRelations();
		for (int i=0; i<rels.length; i++)
		{
		    Relations rel = (Relations) rels[i];
			v.add(rel.getDc());
		}
		return v;
	}


    public String getInverseName(String codingScheme, String version, String relationName, String associationName)
	{
		CodingScheme cs = getCodingScheme(codingScheme, version, null);

		Relations[] rels = cs.getRelations();
		Relations relation = rels[0];
		if (relationName != null)
		{
			for (int i=0; i<rels.length; i++)
			{
				Relations rel = (Relations) rels[i];
				if (rel.getDc().compareTo(relationName) == 0)
				{
					relation = rel;
					break;
				}
			}
		}

		org.LexGrid.relations.Association[] assos = relation.getAssociation();
		for (int j=0; j<assos.length; j++)
		{
			org.LexGrid.relations.Association asso = (org.LexGrid.relations.Association) assos[j];
		    if (asso.getForwardName().compareTo(associationName) == 0)
		    {
				return asso.getReverseName();
			}
			else if (asso.getReverseName() != null)
			{
				if (asso.getReverseName().compareTo(associationName) == 0)
				{
					return asso.getForwardName();
				}
			}
		}
		return null;
	}


    public Vector<String> getAssociationNamesByRelation(String codingScheme, String version, String relationName)
	{
		CodingSchemeVersionOrTag vt = new CodingSchemeVersionOrTag();
		vt.setVersion(version);
		try
		{
			CodingScheme scheme = lbSvc.resolveCodingScheme(codingScheme, vt);
			return getAssociationNamesByRelation(scheme, relationName);
    	} catch (Exception e) {
			return null;
		}
	}

    public boolean hasRelation(String codingSchemeName, String relationName)
    {
		return hasRelation(codingSchemeName, null, null, relationName);
	}

    public boolean hasRelation(String codingSchemeName, String version, String tag, String relationName)
    {
		try
		{
			CodingSchemeVersionOrTag vt = createCodingSchemeVersionOrTag(version, tag);
			CodingScheme scheme = lbSvc.resolveCodingScheme(codingSchemeName, vt);
			return hasRelation(scheme, relationName);
    	} catch (Exception e) {
			return false;
		}
	}


    public boolean hasRelation(CodingScheme cs, String relationName)
    {
		Relations[] rels = cs.getRelations();
		for (int i=0; i<rels.length; i++)
		{
		    Relations rel = (Relations) rels[i];
		    String name = rel.getDc();
		    if (name.compareTo(relationName) == 0)
		    {
				return true;
			}
		}
		return false;
	}


   private Vector getAssociationNames(String codingSchemeName)
    {
	    return getAssociationNames(codingSchemeName, null, null);
    }


    private Vector getAssociationNames(String codingSchemeName, String version, String tag)
    {
		Vector v = new Vector();
		try
		{
			CodingSchemeVersionOrTag vt = createCodingSchemeVersionOrTag(version, tag);
			CodingScheme scheme = lbSvc.resolveCodingScheme(codingSchemeName, vt);
			return getAssociationNames(scheme);
    	} catch (Exception e) {
			return v;
		}
	}


	private Vector<String> getAssociationNames(CodingScheme cs)
	{
		Vector<String> v = new Vector<String>();
		Relations[] relations = cs.getRelations();
		for (int i=0; i<relations.length; i++)
		{
			Relations r = relations[i];
			org.LexGrid.relations.Association[] associations = r.getAssociation();
			for (int j=0; j<associations.length; j++)
			{
				org.LexGrid.relations.Association association = associations[j];
				String name = association.getAssociation();
				v.add(name);
		    }
		}
		return v;
	}


    public Vector<String> getAssociationNamesByRelation(CodingScheme cs, String relation)
    {
		//031507
	    String relationName = relation;
        if (relation != null)
        {
			if (!hasRelation(cs, relation))
			{
				relationName = null;//"Relations";//null;
			}
	    }

		Vector<String> v = new Vector<String>();
	    if (relation == null)
	    {
			// return all association names
			return getAssociationNames(cs);
		}


		Relations[] rels = cs.getRelations();
		/*
		if (relationName == null || relationName.compareTo("") == 0)
		{
		    Relations rel = (Relations) rels[0];
			org.LexGrid.relations.Association[] assos = rel.getAssociation();
			for (int j=0; j<assos.length; j++)
			{
				org.LexGrid.relations.Association asso = (org.LexGrid.relations.Association) assos[j];
				//v.add(asso.getForwardName());
				v.add(asso.getAssociation());
			}
			return v;
		}
		*/

		for (int i=0; i<rels.length; i++)
		{
		    Relations rel = (Relations) rels[i];
		    String name = rel.getDc();
		    if (name.compareTo(relationName) == 0)
		    {
                org.LexGrid.relations.Association[] assos = rel.getAssociation();
				for (int j=0; j<assos.length; j++)
				{
					org.LexGrid.relations.Association asso = (org.LexGrid.relations.Association) assos[j];
					//v.add(asso.getForwardName());
					v.add(asso.getAssociation());
				}
		    }
		}
		return v;
	}


    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param version a coding scheme version
     */
    protected String getNativeRelationName(String codingScheme, String version)
	{
//	    Vector v = new Vector();
		CodingScheme cs = getCodingScheme(codingScheme, version, null);
		Relations[] rels = cs.getRelations();
		for (int i=0; i<rels.length; i++)
		{
		    Relations rel = (Relations) rels[i];
			if (rel.getIsNative()) return rel.getDc();
		}
		return "";
	}



    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param code
     * @param direction
     * @param associationName
     */
    public Boolean hasAssociatedConcepts(String codingScheme,
	                                     String code,
										 boolean direction,
										 String associationName)
	{
		Vector<String> v = findRelatedCodes(codingScheme, version, tag,
	                                code,
									associationName,
									direction);

		if (v == null) return null;
		if (v.size() == 0) return Boolean.FALSE;
		return Boolean.TRUE;
	}

    /*
     * Traverse Graph via Role Links (by recursion using getAssociatedConcepts)
     */

    /*
     * Enumerate Source Concepts for Relation and Target
     */

    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param code
     * @param associationName
     */
    public Vector<CodedEntry> getAssociationSources(String codingScheme,
	                                    String code,
										String associationName)
	{
         return getAssociatedCodedEntries(codingScheme,
	                                  code,
									  false,
									  associationName);
	}


    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param version a coding scheme version
     * @param tag
     * @param code
     * @param properties
     * @param associationName
     */
    public Vector getAssociationSources(String codingScheme,
									   String version,
									   String tag,
	                                   String code,
									   Vector properties,
									   String associationName)
    {
        return getAssociatedCodedEntries(codingScheme,
									   version,
									   tag,
	                                   code,
									   properties,
									   true,
									   associationName);
	}


    /*
     * Enumerate Target Concepts for Relation and Source
     */

    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param code
     * @param associationName
     */
    public Vector getAssociationTargets(String codingScheme,
	                                    String code,
										String associationName)
	{
         return getAssociatedCodedEntries(codingScheme,
	                                  code,
									  true,
									  associationName);
	}


    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param version a coding scheme version
     * @param tag
     * @param code
     * @param properties
     * @param associationName
     */
    public Vector getAssociationTargets(String codingScheme,
									   String version,
									   String tag,
	                                   String code,
									   Vector properties,
									   String associationName)
    {
        return getAssociatedCodedEntries(codingScheme,
									   version,
									   tag,
	                                   code,
									   properties,
									   false,
									   associationName);
	}

////////////////////////////////////////////////////////////////////////////////////

    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param code
     * @param associationName
     */
    public Vector<CodedEntry> getAssociatedCodedEntries(String codingScheme,
	                                    String code,
										String associationName)
    {
         return getAssociatedCodedEntries(codingScheme,
	                                     code,
										 true,
										 associationName);
	}

    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param code
     * @param direction
     * @param associationName
     */
    public Vector<CodedEntry> getAssociatedCodedEntries(String codingScheme,
	                                    String code,
										boolean direction,
										String associationName)
	{

         return getAssociatedCodedEntries(codingScheme,
										 version,
										 tag,
	                                     code,
										 null,
										 direction,
										 associationName);
	}


    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param version a coding scheme version
     * @param tag
     * @param code
     * @param properties
     * @param direction
     * @param associationName
     */
    public Vector<CodedEntry> getAssociatedCodedEntries(String codingScheme,
										 String version,
										 String tag,
	                                     String code,
										 Vector properties,
										 boolean direction,
										 String associationName)
    {
   		return getAssociatedCodedEntries(codingScheme,
										 version,
										 tag,
	                                     code,
										 properties,
										 direction,
										 associationName, 1, 1);
    }




////Associated Concepts////////////////////////////////////////////////////////////////////////////////////
	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param version a coding scheme version
	 * @param code
	 * @param relation
	 */
	public Vector getAssociatedConcepts(String codingScheme,
	                                       String version,
	                                       String tag,
	                                       String code,
	                                       String relation)
    {
	     return getAssociatedConcepts(codingScheme,
	                                  version,
	                                  tag,
	                                  code,
	                                  relation,
	                                  true);
	}


	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param version a coding scheme version
	 * @param code
	 * @param relation
	 * @param direction
	 */
	public Vector<AssociatedConcept> getAssociatedConcepts(String codingScheme,
	                                       String version,
	                                       String tag,
	                                       String code,
	                                       String relation,
	                                       boolean direction)
    {
		Vector<AssociatedConcept> v = new Vector<AssociatedConcept>();
		CodingSchemeVersionOrTag tagOrVersion = createCodingSchemeVersionOrTag(version, tag);
        try {

			// Perform the query ...
			boolean b1 = true;
			boolean b2 = false;

			if (!direction)
			{
				b1 = false;
				b2 = true;
			}

            ResolvedConceptReferenceList matches = null;
            if (relation == null)
            {
				matches =
					lbSvc.getNodeGraph(codingScheme, tagOrVersion, null)
						.resolveAsList(
							createConceptReference(code, codingScheme),
							b1, b2, 1, 1,
							null,//LexBIGCM.createLocalNameList(CONCEPT_NAME),
							null,
							null, 1024);
			}
			else
			{
			    NameAndValueList rel_list =
				    createNameAndValueList(new String[] {relation}, null);

			    CodedNodeGraph cng = lbSvc.getNodeGraph(codingScheme, tagOrVersion, null);
				cng = cng.restrictToAssociations(
						rel_list,
						null);
				matches = cng
					.resolveAsList(
						createConceptReference(code, codingScheme),
						b1, b2, 1, 1,
						null,//LexBIGCM.createLocalNameList(CONCEPT_NAME),
						null,
						null, 1024);

			}

			// Analyze the result ...

			if (matches.getResolvedConceptReferenceCount() > 0) {
				ResolvedConceptReference ref =
					(ResolvedConceptReference) matches.enumerateResolvedConceptReference().nextElement();

                AssociationList assolist = null;
                if (direction)
                {
				    assolist = ref.getSourceOf();
			    }
			    else
			    {
					assolist = ref.getTargetOf();
				}
				if (assolist == null)
				{
					return v;
				}

				Association[] associations = assolist.getAssociation();
				for (int i = 0; i < associations.length; i++) {
					Association assoc = associations[i];
					AssociatedConcept[] acl = assoc.getAssociatedConcepts().getAssociatedConcept();
					for (int j = 0; j < acl.length; j++) {
						AssociatedConcept ac = acl[j];
						v.add(ac);
					}
				}
			}

	    } catch (Exception e) {
			e.printStackTrace();
			return null;
		}

		return v;

	}


	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param version a coding scheme version
	 * @param code
	 * @param direction
	 */
	 // including roles and associations
	public Vector<String> getAssociationNames(String codingScheme,
	                                       String version,
	                                       String tag,
	                                       String code,
	                                       boolean direction)
    {
         HashSet<String> hset = new HashSet<String>();
         Vector<String> v = new Vector<String>();
         Vector<ConceptAssociation> w = getConceptAssociations(
		                                   codingScheme,
                                           version,
                                           tag,
                                           code,
                                           direction);

         String is_a = getHasSubtype(codingScheme);
         /*
         for (int i=0; i<w.size(); i++)
         {
			  NameAndValue nv = (NameAndValue) w.elementAt(i);
			  if (nv.getName().compareToIgnoreCase(is_a) != 0)
			  {
			      if (!hset.contains(nv.getName()));
			      {
					  hset.add(nv.getName());
					  v.add(nv.getName());
				  }
              }
		 }
		 */
         for (int i=0; i<w.size(); i++)
         {
			  gov.nih.nci.lexbig.ext.ConceptAssociation nv = (gov.nih.nci.lexbig.ext.ConceptAssociation) w.elementAt(i);
			  if (nv.getName().compareToIgnoreCase(is_a) != 0)
			  {
			      if (!hset.contains(nv.getName()));
			      {
					  hset.add(nv.getName());
					  v.add(nv.getName());
				  }
              }
		 }

		 return v;
	}


    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param version a coding scheme version
     * @param tag
     * @param code
     * @param properties
     * @param direction
     * @param associationName
     */
    public Boolean hasAssociatedConcepts(String codingScheme,
										 String version,
										 String tag,
	                                     String code,
										 Vector properties,
										 boolean direction,
										 String associationName
										 )
    {
//	    Vector v = new Vector();
	    int resolveAssociationDepth = 1; // distance
		int resolveCodedEntryDepth = 1;
        ResolvedConceptReference rcr = resolveConceptReference(
			                             codingScheme,
		                                 version,
		                                 tag,
		                                 code,
		                                 properties,
                                         resolveAssociationDepth,
										 resolveCodedEntryDepth);

        if (rcr == null)
        {
			return null;
		}
        AssociationList ass0list = null;
		if (direction) // associations of which this code is source
		{
		   ass0list = rcr.getSourceOf();
		}
		else // associations of which this code is target
		{
           ass0list = rcr.getTargetOf();
		}

		if (ass0list == null)
		{
			getLogger().warn("WARNING: ass0list == null. ");
			return Boolean.FALSE;
		}

        Association[] assoArray;
        try {
			assoArray = ass0list.getAssociation();
			if (assoArray == null)
			{
				getLogger().warn("hasAssociatedConcepts assoArray == null. -- return FALSE");
				return Boolean.FALSE;
			}
	    } catch (Exception e) {
			getLogger().error("hasAssociatedConcepts Exception thrown.");
			return Boolean.FALSE;
		}
		// Check association type (is-a, part-of, etc)
		/* 050707
        for (int k=0; k<assoArray.length; k++)
		{
		    String name = assoArray[k].getAssociationName();
			if (name.compareTo(associationName) == 0)
			{
			    return Boolean.TRUE;
			}
		}
		*/

        if (associationName != null)
        {
			for (int k=0; k<assoArray.length; k++)
			{
				String name = assoArray[k].getAssociationName();
				if (name.compareTo(associationName) == 0)
				{
					return Boolean.TRUE;
				}
			}
     	}
     	else
     	{
			if (assoArray.length > 0) return Boolean.TRUE;
		}

        return Boolean.FALSE;
    }


    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param code1
     * @param code2
     * @param direction
     * @param associationName
     */
    public Boolean areAssociatedConcepts(String codingScheme,
	                                     String code1,
										 String code2,
										 boolean direction,
										 String associationName)
	{
		return areAssociatedConcepts(codingScheme,
									version,
									tag,
	                                code1,
									code2,
									direction,
									associationName);
	}



    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param version a coding scheme version
     * @param tag
     * @param code1
     * @param code2
     * @param direction
     * @param associationName
     */
    public Boolean areAssociatedConcepts(String codingScheme,
										 String version,
										 String tag,
	                                     String code1,
										 String code2,
										 boolean direction,
										 String associationName)
    {
         try {
			 Vector assos = getAssociatedConcepts(codingScheme,
											   version,
											   tag,
											   code1,
											   associationName,
											   direction);

			 if (assos == null) return Boolean.FALSE;
			 for (int i=0; i<assos.size(); i++)
			 {
				 AssociatedConcept ac = (AssociatedConcept) assos.elementAt(i);
				 if (ac == null)
				 {
					 getLogger().warn("WARNING: AssociatedConcept is null");
				 }
				 else if (ac.getConceptCode() == null)
				 {
					 getLogger().warn("WARNING: ac.getConceptCode() == null");
				 }
				 else if (ac.getConceptCode().compareTo(code2) == 0)
				 {
					 return Boolean.TRUE;
				 }
			 }
	     } catch (Exception e) {
			 e.printStackTrace();
		 }
		 return Boolean.FALSE;
	 }



    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param version a coding scheme version
     * @param tag
     * @param code
     */
    public Boolean isCodeInGraph(String codingScheme,
								 String version,
								 String tag,
								 String code)
	{

		CodingSchemeVersionOrTag tagOrVersion = createCodingSchemeVersionOrTag(version, tag);
		try {
			CodedNodeGraph cng = lbSvc.getNodeGraph(codingScheme, tagOrVersion, null);
			//ConceptReference ref = createConceptReference(codingScheme, code);

			ConceptReference ref = createConceptReference(code, codingScheme);
			if (ref == null) return Boolean.FALSE;
            try {
				return cng.isCodeInGraph(ref);
			} catch (Exception ex) {
				return Boolean.FALSE;
			}
	    } catch (Exception e) {
			e.printStackTrace();
			getLogger().warn("DLBWrapper isCodeInGraph cng.isCodeInGraph(ref) returns null");
			return Boolean.FALSE;
		}
	}


    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param version a coding scheme version
     * @param tag
     * @param code1
     * @param code2
     * @param associationName
     */
    public Boolean areCodesRelated(String codingScheme,
								 String version,
								 String tag,
								 String code1, // source
								 String code2, // target
								 String associationName)
	{
		CodingSchemeVersionOrTag tagOrVersion = createCodingSchemeVersionOrTag(version, tag);
		try {

			 CodedNodeGraph cng = lbSvc.getNodeGraph(codingScheme, tagOrVersion, null);

 	         NameAndValue association = createNameAndValue(associationName, null);


             ConceptReference ref1 = createConceptReference(code1, codingScheme);
             ConceptReference ref2 = createConceptReference(code2, codingScheme);

             Boolean bool_obj = Boolean.FALSE;
             try {
             	bool_obj = cng.areCodesRelated(association, ref1, ref2, true);
			 } catch (Exception ex) {
				 //ex.printStackTrace();
				 getLogger().warn("WARNING: cng.areCodesRelated throws exceptions");
				 return null;
			 }
			 return bool_obj;
	    } catch (Exception e) {
			e.printStackTrace();
			return null;
		}
    }


///////////////////////////////////////////////////////////////////////////////////////////////




	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param version a coding scheme version
	 * @param tag
	 * @param code
	 * @param direction
	 * @param associationName
	 */

	protected Vector<CodedEntry> getRelatedCodedEntries(
		                 String codingScheme,
						 String version,
						 String tag,
						 String code, // source
						 boolean direction, // true, if code is source
						 String associationName)
	{
        Vector<CodedEntry> v = new Vector<CodedEntry>();
//		String relation = null;
		CodingSchemeVersionOrTag csvt = createCodingSchemeVersionOrTag(version, tag);

		// Perform the query ...
//		NameAndValue nv = new NameAndValue();
		//NameAndValueList nvList = new NameAndValueList();

//		NameAndValueList nvList = null;

        try {

			boolean b1 = true;
			if (!direction)
			{
				b1 = false;
			}
			boolean b2 = false;
			if (!direction)
			{
				b2 = true;
			}

			ResolvedConceptReferenceList matches =
				lbSvc.getNodeGraph(codingScheme, csvt, null)
					.resolveAsList(
						createConceptReference(code, codingScheme),
						b1, b2, 1, 1, new LocalNameList(), null, null, 1024);

			// Analyze the result ...
			if (matches.getResolvedConceptReferenceCount() > 0) {
				ResolvedConceptReference ref =
					(ResolvedConceptReference) matches.enumerateResolvedConceptReference().nextElement();

				// Print the associations
				AssociationList al = null;
				if (direction)
				{
					al = ref.getSourceOf();
				}
				else
				{
					al = ref.getTargetOf();
				}


				Association[] associations = al.getAssociation();
				for (int i = 0; i < associations.length; i++) {
					Association assoc = associations[i];
					if (associationName != null)
					{
						if (assoc.getAssociationName().compareTo(associationName) == 0)
						{
							AssociatedConcept[] acl = assoc.getAssociatedConcepts().getAssociatedConcept();
							if (acl != null)
							{
								for (int j = 0; j < acl.length; j++) {

									AssociatedConcept ac = acl[j];
									if (ac != null)
									{
										String concept_code = ac.getConceptCode();

										CodedEntry ce = getCodedEntry(codingScheme,
											version, tag, concept_code);
										v.add(ce);
									}
								}
							}
						}
				    }
				    else
				    {
						AssociatedConcept[] acl = assoc.getAssociatedConcepts().getAssociatedConcept();
						if (acl != null)
						{
							for (int j = 0; j < acl.length; j++) {
								AssociatedConcept ac = acl[j];
								if (ac != null)
								{
									String concept_code = ac.getConceptCode();

									CodedEntry ce = getCodedEntry(codingScheme,
										version, null, concept_code);
									v.add(ce);
								}
							}
						}
					}
				}

			}
			return v;
	    } catch (Exception e) {
			return null;
		}

	}


    /**
     *
     * @since DLBExt 1.0
     * @param w
     */
    public Vector<String> getCodesInCodedEntries(Vector w)
	{
	    Vector<String> v = new Vector<String>();
        for (int i=0; i<w.size(); i++)
		{
			  CodedEntry ce = (CodedEntry) w.elementAt(i);
              v.add(ce.getConceptCode());
		}
        return v;
	}



//======================================================================================================================
    /*
     * Retrieve Relations for Concept
	 *
     */


    protected boolean isRole(String codingScheme, String associationName)
    {
		Vector associations = getSupportedRoleNames(codingScheme);
		if (associations == null || associations.size() == 0) return false;
		if (associations.contains(associationName)) return true;
		return false;
	}

    /**
	 *
	 *
	 * @param asso a ConceptAssociation
	 *
	 */
    public static Vector<Role> conceptAssociations2Roles(Vector<ConceptAssociation> asso)
    {

    	ConceptAssociation assoInstance = new ConceptAssociation();
    	Vector<Role> roles = new Vector<Role>();
    	for (int i = 0; i < asso.size(); i++)
    	{
    		assoInstance = asso.elementAt(i);
    		Role r = new Role(assoInstance.getName(), assoInstance.getValue());
    		Vector v = assoInstance.getQualifiers();
    		for (int j=0; j<v.size(); j++)
    		{
    			Qualifier q = (Qualifier) v.elementAt(i);
    			r.addQualifier(q);
    		}
    		roles.add(r);

    	}


		return roles;
	}

	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param code
	 */
	public Vector<Role> getRoles(String codingScheme, String code)
    {
	     return getRoles(codingScheme, version, tag, code);
    }


	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param code
	 */
	public Vector<ConceptAssociation> getConceptAssociations(String codingScheme, String code)
    {
		 return getConceptAssociations(codingScheme, version, tag, code);
    }



	public Vector<Role> getRoles(String codingScheme,
						   String code,
						   String associationName)
    {
	   		Vector<ConceptAssociation> ca =  getConceptAssociations(codingScheme,
					  version,
					  tag,
					  code,
					  associationName);
	   		return DLBWrapper.conceptAssociations2Roles(ca);
    }


	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param code
	 * @param associationName
	 */
	public Vector<ConceptAssociation> getConceptAssociations(String codingScheme,
										 String code,
										 String associationName)
    {
	     return getConceptAssociations(codingScheme,
										  version,
										  tag,
										  code,
										  associationName);
    }

   	public Vector<Role> getRoles(String codingScheme,
										  String version,
										  String tag,
										  String code,
										  String associationName)
	{
   		Vector<ConceptAssociation> ca =  getConceptAssociations(codingScheme, version, tag, code, associationName);
   		return DLBWrapper.conceptAssociations2Roles(ca);
	}

	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param version a coding scheme version
	 * @param tag
	 * @param code
	 * @param associationName
	 */
   	public Vector<ConceptAssociation> getConceptAssociations(String codingScheme,
										  String version,
										  String tag,
										  String code,
										  String associationName)
	{
	    Vector<ConceptAssociation> w = getConceptAssociations(codingScheme,
	                                       version,
	                                       tag,
	                                       code,
	                                       true);
	    if (w == null) return null;
	    if (associationName == null) return w;
	    Vector<ConceptAssociation> v = new Vector<ConceptAssociation>();
	    for (int i=0; i<w.size(); i++)
	    {
			gov.nih.nci.lexbig.ext.ConceptAssociation nv = (gov.nih.nci.lexbig.ext.ConceptAssociation) w.elementAt(i);
			if (nv.getName().compareTo(associationName) == 0)
			{
				v.add(nv);
			}
		}
		return v;
	}


	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param version a coding scheme version
	 * @param tag
	 * @param code
	 */

	 // Note: ConceptAssociations does not include roles

	public Vector<ConceptAssociation> getConceptAssociations(String codingScheme,
										  String version,
										  String tag,
										  String code)
	{
	    Vector<ConceptAssociation> v = new Vector<ConceptAssociation>();


	    String relationName = null;
	    if (hasRelation(codingScheme, ASSOCIATION_RELATION)) relationName = ASSOCIATION_RELATION;

        ResolvedConceptReference rcr = resolveConceptReference(codingScheme,
                                                               version,
                                                               tag,
                                                               code,
                                                               null,
                                                               1,
                                                               1,
                                                               true,
                                                               //ASSOCIATION_RELATION);
                                                               relationName);

		if (rcr == null)
		{
			getLogger().warn("WARNING: DLBWrapper getConceptAssociations -- Unable to resolveConceptReference " + code);
			return null;
		}

        AssociationList al = rcr.getSourceOf();

		if (al == null)
		{
			return v;
		}

		org.LexGrid.LexBIG.DataModel.Core.Association[] assoArray = al.getAssociation();

		if (assoArray == null)
		{
			return v;
		}

        String is_a = getHasSubtype(codingScheme);
		for (int i=0; i<assoArray.length; i++)
		{
		    org.LexGrid.LexBIG.DataModel.Core.Association association = assoArray[i];
			String assoname = association.getAssociationName();
			{
				if (!assoname.equals(is_a))
				{
					ConceptReference cr = association.getAssociationReference();
					CodedEntry codedentry_ref = getCodedEntry(cr.getCodingScheme(), version, null, cr.getConceptCode());
					v.add(new ConceptAssociation(assoname, getConceptName(codingScheme, codedentry_ref)));
				}
		    }
		}
		return v;
	}



	//030707
    public Vector<String> getSupportedRoleNames(String codingSchemeName)
	{
		if (!hasRelation(codingSchemeName, ROLE_RELATION)) return new Vector<String>();

		return getAssociationNamesByRelation(codingSchemeName, version, ROLE_RELATION);
    }

    public Vector<String> getSupportedRoleNames(String codingScheme, String version)
	{
		if (!hasRelation(codingScheme, version, null, ROLE_RELATION)) return new Vector<String>();
		return getAssociationNamesByRelation(codingScheme, version, ROLE_RELATION);
    }


	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param version a coding scheme version
	 * @param tag
	 * @param code
	 */
	public Vector<Role> getRoles(String codingScheme,
						   String version,
						   String tag,
					  	   String code)
	{
		Vector associations = getSupportedRoleNames(codingScheme, version);
		if (associations == null || associations.size() == 0) return new Vector<Role>();

	    Vector<Role> v = new Vector<Role>();
        ResolvedConceptReference rcr = resolveConceptReference(codingScheme, version,
		    tag, code);

		if (rcr == null)
		{
			getLogger().warn("WARNING: DLBWrapper getConceptAssociations -- Unable to resolveConceptReference " + code);
			return null;
		}

        AssociationList al = rcr.getSourceOf();

		if (al == null)
		{
			return v;
		}

		org.LexGrid.LexBIG.DataModel.Core.Association[] assoArray = al.getAssociation();

		if (assoArray == null)
		{
			return v;
		}

        String is_a = getHasSubtype(codingScheme);
		for (int i=0; i<assoArray.length; i++)
		{
		    org.LexGrid.LexBIG.DataModel.Core.Association association = assoArray[i];
			String assoname = association.getAssociationName();
			if (associations.contains(assoname))
			{
				if (!assoname.equals(is_a))
				{
					ConceptReference cr = association.getAssociationReference();
					CodedEntry codedentry_ref = getCodedEntry(cr.getCodingScheme(), version, tag, cr.getConceptCode());
					v.add(new Role(assoname, getConceptName(codingScheme, codedentry_ref)));
				}
		    }
		}
		return v;
	}


    /*
     * Enumerate Target Concepts for Relation and Source
     */

	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param code
	 */


	public Vector getInverseRoles(String codingScheme,
										  String code)
    {
         return getInverseRoles(codingScheme, version, tag, code);
    }

	public Vector getInverseConceptAssociations(String codingScheme,
										  String code)
    {
         return getInverseConceptAssociations(codingScheme, version, tag, code);
    }



	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param version a coding scheme version
	 * @param tag
	 * @param code
	 */
	public Vector<ConceptAssociation> getInverseConceptAssociations(String codingScheme,
										  String version,
										  String tag,
										  String code)
	{
		//Vector<String> associations = getSupportedAssociationNames(codingScheme, version, ROOT_ASSOCIATION);
		//Vector associations = getSupportedAssociationNames(codingScheme, version, ROOT_ASSOCIATION);
		//getSupportedConceptAssociationNames();
		//KLO 082307
		Vector associations = getAssociationNamesByRelation(codingScheme, version, ASSOCIATION_RELATION);

		if (associations == null || associations.size() == 0) return new Vector<ConceptAssociation>();

	    Vector<ConceptAssociation> v = new Vector<ConceptAssociation>();
        ResolvedConceptReference rcr = resolveConceptReference(codingScheme, version,
		    tag, code);

		if (rcr == null)
		{
			getLogger().warn("WARNING: DLBWrapper getInverseConceptAssociations -- Unable to resolveConceptReference " + code);
			return null;
		}

        AssociationList al = rcr.getTargetOf();
		if (al == null)
		{
			return v;
		}

		org.LexGrid.LexBIG.DataModel.Core.Association[] assoArray = al.getAssociation();

		if (assoArray == null)
		{
			return v;
		}

        String is_a = getHasSubtype(codingScheme);
		for (int i=0; i<assoArray.length; i++)
		{
		    org.LexGrid.LexBIG.DataModel.Core.Association association = assoArray[i];
			String assoname = association.getAssociationName();
			if (associations.contains(assoname))
			{
				//if (!assoname.equals(IS_A))
				if (!assoname.equals(is_a))
				{
					ConceptReference cr = association.getAssociationReference();
					CodedEntry codedentry_ref = getCodedEntry(cr.getCodingScheme(), version, tag, cr.getConceptCode());
					v.add(new ConceptAssociation(assoname, getConceptName(codingScheme, codedentry_ref)));
				}
		    }
		}

		return v;
	}


    private Vector<Role> nameAndValues2Roles(Vector<NameAndValue> v)
    {

		if (v == null) return new Vector<Role>();
		Vector<Role> u = new Vector<Role>();
		for (int i=0; i<v.size(); i++)
		{
			NameAndValue nv = (NameAndValue) v.elementAt(i);
		    u.add(new Role(nv.getName(), nv.getContent()));
		}
		return u;
	}

	public Vector getInverseRoles(String codingScheme,
										  String version,
										  String tag,
										  String code)
	{


		Vector<ConceptAssociation> w = getConceptAssociations(
											   codingScheme,
											   version,
											   tag,
											   code,
											   false);
		return conceptAssociations2Roles(w);
	//	return nameAndValues2Roles(w);
	}


    /*
     * Enumerate Relation Types for Concept
     */

	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param version a coding scheme version
	 * @param tag
	 * @param code
	 */
	public HashSet<String> getRelationTypesForConcept(String codingScheme,
										  String version,
										  String tag,
										  String code)
	{
	    HashSet<String> hset = new HashSet<String>();
        ResolvedConceptReference rcr = resolveConceptReference(codingScheme, version,
		    tag, code);

        AssociationList al = rcr.getSourceOf();
		org.LexGrid.LexBIG.DataModel.Core.Association[] assoArray = al.getAssociation();
		for (int i=0; i<assoArray.length; i++)
		{
		    org.LexGrid.LexBIG.DataModel.Core.Association association = assoArray[i];
			String assoname = association.getAssociationName();
			if (!hset.contains(assoname))
			{
                hset.add( assoname);
			}
		}
		return hset;
	}


///////////////////////
//  Search
///////////////////////



	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param version a coding scheme version
	 * @param matchText
	 * @param preferredOnly
	 * @param sortByProperty_vec
	 * @param source_vec
	 * @param matchAlgorithm
	 * @param maxToReturn
	 */


	public Vector<CodedEntry> findCodedEntriesWithDesignationMatching(
		                                        String codingScheme,
	                                            String version,
	                                            String matchText,
												boolean preferredOnly,
	                                            Vector sortByProperty_vec,
                                                Vector source_vec,
                                                String matchAlgorithm,
	                                            int maxToReturn)
	{

//	    CodedNodeSet cns_1;
//	    CodedNodeSet cns_2;
	    CodedNodeSet cns;
        Vector<CodedEntry> v = new Vector<CodedEntry>();
        try {
			CodingSchemeVersionOrTag versionOrTag = createCodingSchemeVersionOrTag(version, null);
			cns = lbSvc.getCodingSchemeConcepts(codingScheme, versionOrTag);//, false);
//			cns_1 = lbSvc.getCodingSchemeConcepts(codingScheme, versionOrTag, false);
//			cns_2 = lbSvc.getCodingSchemeConcepts(codingScheme, versionOrTag, false);
//			if (cns_1 == null || cns_2 == null)
			if (cns == null)
			{
				getLogger().warn("WARNING: DLBWrapper getCodingSchemeConcepts returns null");
				return v;
			}

			//Vector property_vec = new Vector();

//            if (hasProperty(codingScheme, "CONCEPT_NAME"))
//            {
//				property_vec.add("CONCEPT_NAME");
//			}
//
//            if (hasProperty(codingScheme, "textualPresentation"))
//            {
//				property_vec.add("textualPresentation");
//			}

			//property_vec.add("textualPresentation");

			//LocalNameList propertyList = vector2LocalNameList(property_vec);
			LocalNameList sourceList = new LocalNameList();

//			LocalNameList contextList = null;
//			NameAndValueList qualifierList = null;

			//cns.restrictToMatchingProperties(propertyList, null, matchText, matchAlgorithm, null);
			cns = cns.restrictToMatchingDesignations(matchText, SearchDesignationOption.PREFERRED_ONLY, matchAlgorithm, null);

//			cns_1 = cns_1.restrictToMatchingProperties(propertyList, null, matchText, matchAlgorithm, null);
//			cns_2	= cns_2.restrictToMatchingDesignations(matchText, preferredOnly, matchAlgorithm, null);
//			cns = cns_1.union(cns_2);


			if (source_vec != null && source_vec.size() > 0)
			{
				// 050507
			    sourceList = vector2LocalNameList(source_vec);
			    //cns = cns.restrictToProperties(propertyList, null, sourceList, null, null);
			    CodedNodeSet.PropertyType[] types = new PropertyType[] {PropertyType.PRESENTATION};
			    //cns = cns.restrictToProperties(null, null, sourceList, null, null);
			    cns = cns.restrictToProperties(null, types, sourceList, null, null);
			}


			LocalNameList restrictToProperties = new LocalNameList();//vector2LocalNameList(restrictToProperties_vec);

//			LocalNameList sortByProperty = vector2LocalNameList(sortByProperty_vec);
			//LocalNameList restrictToProperties = vector2LocalNameList(restrictToProperties_vec);

			// Sort by search engine recommendation & code .
			SortOptionList sortCriteria =
			    Constructors.createSortOptionList(new String[]{"matchToQuery", "code"});

			//ResolvedConceptReferenceList list = cns.resolveToList(sortByProperty,
			ResolvedConceptReferenceList list = cns.resolveToList(sortCriteria,
																  restrictToProperties,
																  null,
																  maxToReturn);

			if (list == null)
			{
				getLogger().warn("WARNING: DLBWrapper resolveToList returns null");
			}

			ResolvedConceptReference[] rcrArray = list.getResolvedConceptReference();

			if (rcrArray == null)
			{
				getLogger().warn("WARNING: DLBWrapper getResolvedConceptReference returns null");
			}

			for (int i=0; i<rcrArray.length; i++)
			{
				ResolvedConceptReference rcr = (ResolvedConceptReference) rcrArray[i];
				v.add(rcr.getReferencedEntry());
			}

	    } catch (Exception e) {
			e.printStackTrace();
			return v;
	    }
		return v;
	}


//////////////////////////////////////////////////////////////////////////////////////////
	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param version a coding scheme version
	 * @param associationName
	 * @param matchAlgorithm
	 * @param maxToReturn
	 */
	public Vector findCodedEntriesWithAssociation(String codingScheme,
	                                              String version,
	                                              String associationName,
	                                              String matchAlgorithm,
	                                              int maxToReturn)
	{ //throws ParameterException, InternalErrorException {
	    return findCodedEntriesWithAssociationMatching(codingScheme,
	                                            version,
	                                            "*",
												associationName,
												matchAlgorithm,
												maxToReturn);
	}

//////////////////////////////////////////////////////////////////////////////////////


	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param version a coding scheme version
	 * @param matchText
	 * @param associationName
	 * @param matchAlgorithm
	 * @param maxToReturn
	 */

	public Vector findCodedEntriesWithAssociationMatching(String codingScheme,
	                                            String version,
	                                            String matchText,
												String associationName,
												String matchAlgorithm,
												int maxToReturn)
	{
	     return findCodedEntriesWithAssociationMatching(codingScheme,
	                                            version,
	                                            matchText,
												associationName,
												matchAlgorithm,
												maxToReturn,
												RESTRICT_TO_TARGET);
    }

	public Vector findCodedEntriesWithAssociationMatching(String codingScheme,
	                                            String version,
	                                            String matchText,
												String associationName,
												String matchAlgorithm,
												int maxToReturn,
												int restriction_direction)
	{
		Vector restricted_property_vec = null;
		return findCodedEntriesWithAssociationMatching(codingScheme,
	                                            version,
	                                            matchText,
												associationName,
												matchAlgorithm,
												maxToReturn,
												restriction_direction,
												restricted_property_vec);
	}

    protected int getCNSSize(CodedNodeSet cns)
    {
         try {
			 ResolvedConceptReferenceList rcrl = cns.resolveToList(null, null, null, 1000);
			 ResolvedConceptReference[] rcrArray = rcrl.getResolvedConceptReference();

			 return rcrArray.length;
	     } catch (Exception e) {
			 e.printStackTrace();
		 }
		 return -1;
	}


    protected int getCNGSize(CodedNodeGraph cng)
    {
		 boolean resolveForward = true;
		 boolean resolveBackward = false;
		 int resolveAssociationDepth = 1;
		 int maxToReturn = 1000;
         try {
             CodedNodeSet cns = cng.toNodeList(null, resolveForward, resolveBackward, resolveAssociationDepth, maxToReturn);

			 return getCNSSize(cns);
	     } catch (Exception e) {
			 e.printStackTrace();
		 }
		 return -1;

	}

	public Vector findCodedEntriesWithAssociationMatching(String codingScheme,
	                                            String version,
	                                            String matchText,
												String associationName,
												String matchAlgorithm,
												int maxToReturn,
												int restriction_direction,
												Vector restricted_property_vec)
	{
	     return findCodedEntriesWithAssociationMatching(codingScheme,
	                                            version,
	                                            matchText,
												associationName,
												matchAlgorithm,
												maxToReturn,
												restriction_direction,
												restricted_property_vec,
												null);
	}



	public Vector<CodedEntry> findCodedEntriesWithAssociationMatching(String codingScheme,
	                                            String version,
	                                            String matchText,
												String associationName,
												String matchAlgorithm,
												int maxToReturn,
												int restriction_direction,
												Vector restricted_property_vec,
												String relationName)
	{
        Vector<CodedEntry> v = new Vector<CodedEntry>();
        CodedNodeSet cns = null;
        CodedNodeGraph cng = null;

        CodingSchemeVersionOrTag versionOrTag = new CodingSchemeVersionOrTag();
        versionOrTag.setVersion(version);

        try {
			cns = lbSvc.getCodingSchemeConcepts(codingScheme, versionOrTag);//, false);
			if (cns == null) return v;
//			boolean preferredOnly = false;
			String language = null;
			//cns = cns.restrictToMatchingDesignations(matchText, preferredOnly, matchAlgorithm, language);

			cns = cns.restrictToMatchingDesignations(matchText,
			               CodedNodeSet.SearchDesignationOption.ALL,
			               matchAlgorithm, language);

            LocalNameList propertyList = null;
            if (restricted_property_vec != null)
            {
				propertyList = vector2LocalNameList(restricted_property_vec);
				cns = cns.restrictToProperties(propertyList, null);
			}

            cng = lbSvc.getNodeGraph(codingScheme, versionOrTag, relationName);

			NameAndValueList asso_list =
				createNameAndValueList(
					new String[] {associationName}, null);

			//NameAndValueList qualifier_list = createNameAndValueList(new String[] {"someValuesFrom"}, null);

            cng = cng.restrictToAssociations(asso_list, null);
            //cng = cng.restrictToAssociations(asso_list, qualifier_list);

            ResolvedConceptReferenceList rcrl = null;

            if (restriction_direction == RESTRICT_TO_TARGET)
            {
				cng = cng.restrictToTargetCodes(cns);
			    rcrl = cng.resolveAsList(null, true, false, // graphFocus, resolveForward, resolveBackward
				   1, 1, null, null, null, maxToReturn);
			}
			else if (restriction_direction == RESTRICT_TO_SOURCE)
			{
				cng = cng.restrictToSourceCodes(cns);
			    rcrl = cng.resolveAsList(null, true, false, // graphFocus, resolveForward, resolveBackward
				   1, 1, null, null, null, maxToReturn);
			}

 			ResolvedConceptReference[] rcrArray = rcrl.getResolvedConceptReference();
 			CodedEntry ce = null;

			for (int i=0; i<rcrArray.length; i++)
			{
				if (restriction_direction == RESTRICT_TO_TARGET)
				{
				    ce = rcrArray[i].getReferencedEntry();
				    if (ce != null)
				    {
						v.add(ce);
					}
				}
				else if (restriction_direction == RESTRICT_TO_SOURCE)
				{
                    AssociationList al = rcrArray[i].getSourceOf();
					Association[] assos = al.getAssociation();
					if (assos != null)
					{
						for (int j=0; j<assos.length; j++)
						{
							Association asso = (Association) assos[j];
                            AssociatedConceptList acl = asso.getAssociatedConcepts();
                            AssociatedConcept[] ac_array = acl.getAssociatedConcept();
                            for (int k=0; k<ac_array.length; k++)
                            {
								AssociatedConcept ac = (AssociatedConcept) ac_array[k];
								String code = ac.getConceptCode();
								ce = getCodedEntry(codingScheme, version, tag, code);
								if (ce != null) v.add(ce);
						    }
						}
					}
					else
					{
						getLogger().warn("assos == null");
					}
				}
			}
	    } catch (Exception e) {
			e.printStackTrace();
		}
		return v;
	}


///////////////////////
//  Tree
///////////////////////


	/** Concepts tree data from the server.
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param version a coding scheme version
	 * @param tag
	 * @param code
	 * @param relationName
	 * @param properties
	 * @param sourcecodes
	 * @param associations
	 * @param direction
	 * @param depth
	 * @return The root (DefaultMutableTreeNode) of the tree
	 */


    /*
     * Generate DAG
	 * Enumerate Concepts by Relationship
	 *
     */

	protected CodedNodeGraph getTreeFromServer(String codingScheme,
													 String version,
													 String tag,
													 String code, // root concept code

													 String relationName,

													 Vector properties,
													 Vector sourcecodes,
													 Vector associations,

													 boolean direction,
													 int depth)

    {
	     CodedNodeGraph cng = null;
	     try {
			 CodingSchemeVersionOrTag versionOrTag = createCodingSchemeVersionOrTag(version, tag);
			 cng = lbSvc.getNodeGraph(codingScheme, versionOrTag, relationName);

            String[] asso_ary = new String[associations.size()];
            for (int i=0; i<associations.size(); i++)
            {
				asso_ary[i] = (String) associations.elementAt(i);
			}

            NameAndValueList restrictToAssociations = createNameAndValueList(asso_ary, null);

			NameAndValueList associationQualifiers = new NameAndValueList();

			cng = cng.restrictToAssociations(restrictToAssociations,
											 associationQualifiers);


			//ConceptReference focusConcept = createConceptReference(codingScheme, code);

//			ConceptReference focusConcept = createConceptReference(code, codingScheme);


			// to be implemented:
			Boolean ret = null;

			 //Boolean ret = cng.setFocus(focusConcept);

			 if (ret == null || ret == Boolean.FALSE) return null;

		 } catch (Exception e) {
			e.printStackTrace();
			return null;
		 }

         return cng;
    }


	/**
	 *
	 * @since DLBExt 1.0
	 * @param HashMap
	 * @param code
	 */
	protected int getChildCount(HashMap HashMap, String code)
	{
		if (HashMap.containsKey(code))
		{
			Vector v = (Vector) HashMap.get(code);
			return v.size();
		}
	    return 0;
	}




	/////////////////////////////////////////////////////////////////////////////////////////



	/**
	 * Convert an NCI concept code to a concept name.  Note that this could be simplified if everyone
	 * agrees that the entityDescription node for the concept code will always be the NC_Name.
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param code Code to be converted
	 * @return name or null if code not located
	 */

	public String codeToName(String codingScheme, String code)
	{
        return codeToName(codingScheme, version, tag, code);
	}


	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param version a coding scheme version
	 * @param code
	 */

	public String codeToName(String codingScheme, String version, String code) {
		return codeToName(codingScheme, version, tag, code);
	}

	public String codeToName(String codingScheme, String version_in, String tag_in, String code) {
		try {
			String version_param = version_in;
			if (version_param == null) version_param = version;

			String tag_param = tag_in;
			if (tag_param == null) tag_param = tag;

			CodingSchemeVersionOrTag vtag = createCodingSchemeVersionOrTag(version_param, tag_param);
			String name = new ConvenienceMethods(lbSvc).codeToName(code, codingScheme, vtag);
			return name;
		} catch (Exception ex) {
			getLogger().warn("Exception." + ex);
		}
		return null;
    }


	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param version a coding scheme version
	 * @param code
	 */
	public CodedEntry getCodedEntryByCode(String codingScheme, String version, String tag, String code)
	{
//        ResolvedConceptReference rcr = null;
        try {
			CodingSchemeVersionOrTag versionOrTag = createCodingSchemeVersionOrTag(version, tag);
			ConceptReferenceList crefs =
				createConceptReferenceList(
					new String[] {code}, codingScheme);

			CodedNodeSet cns = lbSvc.getCodingSchemeConcepts(codingScheme, versionOrTag);
			cns = cns.restrictToCodes(crefs);
			ResolvedConceptReferenceList matches = cns.resolveToList(null, null, null, 1);


			// Analyze the result ...
			if (matches.getResolvedConceptReferenceCount() > 0) {
				ResolvedConceptReference ref =
					(ResolvedConceptReference) matches.enumerateResolvedConceptReference().nextElement();

				CodedEntry entry = ref.getReferencedEntry();
				return entry;
			}
		 } catch (Exception e) {
			 e.printStackTrace();
			 return null;
		 }
		 return null;
	}


	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param version a coding scheme version
	 * @param code
	 */
	public String getConceptName(String codingScheme, String version, String code)
	{
		return codeToName(codingScheme, version, code);
	}



	/**
	 * Return the concept code that corresponds to the passed NC_Name
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param name NC_name to look up
	 * @exception ParameterException
	 * @exception InternalErrorException
	 * @return corresponding code or null if problems
	 */


	public String nameToCode(String codingScheme, String name)
	{
        return nameToCode(codingScheme, version, tag, name);
	}


	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param version_in a coding scheme version
	 * @param tag_in a coding scheme tag
	 * @param name concept name
	 */

	public String nameToCode(String codingScheme, String version_in, String tag_in, String name) {
		try {
			String version_param = version_in;
			if (version_param == null) version_param = version;

			String tag_param = tag_in;
			if (tag_param == null) tag_param = tag;

			CodingSchemeVersionOrTag vtag = createCodingSchemeVersionOrTag(version_param, tag_param);

			String code = new ConvenienceMethods(lbSvc).nameToCode(name, codingScheme, vtag);
			return code;
		} catch (Exception ex) {
			getLogger().warn("Exception." + ex);
		}
		return null;
    }


	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param version a coding scheme version
	 * @param name
	 */

	public CodedEntry getCodedEntryByName(String codingScheme, String version, String tag, String name)
	{
		try {
			String code = nameToCode(codingScheme, version, tag, name);
			return getCodedEntryByCode(codingScheme, version, tag, code);
		} catch (Exception e) {
			getLogger().error("Error occurred in getCodedEntryByName");
			e.printStackTrace();
		}
		return null;
	}

	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param version a coding scheme version
	 * @param matchText
	 * @param property_vec
	 * @param restrictToProperties_vec
	 * @param matchAlgorithm
	 * @param maxToReturn
	 */
	public Vector<CodedEntry> findCodedEntriesWithPropertyMatching(String codingScheme,
	                                                   String version,
	                                                   String matchText,
	                                                   Vector property_vec,
	                                                   Vector restrictToProperties_vec,
	                                                   String matchAlgorithm,
	                                                   int maxToReturn)
    {
	    return findCodedEntriesWithPropertyMatching(codingScheme,
	                                                   version,
	                                                   matchText,
	                                                   property_vec,
	                                                   restrictToProperties_vec,
	                                                   matchAlgorithm,
	                                                   maxToReturn,
	                                                   null);

    }



	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param version a coding scheme version
	 * @param matchText
	 * @param property_vec
	 * @param restrictToProperties_vec
	 * @param matchAlgorithm
	 * @param maxToReturn
	 * @param language
	 */
	public Vector<CodedEntry> findCodedEntriesWithPropertyMatching(String codingScheme,
	                                                   String version,
	                                                   String matchText,
	                                                   Vector property_vec,
	                                                   Vector restrictToProperties_vec,
	                                                   String matchAlgorithm,
	                                                   int maxToReturn,
	                                                   String language)
	{

//		for (int i=0; i<property_vec.size(); i++)
//		{
//			String s = (String)property_vec.elementAt(i);
//		}

         String tag = null;
//         boolean activeOnly = true;
         Vector<CodedEntry> v = new Vector<CodedEntry>();
		 CodingSchemeVersionOrTag tagOrVersion = createCodingSchemeVersionOrTag(version, tag);
	 	 try {
	         //CodedNodeSet cns = new CodedNodeSetImpl(codingScheme, tagOrVersion, true);
			 CodedNodeSet cns = lbSvc.getCodingSchemeConcepts(codingScheme, tagOrVersion);//, activeOnly);

//	         boolean preferredOnly = false;
             LocalNameList propertyList = new LocalNameList();
	         if (property_vec != null || property_vec.size() > 0)
             {
 	             propertyList = vector2LocalNameList(property_vec);
			 }
			 else
			 {
				 propertyList = vector2LocalNameList(getSupportedPropertyNames(getCodingScheme(codingScheme, version, null)));
			 }
             cns = cns.restrictToMatchingProperties(propertyList, null, matchText, matchAlgorithm, language);

//             LocalNameList sortByProperty = null;
             LocalNameList restrictToProperties = vector2LocalNameList(restrictToProperties_vec);

			 // Sort by search engine recommendation & code ...
			 SortOptionList sortCriteria =
			    Constructors.createSortOptionList(new String[]{"matchToQuery", "code"});

             ResolvedConceptReferenceList rcrl = cns.resolveToList(sortCriteria, restrictToProperties,
                  null, maxToReturn);

			 ResolvedConceptReference[] rcrs = rcrl.getResolvedConceptReference();
			 ResolvedConceptReference rcr = null;
			 for (int i=0; i<rcrs.length; i++)
			 {
				 rcr = rcrs[i];
				 CodedEntry ce = rcr.getReferencedEntry();
				 v.add(ce);
			 }
			 return v;

		 } catch (Exception e) {
			 e.printStackTrace();

		 }
		 return v;
	}



	/**
	 * Returns the entity description for a given code, or the code
	 * if no description can be located.
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param code
	 * @exception LBException
	 * @return String
	 */
	protected String findDescription(String codingScheme, String code) {
		// Perform the query .
		try {
			ConceptReferenceList crefs =
				createConceptReferenceList(
					new String[] {code}, codingScheme);

			CodedNodeSet nodes = lbSvc.getCodingSchemeConcepts(codingScheme, createCodingSchemeVersionOrTag(version, tag));
			nodes = nodes.restrictToCodes(crefs);

			// Restrict the properties ...
			ResolvedConceptReferenceList matches =
				nodes.resolveToList(null, new LocalNameList(), null, 1);

			// Analyze the result ...
			if (matches.getResolvedConceptReferenceCount() > 0) {
				ResolvedConceptReference ref =
					(ResolvedConceptReference) matches.enumerateResolvedConceptReference().nextElement();
				return ref.getReferencedEntry().getEntityDescription().getContent();
			}
	    } catch (Exception e) {
			e.printStackTrace();
		}
		return code;
	}



	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param version a coding scheme version
	 * @param code
	 */
	public String findConceptNameByCode(String codingScheme,
	                               String version,
	                               String code)
	{
        return codeToName(codingScheme, version, code);
    }

	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param version a coding scheme version
	 * @param code
	 * @param relation
	 * @param direction
	 */
	public Vector<String> findRelatedCodes(String codingScheme,
	                               String version,
	                               String tag,
	                               String code,
	                               String relation,
	                               boolean direction)
	{
	    Vector w = getAssociatedConcepts(codingScheme,
	                                       version,
	                                       tag,
	                                       code,
	                                       relation,
	                                       direction);
	    if (w == null) return null;
	    Vector<String> v = new Vector<String>();
	    for (int i=0; i<w.size(); i++)
	    {
			AssociatedConcept cs = (AssociatedConcept) w.elementAt(i);
			v.add(cs.getConceptCode());
		}
		return v;
	}



	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param version a coding scheme version
	 * @param code
	 * @param direction
	 */
	public Vector getAllAssociatedConcepts(String codingScheme, String version,
	                              String code, boolean direction)
	{
	    return getAssociatedConcepts(codingScheme,
	                                       version,
	                                       tag,
	                                       code,
	                                       null,
	                                       direction);
	}






///////////////////////
//  Concept
///////////////////////

    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme
     * @param ce
     */
    public String getConceptName(String codingScheme, CodedEntry ce)
	{
	    if (ce == null) return null;
	    return codeToName(codingScheme, ce.getConceptCode());
	}

///////////////////////////////////////////////////////////////////////////////////

    public CodedEntry getCodedEntry(String codingScheme,
	                                String version,
									String tag,
									String code)
	{
         return getCodedEntry(codingScheme, version, tag, code, 3);
	}


     protected LocalNameList atributeSetDescriptorIndex2LocalNameList(int ASD_Index)
     {
		 if (ASD_Index == 1 || ASD_Index == 3) return null;
		 return new LocalNameList();
	 }

    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param version a coding scheme version
     * @param tag
     * @param code
     * @param ASD_Index
     */


    public CodedEntry getCodedEntry(String codingScheme,
	                                String version,
									String tag,
									String code,
									int ASD_Index)

	{
		if (code == null || code.compareTo("") == 0)
		{
			getLogger().error("getCodedEntry getCodedEntry code = null");
			return null;
		}

        ConceptReferenceList crefs = new ConceptReferenceList();
		ConceptReference cr = new ConceptReference();
		cr.setCodingScheme(codingScheme);
		cr.setConceptCode(code);
		crefs.addConceptReference(cr);


		// Perform the query ...
        ResolvedConceptReferenceList matches = null;
		try {
			CodingSchemeVersionOrTag tagOrVersion = createCodingSchemeVersionOrTag(version, tag);
			CodedNodeSet cns = lbSvc.getCodingSchemeConcepts(codingScheme, tagOrVersion);//, false)
			cns = cns.restrictToCodes(crefs);
			matches = cns.resolveToList(null, null, null, 1);

			if (matches == null)
			{
				getLogger().warn("WARNING: getCodedEntry ResolvedConceptReferenceList returns matches = 0)");
				return null;
			}

			// Analyze the result ...
			if (matches.getResolvedConceptReferenceCount() > 0) {
				ResolvedConceptReference ref =
					(ResolvedConceptReference) matches.enumerateResolvedConceptReference().nextElement();
				CodedEntry entry = ref.getReferencedEntry();

				return entry;
			}
	    } catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}


    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param version a coding scheme version
     */
    public Vector getRootCodedEntries(String codingScheme, String version, String tag)
    {
         return getRootCodedEntries(codingScheme,
 								  version,
 								  tag,
 								  null, // relation
 								  getHasSubtype(codingScheme));
    }



    protected CodedEntry getReferencedEntry(String codingScheme, String version, String tag, ResolvedConceptReference rcr)
    {
		if (rcr == null) return null;

		CodedEntry ce = rcr.getReferencedEntry();
		if (ce != null) return ce;
		ce = getCodedEntryByCode(codingScheme, version, tag, rcr.getConceptCode());
		if (ce == null)
		{
			getLogger().warn("CodedEntry is null");
		}
		return ce;
	}


	/**
	 *
	 * @since DLBExt 1.0
	 * @param codingScheme a coding scheme name
	 * @param version a coding scheme version
	 * @param relation
	 * @param association
	 */
	public Vector<CodedEntry> getRootCodedEntries(String codingScheme,
								   String version,
								   String tag,
								   String relation,
								   String association)
    {
		Vector<CodedEntry> v = new Vector<CodedEntry>();
		CodingSchemeVersionOrTag tagOrVersion = createCodingSchemeVersionOrTag(version, tag);
        try {
			ResolvedConceptReferenceList rcrl = null;
			try {
				rcrl = lbConvMethods.getTopNodes(codingScheme, tagOrVersion, relation, association); //"hasSubtype");
			} catch (Exception ex) {
				ex.printStackTrace();
				getLogger().warn("lbConvMethods.getTopNodes threws exceptions");
				return null;
			}

			ResolvedConceptReference[] rcrs = rcrl.getResolvedConceptReference();
			if (rcrs == null)
			{
				getLogger().warn("WARNING: getResolvedConceptReference returns rcrs == null");
			}
			else if (rcrs.length == 0)
			{
				getLogger().warn("WARNING: getResolvedConceptReference returns 0 entries.");
			}
			else
			{
				for (int i=0; i<rcrs.length; i++)
				{
					ResolvedConceptReference rcr = rcrs[i];
					CodedEntry ce = getReferencedEntry(codingScheme, version, tag, rcr);
					if (ce != null)
					{
						v.add(ce);
					}
			    }
			}
	    } catch (Exception e) {
			e.printStackTrace();
			return null;
		}
		return v;
	}



    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param version a coding scheme version
     * @param tag
     * @param code
     * @param properties
     * @param direction
     * @param associationName
     * @param resolveAssociationDepth
     * @param resolveCodedEntryDepth
     */
    public Vector<CodedEntry> getAssociatedCodedEntries(String codingScheme,
										 String version,
										 String tag,
	                                     String code,
										 Vector properties,
										 boolean direction,
										 String associationName,
										 int resolveAssociationDepth,
										 int resolveCodedEntryDepth
										 )
    {
	    Vector<CodedEntry> v = new Vector<CodedEntry>();
	    try {
			ResolvedConceptReference rcr = resolveConceptReference(
											 codingScheme,
											 version,
											 tag,
											 code,
											 properties,
											 resolveAssociationDepth,
											 resolveCodedEntryDepth,
											 direction);

			if (rcr == null)
			{
				getLogger().warn("WARNING: resolveConceptReference returns null.");
				return null;
			}



			AssociationList ass0list = null;
			if (direction) // associations of which this code is source
			{
			   ass0list = rcr.getSourceOf();
			}
			else // associations of which this code is target
			{
			   ass0list = rcr.getTargetOf();
			}

			if (ass0list == null)
			{
				return v;
			}

			Association[] assoArray = ass0list.getAssociation();

			if (assoArray == null)
			{
				return v;
			}

			// Check association type (is-a, part-of, etc)
			for (int k=0; k<assoArray.length; k++)
			{
				String name = assoArray[k].getAssociationName();
				// 050707
				if (associationName != null)
				{
					if (name.compareTo(associationName) == 0)
					{
						AssociatedConceptList acl =	assoArray[k].getAssociatedConcepts();
						if (acl != null)
						{
							AssociatedConcept[] acArray = acl.getAssociatedConcept();
							if (acArray != null && acArray.length > 0)
							{
								for (int i=0; i<acArray.length; i++)
								{
									if (acArray[i] != null)
									{
										CodedEntry ce = null;
										try {
											ce = acArray[i].getReferencedEntry();
										} catch (Exception ex) {

										}

										if (ce != null)
										{
											v.add(ce);
										}
									}
								}
							}
						}
					}
			    }
			    else
			    {
					AssociatedConceptList acl =	assoArray[k].getAssociatedConcepts();
					if (acl != null)
					{
						AssociatedConcept[] acArray = acl.getAssociatedConcept();
						if (acArray != null && acArray.length > 0)
						{
							for (int i=0; i<acArray.length; i++)
							{
								if (acArray[i] != null)
								{
									CodedEntry ce = null;
									try {
										ce = acArray[i].getReferencedEntry();
									} catch (Exception ex) {

									}

									if (ce != null)
									{
										v.add(ce);
									}
								}
							}
						}
					}
				}
			}
	    } catch (Exception e) {
			e.printStackTrace();
			return null;
		}
        return v;
    }



    // 120406
    public Vector<CodedEntry> getAssociatedCodedEntriesBySource(String codingScheme,
										 String version,
										 String tag,
	                                     String code,
										 Vector properties,
										 boolean direction,
										 String associationName,
										 int resolveAssociationDepth,
										 int resolveCodedEntryDepth,
										 String sourceAbbr
										 )
    {
	    Vector<CodedEntry> v = new Vector<CodedEntry>();
	    try {
			ResolvedConceptReference rcr = resolveConceptReference(
											 codingScheme,
											 version,
											 tag,
											 code,
											 properties,
											 resolveAssociationDepth,
											 resolveCodedEntryDepth,
											 direction);

			if (rcr == null)
			{
				getLogger().warn("WARNING: resolveConceptReference returns null.");
				//050807
				//return null;
				return v;
			}

			AssociationList ass0list = null;
			if (direction) // associations of which this code is source
			{
			   ass0list = rcr.getSourceOf();
			}
			else // associations of which this code is target
			{
			   ass0list = rcr.getTargetOf();
			}

			if (ass0list == null)
			{
				return v;
			}

			Association[] assoArray = ass0list.getAssociation();

			if (assoArray == null)
			{
				return v;
			}

			// Check association type (is-a, part-of, etc)
			for (int k=0; k<assoArray.length; k++)
			{
				String name = assoArray[k].getAssociationName();

				// 050707
				if (associationName != null)
				{
					if (name.compareTo(associationName) == 0)
					{
						AssociatedConceptList acl =	assoArray[k].getAssociatedConcepts();
						if (acl != null)
						{
							AssociatedConcept[] acArray = acl.getAssociatedConcept();
							if (acArray != null && acArray.length > 0)
							{
								for (int i=0; i<acArray.length; i++)
								{
									boolean matched = false;
									if (acArray[i] != null)
									{
										if (sourceAbbr != null && sourceAbbr.compareTo("") != 0)
										{
											// modified to be compatible with version 2
											//LocalNameList lnl =	acArray[i].getAssociationQualifiers();
											NameAndValueList lnl =	acArray[i].getAssociationQualifiers();
											NameAndValue[] nvArray = lnl.getNameAndValue();
											for (int k2=0; k2<nvArray.length; k2++)
											{
												NameAndValue nv = nvArray[k2];
												if (nv.getName().compareTo(sourceAbbr) == 0)
												{
													matched = true;
													break;
												}
											}


										}
										else
										{
											matched = true;
										}

										if (matched)
										{
											CodedEntry ce = null;
											try {
												ce = acArray[i].getReferencedEntry();
											} catch (Exception ex) {
                                                ex.printStackTrace();
											}

											if (ce != null)
											{
												v.add(ce);
											}
										}
									}
								}
							}
						}
				    }
				}
				else
				{
					AssociatedConceptList acl =	assoArray[k].getAssociatedConcepts();
					if (acl != null)
					{
						AssociatedConcept[] acArray = acl.getAssociatedConcept();
						if (acArray != null && acArray.length > 0)
						{
							for (int i=0; i<acArray.length; i++)
							{
								boolean matched = false;
                                if (acArray[i] != null)
                                {
									if (sourceAbbr != null && sourceAbbr.compareTo("") != 0)
									{
										// check if source matches
										// Modified to be compatible with version 2
										// LocalNameList lnl =	acArray[i].getAssociationQualifiers();
										NameAndValueList lnl =	acArray[i].getAssociationQualifiers();
										NameAndValue[] nvArray = lnl.getNameAndValue();
										for (int k2=0; k2<nvArray.length; k2++)
										{
											NameAndValue nv = nvArray[k2];
											if (nv.getName().compareTo(sourceAbbr) == 0)
											{
												matched = true;
												break;
											}
										}
									}
									else
									{
										matched = true;
									}

									if (matched)
									{
										CodedEntry ce = null;
										try {
											ce = acArray[i].getReferencedEntry();
										} catch (Exception ex) {
                                            ex.printStackTrace();
										}

										if (ce != null)
										{
											v.add(ce);
										}
								    }
							    }
							}
					    }
				    }
				}
			}
	    } catch (Exception e) {
			e.printStackTrace();
			return null;
		}
        return v;
    }





    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param relationship
     * @param association
     * @param conceptCode
     * @param direction
     */
    public Boolean hasAssociatedCodedEntries(String codingScheme,
                                           String relationship,
                                           String association,
                                           String conceptCode,
                                           boolean direction)
    {
        Vector v = getAssociatedCodedEntries(codingScheme,
                                           relationship,
                                           association,
                                           conceptCode,
                                           direction, true);
        if (v == null) return Boolean.FALSE;
        if (v.size() > 0) return Boolean.TRUE;
        return Boolean.FALSE;
    }


    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param version a coding scheme version
     * @param conceptCode
     * @param direction
     */
    public Vector<ConceptAssociation> getConceptAssociations(
		                                   String codingScheme,
                                           String version,
                                           String tag,
                                           String conceptCode,
                                           boolean direction
                                           )
    {
         Vector<ConceptAssociation> v = getConceptAssociations(
		                                   codingScheme,
                                           version,
                                           tag,
                                           conceptCode,
                                           direction,
                                           false);
         return v;
	}


    // 031007
    public Vector<ConceptAssociation> getConceptAssociations(
			                                   String codingScheme,
	                                           String version,
	                                           String tag,
	                                           String conceptCode,
	                                           boolean direction,
	                                           String relation
	                                           )
	{
	         Vector<ConceptAssociation> v = getConceptAssociations(
			                                   codingScheme,
	                                           version,
	                                           tag,
	                                           conceptCode,
	                                           direction,
	                                           true, //080307
	                                           false,
	                                           relation);
	         return v;
	}


    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param version a coding scheme version
     * @param conceptCode
     * @param direction
     * @param returncode
     */
    public Vector<ConceptAssociation> getConceptAssociations(
		                                   String codingScheme,
                                           String version,
                                           String tag,
                                           String conceptCode,
                                           boolean direction,
                                           boolean returncode)
    {
		boolean forTreeTraversal = false;
        return getConceptAssociations(
		                                   codingScheme,
                                           version,
                                           tag,
                                           conceptCode,
                                           direction,
                                           returncode,
                                           forTreeTraversal
                                           );
 	}




     /**
      *
      * @since DLBExt 1.0
      * @param codingScheme a coding scheme name
      * @param relationship
      * @param association
      * @param conceptCode
      * @param direction
      * @param buildReferencedEntries
      */

     public Vector<CodedEntry> getAssociatedCodedEntries(String codingScheme,
                                           String relationship,
                                           String association,
                                           String conceptCode,
                                           boolean direction,
                                           boolean buildReferencedEntries)
    {
	    Vector<CodedEntry> v = new Vector<CodedEntry>();
	    try {
			Association asso = null;
			CodingSchemeVersionOrTag versionOrTag = createCodingSchemeVersionOrTag(version, tag);
			if (direction)
			{
				asso = lbConvMethods.getChildrenOf(conceptCode, relationship, association,
											   codingScheme, versionOrTag, buildReferencedEntries);
			}
			else
			{
				asso = lbConvMethods.getParentsOf(conceptCode, relationship, association,
											   codingScheme, versionOrTag, buildReferencedEntries);
			}
			if (asso != null)
			{
				AssociatedConceptList acl =	asso.getAssociatedConcepts();
				if (acl != null)
				{
					AssociatedConcept[] acs = acl.getAssociatedConcept();
					if (acs != null)
					{
						for (int i=0; i<acs.length; i++)
						{
							CodedEntry ce = acs[i].getReferencedEntry();
							if (ce == null)
							{
								getLogger().warn("WARNING: getReferencedEntry returns null");
							}
							else
							{
								v.add(ce);
							}
						}
					}
				}
		    }
	    } catch (Exception e) {
			e.printStackTrace();
			getLogger().error("Error encountered when calling getAssociatedCodedEntries.");
		}
		return v;
    }



    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param code
     */
    public Boolean codeExists(String codingScheme, String code)
    {
        try
        {
            CodedNodeSet cns = lbSvc.getCodingSchemeConcepts(codingScheme, createCodingSchemeVersionOrTag(version, tag));
            return cns.isCodeInSet(Constructors.createConceptReference(code, codingScheme));
        }
        catch (Exception e)
        {
            e.printStackTrace();
            return null;
        }
    }


///////////////////////
//  History
///////////////////////

    public boolean isValidHistoryDate(String codingScheme, Date historydate)
    {
		Vector v = getHistoryBaselineDates(codingScheme);
		for (int i=0;i<v.size(); i++)
		{
			java.util.Date d = (java.util.Date) v.elementAt(i);
			if (d.equals(historydate)) return true;
		}
		return false;
	}

    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     */
    public Vector<Date> getHistoryBaselineDates(String codingScheme)
    {
		Vector<Date> v = new Vector<Date>();
        try
        {
			HistoryService historySvc = lbSvc.getHistoryService(codingScheme);
            java.util.Date releasedAfter = null;
            java.util.Date releasedBefore = null;
 			SystemReleaseList srl = historySvc.getBaselines(releasedAfter,
                                releasedBefore);

            SystemRelease[] srs = srl.getSystemRelease();
            for (int i=0;i<srs.length; i++)
            {
				SystemRelease sr = srs[i];
				java.util.Date d = sr.getReleaseDate();
				v.add(d);
			}
        }
        catch (Exception e)
        {
			e.printStackTrace();
        }
        return v;
    }


    public java.util.Date getConceptCreationDate(String codingScheme, String code)
    {
        try
        {
			HistoryService historySvc = lbSvc.getHistoryService(codingScheme);
            CodingSchemeVersion csv = historySvc.getConceptCreationVersion(
				 Constructors.createConceptReference(code, null));

            return csv.getVersionDate();
        }
        catch (Exception e)
        {
			e.printStackTrace();
        }
        return null;
	}


    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param code
     */

    public Vector<NCIChangeEvent> getConceptHistory(String codingScheme,
                                    String code)
	{
        return getConceptHistory(codingScheme,
                                 code,
                                 getHistoryStartBaselineDate(codingScheme),
                                 getHistoryEndBaselineDate(codingScheme));
	}



    public Vector<NCIChangeEvent> getConceptHistory(String codingScheme,
                                    String code,
                                    java.util.Date beginDate,
                                    java.util.Date endDate)
    {
		Vector<NCIChangeEvent> v = new Vector<NCIChangeEvent>();
        try
        {
			HistoryService historySvc = lbSvc.getHistoryService(codingScheme);
			NCIChangeEventList cel = historySvc.getEditActionList(Constructors
                    .createConceptReference(code, null), beginDate, endDate);
            NCIChangeEvent[] nces = cel.getEntry();
            for (int i=0;i<nces.length; i++)
            {
				NCIChangeEvent nce = nces[i];
				v.add(nce);
			}
        }
        catch (Exception e)
        {
			e.printStackTrace();
        }
        return v;
    }


    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param code
     * @param csv
     */
    public Vector<NCIChangeEvent> getConceptHistory(String codingScheme,
                                  String code,
                                  CodingSchemeVersion csv)
    {
		Vector<NCIChangeEvent> v = new Vector<NCIChangeEvent>();
        try
        {
			HistoryService historySvc = lbSvc.getHistoryService(codingScheme);
			NCIChangeEventList cel = historySvc.getEditActionList(Constructors
                    .createConceptReference(code, null), csv);
            NCIChangeEvent[] nces = cel.getEntry();
            for (int i=0;i<nces.length; i++)
            {
				NCIChangeEvent nce = nces[i];
				v.add(nce);
			}
        }
        catch (Exception e)
        {
			e.printStackTrace();
        }
        return v;
    }


    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param code
     */
    public Vector<CodingSchemeVersion> getConceptChangeVersions(String codingScheme, String code)
    {
		Vector<CodingSchemeVersion> v = new Vector<CodingSchemeVersion>();
        try
        {
			HistoryService historySvc = lbSvc.getHistoryService(codingScheme);
            java.util.Date beginDate = null;
            java.util.Date endDate = null;

            ConceptReference ref = Constructors.createConceptReference(code, null);
			CodingSchemeVersionList csvl = historySvc.getConceptChangeVersions(
				ref, beginDate, endDate);

			CodingSchemeVersion[] csvs = csvl.getEntry();
            for (int i=0;i<csvs.length; i++)
            {
				CodingSchemeVersion csv = csvs[i];
				v.add(csv);
			}
        }
        catch (Exception e)
        {
			e.printStackTrace();
        }
        return v;
    }


    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param code
     */
    public CodingSchemeVersion getConceptCreateVersion(String codingScheme, String code)
    {
        try
        {
			HistoryService historySvc = lbSvc.getHistoryService(codingScheme);
//            java.util.Date beginDate = null;
//            java.util.Date endDate = null;

            ConceptReference ref = Constructors.createConceptReference(code, null);
            if (ref == null)
            {
				getLogger().warn("getConceptCreateVersion Constructors returns null");
				return null;
			}
            CodingSchemeVersion csv = null;
            try {
            	csv = historySvc.getConceptCreationVersion(ref);
			} catch (Exception ex) {
				ex.printStackTrace();
				return null;
			}
            return csv;

        }
        catch (Exception e)
        {
			e.printStackTrace();
        }
        return null;
    }


    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     */
    public java.util.Date getHistoryStartBaselineDate(String codingScheme)
    {
        try
        {
			HistoryService historySvc = lbSvc.getHistoryService(codingScheme);
			SystemRelease sr = historySvc.getEarliestBaseline();
			java.util.Date d = sr.getReleaseDate();
			return d;
        }
        catch (Exception e)
        {
 			e.printStackTrace();
 		}
 		return null;
    }


    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     */
    public java.util.Date getHistoryEndBaselineDate(String codingScheme)
    {
        try
        {
			HistoryService historySvc = lbSvc.getHistoryService(codingScheme);
			SystemRelease sr = historySvc.getLatestBaseline();
			java.util.Date d = sr.getReleaseDate();
			return d;
        }
        catch (Exception e)
        {
 			e.printStackTrace();
 		}
 		return null;
    }


	public Vector<Date> getConceptEditActionDates(String codingScheme, String code, int iAction)
	{
		Vector<Date> date_vec = new Vector<Date>();
        Vector<NCIChangeEvent> v = getConceptHistory(codingScheme,
                                    code,
                                    getHistoryStartBaselineDate(codingScheme),
                                    getHistoryEndBaselineDate(codingScheme));
        for (int i=0; i<v.size(); i++)
        {
			NCIChangeEvent nce = (NCIChangeEvent) v.elementAt(i);
			int actionId = nce.getEditaction().getType();
			if (actionId == iAction)
			{
				date_vec.add(nce.getEditDate());
			}
		}

		return date_vec;
	}


	/**
	 * Extracts an edit action string (for example, 'CREATE') from a
	 * edit action constance string (form example, 'EDIT_ACTION_CREATE_STR') corresponding
	 * to the specified edit action code.
	 *
	 * @param actionId
	 * @return an edit action string (for example, 'CREATE')
	 * @since dlbExt 1.0
	 */
	protected String iActiontoStr(int actionId)
	{
		String action = "";
		if (actionId == ChangeType.CREATE_TYPE)
		{
		    action = "Create";
		}
		else if (actionId == ChangeType.MERGE_TYPE)
		{
		    action = "Merge";
		}
		else if (actionId == ChangeType.MODIFY_TYPE)
		{
		    action = "Modify";
		}
		else if (actionId == ChangeType.RETIRE_TYPE)
		{
		    action = "Retire";
		}
		else if (actionId == ChangeType.SPLIT_TYPE)
		{
		    action = "Split";
		}
		return action;
	}


	public int toActionIndex(String action)
	{
		int iAction = -1;
		if (action.compareToIgnoreCase("Create") == 0)
		{
		    iAction = ChangeType.CREATE_TYPE;
		}
		else if (action.compareToIgnoreCase("Merge") == 0)
		{
		    iAction = ChangeType.MERGE_TYPE;
		}
		else if (action.compareToIgnoreCase("Modify") == 0)
		{
		    iAction = ChangeType.MODIFY_TYPE;
		}
		else if (action.compareToIgnoreCase("Retire") == 0)
		{
		    iAction = ChangeType.RETIRE_TYPE;
		}
		else if (action.compareToIgnoreCase("Split") == 0)
		{
		    iAction = ChangeType.SPLIT_TYPE;
		}
		return iAction;
	}



	protected String dateToString(java.util.Date aDate)
	{
		if (aDate == null) return null;
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
		String theDate = sdf.format(aDate);
		return theDate;
	}


	public Vector<EditActionDate> getConceptEditAction(String codingScheme, String code)
	{
		Vector<EditActionDate> w = new Vector<EditActionDate>();
        Vector<NCIChangeEvent> v = getConceptHistory(codingScheme,
                                    code,
                                    getHistoryStartBaselineDate(codingScheme),
                                    getHistoryEndBaselineDate(codingScheme));
        for (int i=0; i<v.size(); i++)
        {
			NCIChangeEvent nce = (NCIChangeEvent) v.elementAt(i);
			EditActionDate e = new EditActionDate();
			e.setEditDate(nce.getEditDate());
			e.setAction(nce.getEditaction().getType());
			w.add(e);
		}
		return w;
	}


	public Boolean isConceptEdited(
		                String codingScheme,
						String code,
						boolean mrg_splt,
						java.util.Date beginDate,
						java.util.Date endDate)
	{
        try
        {
			HistoryService historySvc = lbSvc.getHistoryService(codingScheme);
			NCIChangeEventList cel = historySvc.getEditActionList(Constructors
                    .createConceptReference(code, null), beginDate, endDate);
            NCIChangeEvent[] nces = cel.getEntry();
//            int knt = 0;
            for (int i=0;i<nces.length; i++)
            {
				NCIChangeEvent nce = nces[i];
				int actionId = nce.getEditaction().getType();
				if (mrg_splt)
				{
					if (actionId == ChangeType.SPLIT_TYPE ||
					    actionId == ChangeType.MERGE_TYPE)
					{
						return Boolean.TRUE;
					}
				}
				else
				{
					if (actionId != ChangeType.CREATE_TYPE) return Boolean.TRUE;
				}
			}

        }
        catch (Exception e)
        {
			e.printStackTrace();
        }
        return Boolean.FALSE;
	}


	public Integer getNumberEditActions(
		                String codingScheme,
						String code,
						java.util.Date baselineDate_in)
	{
		return getNumberEditActions(
		                codingScheme,
						code,
						baselineDate_in,
						baselineDate_in);
	}


	public Integer getNumberEditActions(
		                String codingScheme,
						String code,
						java.util.Date beginDate,
						java.util.Date endDate)
	{
        try
        {
			HistoryService historySvc = lbSvc.getHistoryService(codingScheme);
			NCIChangeEventList cel = historySvc.getEditActionList(Constructors
                    .createConceptReference(code, null), beginDate, endDate);
            NCIChangeEvent[] nces = cel.getEntry();
            return new Integer(nces.length);
        }
        catch (Exception e)
        {
			e.printStackTrace();
        }
        return null;
	}


	public Vector<String> getAncestorCodeActionDates(
		                String codingScheme,
						String inCode,
						java.util.Date iBaselineDate_in,
						java.util.Date fBaselineDate_in)
	{
		Vector<String> v = new Vector<String>();
        try
        {
			java.util.Date beginDate = iBaselineDate_in;
			java.util.Date endDate = fBaselineDate_in;

			HistoryService historySvc = lbSvc.getHistoryService(codingScheme);
			NCIChangeEventList cel = historySvc.getEditActionList(Constructors
                    .createConceptReference(inCode, null), beginDate, endDate);
            NCIChangeEvent[] nces = cel.getEntry();
            boolean isRetired = false;
            boolean retire_thru_merge = false;
            boolean create_thru_split = false;
            String merge_into = null;
            java.util.Date retire_date = null;
            java.util.Date merge_into_date = null;
            String split_from = null;
            java.util.Date split_from_date = null;

            for (int i=0;i<nces.length; i++)
            {
				NCIChangeEvent nce = nces[i];
                String code = nce.getConceptcode();
                String ref_code = nce.getReferencecode();
                int actionId =	nce.getEditaction().getType();
                if (code.compareTo(inCode) == 0 &&
                    actionId == ChangeType.RETIRE_TYPE)
                {
					isRetired = true;
					retire_date = nce.getEditDate();
				}
                else if (code.compareTo(inCode) == 0 &&
                         actionId == ChangeType.MERGE_TYPE &&
                         code.compareTo(ref_code) != 0)
				{
					retire_thru_merge = true;
					merge_into = ref_code;
					merge_into_date = nce.getEditDate();
					String s = dateToString(merge_into_date);
					v.add(ref_code+"|"+s);
					return v;
				}
                else if (ref_code != null && ref_code.compareTo(inCode) == 0 &&
                         actionId == ChangeType.SPLIT_TYPE &&
                         code.compareTo(ref_code) != 0)
				{
					create_thru_split = true;
					split_from = code;
					split_from_date = nce.getEditDate();
				}
			}

			if (isRetired && !retire_thru_merge)
			{
	            for (int i=0;i<nces.length; i++)
                {
					NCIChangeEvent nce = nces[i];
					String code = nce.getConceptcode();
					String ref_code = nce.getReferencecode();
					int actionId =	nce.getEditaction().getType();
					if (code.compareTo(inCode) == 0 &&
						actionId == ChangeType.RETIRE_TYPE)
					{
						if (ref_code != null && ref_code.compareTo("") != 0)
						{
							retire_date = nce.getEditDate();
							String s = dateToString(retire_date);
							v.add(ref_code+"|"+s);
						}
					}
				}
				return v;
			}

        }
        catch (Exception e)
        {
			e.printStackTrace();
        }
        return v;

	}

	public Vector getCodeActionParents(
		                String codingScheme,
						String inCode,
						java.util.Date baselineDate_in)
	{
				 return getAncestorCodes(
		                codingScheme,
						inCode,
						true,
						baselineDate_in,
						baselineDate_in,
						true);
	}




	public Vector getAncestorCodes(
		                String codingScheme,
						String inCode,
						boolean atBaseline,
						java.util.Date iBaselineDate_in,
						java.util.Date fBaselineDate_in)
	{
		boolean split_merge_only = false;
				 return getAncestorCodes(
		                codingScheme,
						inCode,
						atBaseline,
						iBaselineDate_in,
						fBaselineDate_in,
						split_merge_only);

	}


	public Vector<String> getAncestorCodes(
		                String codingScheme,
						String inCode,
						boolean atBaseline,
						java.util.Date iBaselineDate_in,
						java.util.Date fBaselineDate_in,
						boolean split_merge_only)
	{
		Vector<String> codes = new Vector<String>();
        try
        {
			java.util.Date beginDate = iBaselineDate_in;
			java.util.Date endDate = fBaselineDate_in;
			if (atBaseline) endDate = iBaselineDate_in;

			HistoryService historySvc = lbSvc.getHistoryService(codingScheme);
			NCIChangeEventList cel = historySvc.getEditActionList(Constructors
                    .createConceptReference(inCode, null), beginDate, endDate);
            NCIChangeEvent[] nces = cel.getEntry();
            boolean isRetired = false;
            boolean retire_thru_merge = false;
//            boolean create_thru_split = false;
//            String merge_into = null;
            java.util.Date retire_date = null;
//            java.util.Date merge_into_date = null;
//            String split_from = null;
//            java.util.Date split_from_date = null;

            for (int i=0;i<nces.length; i++)
            {
				NCIChangeEvent nce = nces[i];
                String code = nce.getConceptcode();
                String ref_code = nce.getReferencecode();
                int actionId =	nce.getEditaction().getType();

                if (code.compareTo(inCode) == 0 && actionId == ChangeType.RETIRE_TYPE)
                {
					isRetired = true;
					retire_date = nce.getEditDate();
				}
				else if (actionId == ChangeType.MERGE_TYPE)
				{
					retire_thru_merge = true;
					if (ref_code != null && ref_code.compareTo(inCode) == 0 && ref_code.compareTo(code) != 0)
					{
						codes.add(code);
						codes.add(ref_code);
					}
				}

				/*
                else if (code.compareTo(inCode) == 0 &&
                         actionId == ChangeType.MERGE_TYPE &&
                         code.compareTo(ref_code) != 0)
				{
					retire_thru_merge = true;
					merge_into = ref_code;
					merge_into_date = nce.getEditDate();
					codes.add(ref_code);
					return codes;
				}
                else if (ref_code != null && ref_code.compareTo(inCode) == 0 &&
                         actionId == ChangeType.SPLIT_TYPE &&
                         code.compareTo(ref_code) != 0)
				{
					create_thru_split = true;
					split_from = code;
					split_from_date = nce.getEditDate();
				}
				*/
				else if (actionId == ChangeType.SPLIT_TYPE)
				{
					if (ref_code != null && ref_code.compareTo(inCode) != 0)
					{
						codes.add(inCode);
					}
					else if (ref_code != null && ref_code.compareTo(inCode) == 0 && ref_code.compareTo(code) != 0)
					{
						codes.add(code);
					}
				}
			}

			if (isRetired && !retire_thru_merge && !split_merge_only)
			{
	            for (int i=0;i<nces.length; i++)
                {
					NCIChangeEvent nce = nces[i];
					String code = nce.getConceptcode();
					String ref_code = nce.getReferencecode();
					int actionId =	nce.getEditaction().getType();
					if (code.compareTo(inCode) == 0 && actionId == ChangeType.RETIRE_TYPE)
					{
						if (ref_code != null && ref_code.compareTo("") != 0)
						{
							codes.add(ref_code);
						}
					}
				}
				return codes;
			}

        }
        catch (Exception e)
        {
			e.printStackTrace();
        }
        return codes;
	}





	public Vector<String> getDescendantCodeActionDates(
		                String codingScheme,
						String inCode,
						java.util.Date iBaselineDate_in,
						java.util.Date fBaselineDate_in)
	{
		Vector<String> v = new Vector<String>();
        try
        {
			java.util.Date beginDate = iBaselineDate_in;
			java.util.Date endDate = fBaselineDate_in;

			HistoryService historySvc = lbSvc.getHistoryService(codingScheme);
			NCIChangeEventList cel = historySvc.getEditActionList(Constructors
                    .createConceptReference(inCode, null), beginDate, endDate);
            NCIChangeEvent[] nces = cel.getEntry();
//            boolean isRetired = false;
//            boolean retire_thru_merge = false;
//            boolean create_thru_split = false;
//            String merge_into = null;
            java.util.Date retire_date = null;
//            java.util.Date merge_into_date = null;
//            String split_from = null;
//            java.util.Date split_from_date = null;

            for (int i=0;i<nces.length; i++)
            {
				NCIChangeEvent nce = nces[i];
                String code = nce.getConceptcode();
                String ref_code = nce.getReferencecode();
                int actionId =	nce.getEditaction().getType();
                if (ref_code != null && ref_code.compareTo(inCode) == 0 &&
                    actionId == ChangeType.RETIRE_TYPE)
                {
					retire_date = nce.getEditDate();
					String s = dateToString(retire_date);
					v.add(ref_code+"|"+s);
				}
                else if (code.compareTo(inCode) == 0 &&
                         actionId == ChangeType.SPLIT_TYPE &&
                         code.compareTo(ref_code) != 0)
				{
					Date split_date = nce.getEditDate();
					String s = dateToString(split_date);
					v.add(ref_code+"|"+s);
				}
			}
        }
        catch (Exception e)
        {
			e.printStackTrace();
        }
        return v;
	}


	public Vector getCodeActionChildren(
		                String codingScheme,
						String inCode,
						java.util.Date baselineDate_in)
	{
				 return getDescendantCodes(
		                codingScheme,
						inCode,
						true,
						baselineDate_in,
						baselineDate_in,
						true);
	}


	public Vector getDescendantCodes(
		                String codingScheme,
						String inCode,
						boolean atBaseline,
						java.util.Date iBaselineDate_in,
						java.util.Date fBaselineDate_in)
	{
         boolean merge_split_only = false;
 	     return getDescendantCodes(
		                codingScheme,
						inCode,
						atBaseline,
						iBaselineDate_in,
						fBaselineDate_in,
						merge_split_only);

	}


	public Vector<String> getDescendantCodes(
		                String codingScheme,
						String inCode,
						boolean atBaseline,
						java.util.Date iBaselineDate_in,
						java.util.Date fBaselineDate_in,
						boolean merge_split_only)
	{
		Vector<String> codes = new Vector<String>();
        try
        {
			java.util.Date beginDate = iBaselineDate_in;
			java.util.Date endDate = fBaselineDate_in;
			if (atBaseline) endDate = iBaselineDate_in;

			HistoryService historySvc = lbSvc.getHistoryService(codingScheme);
			NCIChangeEventList cel = historySvc.getEditActionList(Constructors
                    .createConceptReference(inCode, null), beginDate, endDate);

            NCIChangeEvent[] nces = cel.getEntry();

//            boolean isRetired = false;
//            boolean retire_thru_merge = false;
//            boolean create_thru_split = false;
//            String merge_into = null;
//            java.util.Date retire_date = null;
//            java.util.Date merge_into_date = null;
//            String split_from = null;
//            java.util.Date split_from_date = null;

            for (int i=0;i<nces.length; i++)
            {
				NCIChangeEvent nce = nces[i];
                String code = nce.getConceptcode();
                String ref_code = nce.getReferencecode();

                int actionId =	nce.getEditaction().getType();
                if (ref_code != null && ref_code.compareTo(inCode) == 0 &&
                    actionId == ChangeType.RETIRE_TYPE)
                {
					if (!merge_split_only)
					{
						codes.add(code);
				    }
				}

				else if (actionId == ChangeType.SPLIT_TYPE)
				{
					if (ref_code != null && code.compareTo(inCode) == 0)
					{
						codes.add(code);
						codes.add(ref_code);
					}
				}

				else if (actionId == ChangeType.MERGE_TYPE)
				{
					if (ref_code != null && ref_code.compareTo(inCode) == 0 && ref_code.compareTo(code) == 0)
					{
						codes.add(code);
					}
					else if (ref_code != null && code.compareTo(inCode) == 0)
					{
						codes.add(ref_code);
					}
				}


			}
        }
        catch (Exception e)
        {
			e.printStackTrace();
        }
        return codes;
	}





    // 073006 (v1rc6 addition)
 	public Vector<String> getAncestors(String codingScheme, String inCode)
 	{
 		 Vector<String> codes = new Vector<String>();
         try
         {
 			 HistoryService historySvc = lbSvc.getHistoryService(codingScheme);
 			 NCIChangeEventList cel = historySvc.getAncestors(Constructors
                     .createConceptReference(inCode, null));

             NCIChangeEvent[] nces = cel.getEntry();
             for (int i=0;i<nces.length; i++)
             {
 				 NCIChangeEvent nce = nces[i];
                 String ref_code = nce.getReferencecode();
                 codes.add(ref_code);
			 }
			 return codes;

         }
         catch (Exception e)
         {
 			e.printStackTrace();
         }
         return codes;
	}

	// 073006 (v1rc6 addition)
 	public Vector<String> getDescendants(String codingScheme, String inCode)
 	{
 		 Vector<String> codes = new Vector<String>();
         try
         {
 			 HistoryService historySvc = lbSvc.getHistoryService(codingScheme);
 			 NCIChangeEventList cel = historySvc.getDescendants(Constructors
                     .createConceptReference(inCode, null));

             NCIChangeEvent[] nces = cel.getEntry();
             for (int i=0;i<nces.length; i++)
             {
 				 NCIChangeEvent nce = nces[i];
                 String ref_code = nce.getReferencecode();
                 codes.add(ref_code);
			 }
			 return codes;

         }
         catch (Exception e)
         {
 			e.printStackTrace();
         }
         return codes;
	}



	public Vector<String> getSupportedPropertyQualifier(String codingScheme)
	{
		if (codingScheme == null) return null;
        Vector<String> v = new Vector<String>();
        try {
			CodingScheme cs = getCodingScheme(codingScheme);
			org.LexGrid.naming.SupportedPropertyQualifier[] supportedPropertyQualifiers
			    = cs.getMappings().getSupportedPropertyQualifier();
			if (supportedPropertyQualifiers == null) return null;
			for (int i=0; i<supportedPropertyQualifiers.length; i++)
			{
				SupportedPropertyQualifier q = supportedPropertyQualifiers[i];
				//v.add(q.getContent());
				v.add(q.getLocalId());
			}
			return v;
	    } catch (Exception e) {
			return null;
		}
    }

	public Vector<String> getSupportedAssociationQualifier(String codingScheme)
	{
		if (codingScheme == null) return null;
        Vector<String> v = new Vector<String>();
        try {
			CodingScheme cs = getCodingScheme(codingScheme);
			org.LexGrid.naming.SupportedAssociationQualifier[] supportedAssociationQualifiers
			    = cs.getMappings().getSupportedAssociationQualifier();
			if (supportedAssociationQualifiers == null) return null;
			for (int i=0; i<supportedAssociationQualifiers.length; i++)
			{
				SupportedAssociationQualifier q = supportedAssociationQualifiers[i];
				//v.add(q.getContent());
				v.add(q.getLocalId());
			}
			return v;
	    } catch (Exception e) {
			return null;
		}
	}




    public int getSupportedContextCount(String codingScheme, String version, String tag)
	{
		CodingScheme cs = getCodingScheme(codingScheme, version, tag);
        if (cs == null) return -1;
        return getSupportedContextCount(cs);
	}


    public int getSupportedContextCount(CodingScheme cs)
 	{
 		if (cs == null) return -1;
//        Vector v = new Vector();
 	    int n = cs.getMappings().getSupportedContextCount();
        return n;
	}


	// rc7
    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param version a coding scheme version
     * @param tag
     */


    public Vector getSupportedContext(String codingScheme, String version, String tag)
	{
		CodingScheme cs = getCodingScheme(codingScheme, version, tag);
        if (cs == null) return null;
        return getSupportedContextNames(cs);
	}


    public Vector<SupportedContext> getSupportedContext(CodingScheme cs)
 	{
 		if (cs == null) return null;
        Vector<SupportedContext> v = new Vector<SupportedContext>();
 	    SupportedContext[] contexts = cs.getMappings().getSupportedContext();
 		for (int i=0; i<contexts.length; i++)
 		{
 		     SupportedContext sc = (SupportedContext) contexts[i];
 		     v.add(sc);
 		}
        return v;
	}


    public Vector getSupportedContextNames(CodingScheme cs)
 	{
        Vector<SupportedContext> w = getSupportedContext(cs);
 		if (w == null) return null;

        Vector<String> v = new Vector<String>();
 		for (int i=0; i<w.size(); i++)
 		{
 		     SupportedContext sc = (SupportedContext) w.elementAt(i);
 		     v.add(sc.getContent());
 		}
        return v;
	}





    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     * @param version a coding scheme version
     * @param tag
     */
    public Vector getSupportedSource(String codingScheme, String version, String tag)
	{
        CodingScheme cs = getCodingScheme(codingScheme, version, tag);
        if (cs == null) return null;
        return getSupportedSource(cs);
	}


    public Vector<SupportedSource> getSupportedSource(CodingScheme cs)
 	{
 		if (cs == null) return null;
        Vector<SupportedSource> v = new Vector<SupportedSource>();
 	    SupportedSource[] sources = cs.getMappings().getSupportedSource();
 		for (int i=0; i<sources.length; i++)
 		{
 		     SupportedSource src = (SupportedSource) sources[i];
 		     v.add(src);
 		}
        return v;
	}


    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme
     * @param version
     * @param tag
     */

    public Vector getSupportedSourceNames(String codingScheme, String version, String tag)
	{
        CodingScheme cs = getCodingScheme(codingScheme, version, tag);
        if (cs == null) return null;
        return getSupportedSourceNames(cs);
	}

    public Vector<String> getSupportedSourceNames(CodingScheme cs)
 	{
        Vector<SupportedSource> w = getSupportedSource(cs);
 		if (w == null) return null;

        Vector<String> v = new Vector<String>();
 		for (int i=0; i<w.size(); i++)
 		{
 		     SupportedSource src = (SupportedSource) w.elementAt(i);
 		     //v.add(src.getContent());
 		     v.add(src.getLocalId());
 		}
        return v;
	}


   public Vector<CodedEntry> findCodedEntriesWithSourceCodeMatching(String codingScheme, String sourceAbbr, String code, int maxToReturn)
   {
        return findCodedEntriesWithSourceCodeMatching(codingScheme, version, tag, sourceAbbr, code, maxToReturn);
   }

   public Vector<CodedEntry> findCodedEntriesWithSourceCodeMatching(String codingScheme, String version, String tag,
                                                        String sourceAbbr, String code, int maxToReturn)
   {
		CodingScheme cs = getCodingScheme(codingScheme);
        CodingSchemeVersionOrTag versionOrTag = createCodingSchemeVersionOrTag(version, tag);

        LocalNameList contextList = null;
        NameAndValueList qualifierList = null;
        Vector supported_qualifiers = getSupportedPropertyQualifiers(cs);
        if (supported_qualifiers.contains("source-code"))
        {
			if (code != null && code.compareTo("") != 0)
			{
				qualifierList = new NameAndValueList();
				qualifierList.addNameAndValue(createNameAndValue("source-code", code));
			}
		}

     // propertyLnL
        Vector<String> w1 = new Vector<String>();
        String prop_name = "FULL_SYN";
        if (hasProperty(codingScheme, prop_name))
        {
	        w1.add(prop_name);
		}
		else
		{
			prop_name = "textualPresentation";
			if (hasProperty(codingScheme, prop_name))
			{
				w1.add(prop_name);
			}
		}
        LocalNameList propertyLnL = vector2LocalNameList(w1);

     // sourceLnL
        Vector<String> w2 = new Vector<String>();
        w2.add(sourceAbbr);
        LocalNameList sourceLnL = vector2LocalNameList(w2);

        if (sourceAbbr.compareTo("*") == 0)
        {
			sourceLnL = null;
		}

        Vector<CodedEntry> v = new Vector<CodedEntry>();

		SortOptionList sortCriteria =
			    Constructors.createSortOptionList(new String[]{"matchToQuery", "code"});

		try {
			CodedNodeSet cns = lbSvc.getCodingSchemeConcepts(codingScheme, versionOrTag);//, false);
			if (cns == null) return new Vector<CodedEntry>();
			CodedNodeSet.PropertyType[] types = new PropertyType[] {PropertyType.PRESENTATION};
			cns = cns.restrictToProperties(propertyLnL, types, sourceLnL, contextList, qualifierList);
            ResolvedConceptReferenceList rcrl = cns.resolveToList(sortCriteria, propertyLnL, types, maxToReturn);
			ResolvedConceptReference[] rcrs = rcrl.getResolvedConceptReference();
			for (int i=0; i<rcrs.length; i++)
			{
				 ResolvedConceptReference rcr = rcrs[i];
				 CodedEntry ce = rcr.getReferencedEntry();
				 v.add(ce);
			}

		} catch (Exception e) {
			 getLogger().error("ERROR: Exception in findCodedEntriesWithSourceCodeMatching.");
			 return null;
		}
		return v;
    }

    public static CodingSchemeVersionOrTag createProductionTag()
    {
        CodingSchemeVersionOrTag tag = new CodingSchemeVersionOrTag();
        //tag.setTag(LBConstants.KnownTags.PRODUCTION.toString());
        tag.setTag("PRODUCTION");
        return tag;
    }

   public static SortOption createSortOption(String sortExtensionName, Boolean ascending)
    {
        SortOption result = new SortOption();
        result.setExtensionName(sortExtensionName);
        result.setAscending(ascending);
        return result;
    }

    public static SortOptionList createSortOptionList(String[] sortExtensionNames, Boolean[] ascending) throws LBParameterException
    {
        SortOptionList result = new SortOptionList();
        if (sortExtensionNames == null || ascending == null || sortExtensionNames.length != ascending.length)
        {
            throw new LBParameterException(
                    "Invalid Parameters - sortExtensionNames array must be the same length as the ascending array - and neither can be null");
        }
        for (int i = 0; i < ascending.length; i++)
        {
            SortOption temp = new SortOption();
            temp.setExtensionName(sortExtensionNames[i]);
            temp.setAscending(ascending[i]);
            result.addEntry(temp);
        }

        return result;
    }

    public static SortOptionList createSortOptionList(String[] sortExtensionNames)
    {
        SortOptionList result = new SortOptionList();
        if (sortExtensionNames == null)
        {
            return null;
        }
        for (int i = 0; i < sortExtensionNames.length; i++)
        {
            SortOption temp = new SortOption();
            temp.setExtensionName(sortExtensionNames[i]);
            temp.setAscending(null);
            result.addEntry(temp);
        }

        return result;
    }


    public String getSemanticTypeCode(String name)
    {
		if (semanticTypeName2Code_hashmap == null)
		{
			initializeSemanticTypes();
		}
		if (semanticTypeName2Code_hashmap.containsKey(name))
		{
			String code = (String) semanticTypeName2Code_hashmap.get(name);
			return code;
		}
		else
		{
			return "Unknown";
		}
	}

    public Vector<String> getSemanticTypeCodes(Vector names)
    {
		Vector<String> u = new Vector<String>();
		for (int i=0; i<names.size(); i++)
		{
			String name = (String) names.elementAt(i);
			u.add(getSemanticTypeCode(name));
		}
		return u;
	}

	public boolean initializeSemanticTypes()
	{
		//LexBIGService lbSvc = new LexBIGServiceImpl();
		semanticType_vec = new Vector<String>();
		semanticType_hashmap = new HashMap<String,String>();
		semanticTypeName2Code_hashmap = new HashMap<String,String>();
		try {

			CodedNodeSet cns = lbSvc.getCodingSchemeConcepts(
					SEMANTIC_NET, createProductionTag());//, true);
            if (cns == null)
            {
				getLogger().error("SEMANTIC_NET data are either not available or accessible.");
				return false;
			}
			for (
				ResolvedConceptReferencesIterator concepts = cns.resolve(
					createSortOptionList(new String[] {"code"}),
					new LocalNameList(), null);
				concepts.hasNext(); )
			{
				ResolvedConceptReference ref = concepts.next();
				if (ref != null)
				{
					String semantictype = ref.getEntityDescription().getContent();
					if (semantictype.compareTo("topThing") != 0)
					{
						semanticType_vec.add(ref.getEntityDescription().getContent());
						semanticType_hashmap.put(ref.getConceptCode(), ref.getEntityDescription().getContent());
						semanticTypeName2Code_hashmap.put(ref.getEntityDescription().getContent(), ref.getConceptCode());
					}
			    }

			}
			return true;
	    } catch (Exception e) {
			//e.printStackTrace();
			getLogger().error("ERROR: Unable to initialize semantic types.");
			return false;
		}
		//return v;
	}

	public HashMap getSemanticTypes()
	{
		if (semanticType_hashmap == null)
		{
			boolean retval = initializeSemanticTypes();
			if (!retval) return null;
		}
		return semanticType_hashmap;
	}

    public int getChangeTypeNumber(String s)
    {
		if (s.compareToIgnoreCase("CREATE") == 0)
		{
			return ChangeType.CREATE_TYPE;
		}
		else if (s.compareToIgnoreCase("MERGE") == 0)
		{
			return ChangeType.MERGE_TYPE;
		}
		else if (s.compareToIgnoreCase("MODIFY") == 0)
		{
			return ChangeType.MODIFY_TYPE;
		}
		else if (s.compareToIgnoreCase("RETIRE") == 0)
		{
			return ChangeType.RETIRE_TYPE;
		}
		else if (s.compareToIgnoreCase("SPLIT") == 0)
		{
			return ChangeType.SPLIT_TYPE;
		}
		return -1;
	}

	protected boolean hasProperty(String codingScheme, String property)
	{
		CodingScheme cs = getCodingScheme(codingScheme);
		SupportedProperty[] properties = cs.getMappings().getSupportedProperty();
		for (int i=0; i<properties.length; i++)
		{
			//String name = properties[i].getContent();
			String name = properties[i].getLocalId();
			if (name.compareTo(property) == 0) return true;
		}
		return false;

	}



    // 090506
    protected CodedNodeSet createCodeNodeSet(String[] conceptCodes, String codingScheme,
        String version, String tag)
    {
		CodingSchemeVersionOrTag versionOrTag = createCodingSchemeVersionOrTag(version, tag);
        try {
			ConceptReferenceList crl = createConceptReferenceList(conceptCodes, codingScheme);
			CodedNodeSet cns = lbSvc.getCodingSchemeConcepts(codingScheme, versionOrTag);//, false);
			cns = cns.restrictToCodes(crl);
			return cns;
	    } catch (Exception e) {
			return null;
		}
    }

    // 090506
    protected Association[] getChildrenOf(String conceptCode, String relationship, String codingScheme,
            String version, String tag, boolean buildReferencedEntries) //throws LBException
    {
		CodingSchemeVersionOrTag versionOrTag = createCodingSchemeVersionOrTag(version, tag);
	 	try {
			CodedNodeGraph cng = lbSvc.getNodeGraph(codingScheme, versionOrTag, relationship);
			cng = cng.restrictToSourceCodes(createCodeNodeSet(new String[]{conceptCode}, codingScheme, version, tag));
			int buildRefEntries = -1;
			if (buildReferencedEntries)
			{
				buildRefEntries = 1;
			}
			ResolvedConceptReferenceList rcrl = cng.resolveAsList(createConceptReference(conceptCode, codingScheme),
																  true, false, buildRefEntries, 1, null, null, null, 0);
			if (rcrl != null)
			{
				if (rcrl.getResolvedConceptReferenceCount() > 0)
				{
					AssociationList al = rcrl.getResolvedConceptReference(0).getSourceOf();
					Association[] associations = al.getAssociation();
					return associations;

				}
		    }
		} catch (Exception e) {
            getLogger().warn("WARNING: DLBWrapper getChildrenOf throws exception.");
	    }
        return null;
    }

    // 090506
    protected Association[] getParentsOf(String conceptCode, String relationship, String codingScheme,
                                     String version, String tag, boolean buildReferencedEntries)
    {
		CodingSchemeVersionOrTag versionOrTag = createCodingSchemeVersionOrTag(version, tag);
	 	try {
			 CodedNodeGraph cng = lbSvc.getNodeGraph(codingScheme, versionOrTag, relationship);
             cng = cng.restrictToTargetCodes(createCodeNodeSet(new String[]{conceptCode}, codingScheme, version, tag));
			int buildRefEntries = -1;
			if (buildReferencedEntries)
			{
				buildRefEntries = 1;
			}
            ResolvedConceptReferenceList rcrl = null;
			rcrl = cng.resolveAsList(createConceptReference(conceptCode,codingScheme),
																  false, true, buildRefEntries, 1, null, null, null, 0);
			if (rcrl != null)
			{
				if (rcrl.getResolvedConceptReferenceCount() > 0)
				{
					AssociationList al = rcrl.getResolvedConceptReference(0).getTargetOf();
					Association[] associations = al.getAssociation();
					return associations;
				}
		    }
		} catch (Exception e) {
            getLogger().warn("WARNING: DLBWrapper getParentsOf throws exception.");
	    }

		return null;
    }


    // 031207
    public Vector<ConceptAssociation> getConceptAssociations(
		                           String codingScheme,
                                           String version,
                                           String tag,
                                           String conceptCode,
                                           boolean direction,
                                           boolean returncode,
                                           boolean forTreeTraversal)
    {
    	return getConceptAssociations(
		                                   codingScheme,
                                           version,
                                           tag,
                                           conceptCode,
                                           direction,
                                           returncode,
                                           forTreeTraversal,
					                       null);
    }


    // 090506
    public Vector<ConceptAssociation> getConceptAssociations(
		                                   String codingScheme,
                                           String version,
                                           String tag,
                                           String conceptCode,
                                           boolean direction,
                                           boolean returncode,
                                           boolean forTreeTraversal,
                                           String relation)
    {
		if (relation != null)
		{
			if (relation.compareToIgnoreCase(ROLE_RELATION) == 0 && !hasRelation(codingScheme, ROLE_RELATION))
			{
				return new Vector<ConceptAssociation>();
			}
		}

	    String relationName = relation;
        if (relation != null)
        {
			if (!hasRelation(codingScheme, version, tag, relation))
			{
				relationName = null;//"Relations";//null;
			}

	    }

        CodingSchemeVersionOrTag versionOrTag
            = createCodingSchemeVersionOrTag(version, null);

        //boolean broader_to_narrower = direction; // traverse_down
        //if (!direction) broader_to_narrower = false;

        boolean traverse_dir_isa = direction;
        boolean traverse_dir_role = direction;

        if (forTreeTraversal)
        {
			traverse_dir_isa = direction;
			traverse_dir_role = !direction;
		}

		boolean traverse_dir;
	    Vector<ConceptAssociation> v = new Vector<ConceptAssociation>();
	    try {
			CodingScheme scheme = lbSvc.resolveCodingScheme(codingScheme, versionOrTag);
			if (scheme == null)
			{
				getLogger().warn("WARNING: Unable to resolve coding scheme,");
				return null;
			}

			// IS_A
            //String associationname = DLBWrapper.IS_A;
            String is_a = getHasSubtype(codingScheme);
            String associationname = is_a;
            traverse_dir = traverse_dir_isa;

			Association asso = null;
			if (traverse_dir)
			{
				asso = lbConvMethods.getChildrenOf(conceptCode, relationName, associationname,
											   codingScheme, createCodingSchemeVersionOrTag(version, tag), false);
			}
			else
			{
				asso = lbConvMethods.getParentsOf(conceptCode, relationName, associationname,
											   codingScheme, createCodingSchemeVersionOrTag(version, tag), false);
			}

			if (asso != null)
			{
				AssociatedConceptList acl =	asso.getAssociatedConcepts();
				if (acl != null)
				{
					AssociatedConcept[] acs = acl.getAssociatedConcept();
					if (acs != null)
					{
						for (int k=0; k<acs.length; k++)
						{
							if (acs[k] != null)
							{
	                            gov.nih.nci.lexbig.ext.ConceptAssociation concept_asso = convertConceptAssociation(codingScheme,
	                                                                    version,
	                                                                    associationname,
	                                                                    acs[k],
	                                                                    returncode);
	                            v.add(concept_asso);

							}
						}
					}
				}
			}

			// Other associations
			Association[] associations = null;
			traverse_dir = traverse_dir_role;
			if (traverse_dir)
			{
				associations = getChildrenOf(conceptCode, relationName, codingScheme, version, tag, false);
			}
			else
			{
				associations = getParentsOf(conceptCode, relationName, codingScheme, version, tag, false);
			}

			if (associations != null)
			{
				for (int i=0; i<associations.length; i++)
				{
					asso = associations[i];
					associationname = asso.getAssociationName();
					//if (associationname.compareTo(DLBWrapper.IS_A) != 0)

					if (is_a != null)
					{
						if (associationname.compareTo(is_a) != 0)
						{
							AssociatedConceptList acl =	asso.getAssociatedConcepts();
							if (acl != null)
							{
								AssociatedConcept[] acs = acl.getAssociatedConcept();
								if (acs != null)
								{
									for (int k=0; k<acs.length; k++)
									{
										if (acs[k] != null)
										{
											gov.nih.nci.lexbig.ext.ConceptAssociation concept_asso = convertConceptAssociation(codingScheme,
																					version,
																					associationname,
																					acs[k],
																					returncode);
											v.add(concept_asso);

										}
									}
								}
							}
						}
				    }
			    }
			}
	    } catch (Exception e) {
			//e.printStackTrace();
			getLogger().error("Error encountered when calling getConceptAssociations.");
		}
		return v;

    }



///////////////////////
//  Associations
///////////////////////

    /*
     * Enumerate Relationships (i.e., supported relationships in vocabulary)
     */

    /**
     *
     * @since DLBExt 1.0
     */
    public Vector<SupportedAssociation> getSupportedAssociations()
	{
		Vector<SupportedAssociation> v = new Vector<SupportedAssociation>();
		HashSet<String> hset = new HashSet<String>();
		Vector<CodingScheme> codingschemes = getSupportedCodingSchemes();
		for (int i=0; i<codingschemes.size(); i++)
		{
			CodingScheme cs = (CodingScheme) codingschemes.elementAt(i);
			SupportedAssociation[] assos = cs.getMappings().getSupportedAssociation();
			for (int j=0; j<assos.length; j++)
			{
				 SupportedAssociation sa = (SupportedAssociation) assos[j];
				 if (!hset.contains(sa.getContent()))
				 {
				     v.add(sa);
				     hset.add(sa.getContent());
				 }
			}
	    }
        return v;
	}


    /**
     *
     * @since DLBExt 1.0
     * @param cs
     */
    public Vector<SupportedAssociation> getSupportedAssociations(CodingScheme cs)
	{
		if (cs == null) return null;

        Vector<SupportedAssociation> v = new Vector<SupportedAssociation>();
	    SupportedAssociation[] assos = cs.getMappings().getSupportedAssociation();
		for (int i=0; i<assos.length; i++)
		{
		     SupportedAssociation sa = (SupportedAssociation) assos[i];
		     v.add(sa);
		}
        return v;
	}



    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     */
    public Vector<SupportedAssociation> getSupportedAssociations(String codingScheme)
	{
        CodingScheme cs = getCodingScheme(codingScheme);
		if (cs == null) return null;

        Vector<SupportedAssociation> v = new Vector<SupportedAssociation>();

	    SupportedAssociation[] assos = cs.getMappings().getSupportedAssociation();
		for (int i=0; i<assos.length; i++)
		{
		     SupportedAssociation sa = (SupportedAssociation) assos[i];
		     v.add(sa);
		}
        return v;
	}



    /**
     *
     * @since DLBExt 1.0
     * @param codingScheme a coding scheme name
     */
     // include roles and associations
    public Vector<String> getSupportedAssociationNames(String codingScheme)
	{
        CodingScheme cs = getCodingScheme(codingScheme);
		if (cs == null) return null;

        Vector<String> v = new Vector<String>();
	    SupportedAssociation[] assos = cs.getMappings().getSupportedAssociation();
		for (int i=0; i<assos.length; i++)
		{
		     SupportedAssociation sa = (SupportedAssociation) assos[i];
		     v.add(sa.getLocalId());
		}
        return v;
	}

    protected Set<String> getTargetNames(ResolvedConceptReferenceList rcrl) {
    	Set<String> targetNames = new TreeSet();
    	for (Enumeration rcrEnum = rcrl.enumerateResolvedConceptReference(); rcrEnum.hasMoreElements(); ) {
    		ResolvedConceptReference rcr = (ResolvedConceptReference) rcrEnum.nextElement();
    		AssociationList targetList = rcr.getSourceOf();
    		if (targetList.getAssociationCount() > 0)
	    		for (Enumeration targetEnum = targetList.enumerateAssociation(); targetEnum.hasMoreElements(); ) {
	    			AssociatedConceptList conceptList = ((Association) targetEnum.nextElement()).getAssociatedConcepts();
	    			for (Enumeration conceptEnum = conceptList.enumerateAssociatedConcept(); conceptEnum.hasMoreElements(); ) {
						Object obj = conceptEnum.nextElement();
						if (obj != null)
						{
							AssociatedConcept asso_concept = (AssociatedConcept) obj;
							if (asso_concept.getEntityDescription() != null)
							{
                            	targetNames.add(asso_concept.getEntityDescription().getContent());
							}
					    }
	    				//targetNames.add(((AssociatedConcept) conceptEnum.nextElement()).getEntityDescription().getContent());
	    			}
	    		}
    	}
    	return targetNames;
    }


    public Vector<String> getSupportedAssociationNames(String codingSchemeName, String version, String type)
    {
		if (type == null)
		{
		    return getAssociationNamesByRelation(codingSchemeName, version, ASSOCIATION_RELATION);
		}
		if (type.compareTo(ROOT_ASSOCIATION) != 0 && type.compareTo(ROOT_ROLE) != 0)
		{
		    return null;
	    }
		else if (type.compareTo(ROOT_ROLE) == 0)
		{
			return getAssociationNamesByRelation(codingSchemeName, version, ROLE_RELATION);
		}
	    else if (type.compareTo(ROOT_ASSOCIATION) == 0)
	    {
			return getAssociationNamesByRelation(codingSchemeName, version, ASSOCIATION_RELATION);
		}
		return null;

	}

	public Vector getSupportedPropertyQualifiers(String codingScheme)
	{
		CodingScheme cs = getCodingScheme(codingScheme);
		return getSupportedPropertyQualifiers(cs);
	}

	public Vector<String> getSupportedPropertyQualifiers(CodingScheme cs)
	{
		Vector<String> v = new Vector<String>();
        SupportedPropertyQualifier[] supported_qualifiers = cs.getMappings().getSupportedPropertyQualifier();
        for (int i=0; i<supported_qualifiers.length; i++)
        {
			//v.add(supported_qualifiers[i].getContent());
			v.add(supported_qualifiers[i].getLocalId());
		}
		return v;
	}


    public Vector<Property> convertProperties(org.LexGrid.commonTypes.Property[] properties)
    {
		Vector<Property> v = new Vector<Property>();
		for (int i=0; i<properties.length; i++)
		{
			org.LexGrid.commonTypes.Property p = properties[i];
			gov.nih.nci.lexbig.ext.Property prop
			   = new gov.nih.nci.lexbig.ext.Property(p.getProperty(), p.getText().getContent());
			PropertyQualifier[] qualifiers = p.getPropertyQualifier();
			for (int j=0; j<qualifiers.length; j++)
			{
				prop.addQualifier(new gov.nih.nci.lexbig.ext.Qualifier(qualifiers[j].getPropertyQualifierId(), qualifiers[j].getContent()));
			}
			//090107
			org.LexGrid.commonTypes.Source[] sources = p.getSource();
			for (int j=0; j<sources.length; j++)
			{
				org.LexGrid.commonTypes.Source src = (org.LexGrid.commonTypes.Source) sources[j];
				prop.addQualifier(new gov.nih.nci.lexbig.ext.Qualifier("Source", src.getContent()));
			}
			v.add(prop);
		}
		return v;
	}


	public Vector<Property> getProperties(CodedEntry ce)
	{
        if (ce == null)
        {
 			cat.error("wrapper.getProperties invalid input -- ce is null.");
			return null;
		}
        Vector<Property> v = new Vector<Property>();
		Vector<Property> w = convertProperties(ce.getDefinition());
		for (int i=0; i<w.size(); i++)
		{
			v.add((gov.nih.nci.lexbig.ext.Property) w.elementAt(i));
		}
		w = convertProperties(ce.getComment());
		for (int i=0; i<w.size(); i++)
		{
			v.add((gov.nih.nci.lexbig.ext.Property) w.elementAt(i));
		}
		w = convertProperties(ce.getInstruction());
		for (int i=0; i<w.size(); i++)
		{
			v.add((gov.nih.nci.lexbig.ext.Property) w.elementAt(i));
		}
		w = convertProperties(ce.getPresentation());
		for (int i=0; i<w.size(); i++)
		{
			v.add((gov.nih.nci.lexbig.ext.Property) w.elementAt(i));
		}
		w = convertProperties(ce.getConceptProperty());
		for (int i=0; i<w.size(); i++)
		{
			v.add((gov.nih.nci.lexbig.ext.Property) w.elementAt(i));
		}
		return v;
	}


	/*

	public Vector getProperties(CodedEntry ce)
    {
        if (ce == null)
        {
 			cat.error("wrapper.getProperties invalid input -- ce is null.");
			return null;
		}

        Vector v = new Vector();
        // Properties
		Definition[] definitions = ce.getDefinition();
		for (int i=0; i<definitions.length; i++)
		{
		    Definition d = definitions[i];
			String s = "definition" + "$" + d.getText().getContent();
			v.add(s);
		}

		Comment[] comments = ce.getComment();
		for (int i=0; i<comments.length; i++)
		{
		    Comment c = comments[i];
			String s = "comment" + "$" + c.getText().getContent();
			v.add(s);
		}

		Instruction[] instructions = ce.getInstruction();
		for (int i=0; i<instructions.length; i++)
		{
		    Instruction inst = instructions[i];
			String s = "instruction" + "$" + inst.getText().getContent();

			v.add(s);
		}

		Presentation[] presentations = ce.getPresentation();
		if (presentations != null)
		{
			for (int i=0; i<presentations.length; i++)
			{
				Presentation p = presentations[i];
				Vector source_vec = getPropertySources(p);
				if (source_vec.size() == 0)
				{
					//gov.nih.nci.lexrpc.client.Property property
					//   = new gov.nih.nci.lexrpc.client.Property();
					java.lang.String prop_name = p.getProperty();
					java.lang.String prop_value = p.getText().getContent();

					String s = prop_name + "$" + prop_value;
					v.add(s);
				}
			}
		}


 	    org.LexGrid.commonTypes.Property[] properties = ce.getConceptProperty();
		for (int i=0; i<properties.length; i++)
		{
			org.LexGrid.commonTypes.Property p = properties[i];
			if (p != null)
			{
				String s = p.getProperty() + "$" + p.getText().getContent();
				v.add(s);
		    }
		}
		return v;
	}
	*/


	public Vector getPropertiesByCode(String codingSchemeName, String versionName, String code)
    {
		 if (code == null)
		 {
			 cat.error("WARNING: code for " + code + " not found.");
			 return null;
		 }

	    CodedEntry ce = getCodedEntry(codingSchemeName, versionName, tag, code);//, restrictproperties);
	    return getProperties(ce);
	}



	public Vector<Property> getPropertiesByConceptName(String codingSchemeName, String versionName, String tagName, String conceptname)
    {
//		Vector<Property> restrictproperties = null;
        String code = nameToCode(codingSchemeName, versionName, tagName, conceptname);

		 if (code == null)
		 {
			 cat.error("WARNING: code for " + conceptname + " not found.");
			 return null;
		 }

	    CodedEntry ce = getCodedEntry(codingSchemeName, versionName, tagName, code);//, restrictproperties);
	    return getProperties(ce);
	}


    public Vector<String> getPropertyNames(CodedEntry ce)
    {
		Vector<Property> w = getProperties(ce);
	    HashSet<String> hset = new HashSet<String>();
	    Vector<String> v = new Vector<String>();
        for (int i=0; i<w.size(); i++)
        {
			String s = (String) w.elementAt(i).getName();
			int pos = s.indexOf("$");
			String name = s.substring(0, pos);
			if (!hset.contains(name))
			{
				hset.add(name);
				v.add(name);
			}
		}
		hset.clear();
		return v;
    }


    public Vector<String> getPropertyNames(String codingScheme,
	                                 String version,
									 String tag,
									 String conceptname)
    {
		Vector<Property> w = getPropertiesByConceptName(codingScheme, version, tag, conceptname);
		if (w == null)
		{
			return null;
		}
	    HashSet<String> hset = new HashSet<String>();
	    Vector<String> v = new Vector<String>();
        for (int i=0; i<w.size(); i++)
        {
			gov.nih.nci.lexbig.ext.Property property = (gov.nih.nci.lexbig.ext.Property) w.elementAt(i);
			String name = property.getName();
			/*
			String s = (String) w.elementAt(i);
			int pos = s.indexOf("$");
			String name = s.substring(0, pos);
			*/
			if (!hset.contains(name))
			{
				hset.add(name);
				v.add(name);
			}
		}
		hset.clear();
		return v;
    }


    public Vector<String> getPropertyValues(String codingScheme,
	                                 String version,
									 String tag,
									 String conceptname,
									 String propertyname
									 )
    {
        Vector<String> v = new Vector<String>();
        Vector<Property> w = getPropertiesByConceptName(codingScheme, version, tag, conceptname);
        if (w == null) return null;
        for (int i=0; i<w.size(); i++)
        {
			gov.nih.nci.lexbig.ext.Property property = (gov.nih.nci.lexbig.ext.Property) w.elementAt(i);
			if (property.getName().compareTo(propertyname) == 0)
			{
				v.add(property.getValue());
			}
			//080307
			/*
			String s = (String) w.elementAt(i);
			int pos = s.indexOf("$");
			String name = s.substring(0, pos);
			String value = s.substring(pos+1, s.length());
			if (name.compareTo(propertyname) == 0) v.add(value);
			*/

		}
		return v;
    }

    public Vector<String> getPropertyValues(CodedEntry ce, String propertyname)
    {
        Vector<String> v = new Vector<String>();
        Vector<Property> w = getProperties(ce);
        for (int i=0; i<w.size(); i++)
        {
			gov.nih.nci.lexbig.ext.Property property = (gov.nih.nci.lexbig.ext.Property) w.elementAt(i);
			if (property.getName().compareTo(propertyname) == 0)
			{
				v.add(property.getValue());
			}
		}
		return v;
    }

    /** Gets property sources.
     *
     * @since DLBExt 1.0
     * @param prop property name
     * @return a Vector containing property sources
     */
    public Vector<String> getPropertySources(org.LexGrid.commonTypes.Property prop)
	{
		Vector<String> source_vec = new Vector<String>();
		/*
		java.lang.String[] sources = prop.getSource();
		if (sources != null)
		{
			for (int i=0; i<sources.length; i++)
			{
				source_vec.add(sources[i]);
			}
		}
		*/
		org.LexGrid.commonTypes.Source[] sources = prop.getSource();
		if (sources != null)
		{
			for (int i=0; i<sources.length; i++)
			{
				source_vec.add(sources[i].getContent());
			}
		}

		return source_vec;
 	}

    public Vector<String> array2Vector(String[] a)
    {
		if (a == null) return null;
		Vector<String> v = new Vector<String>();
		if (a.length == 0) return v;
		for (int i=0; i<a.length; i++)
		{
			Object obj = (Object) a[i];
			v.add((String) obj);
		}
		return v;
	}


    // Version 2 beta
    public Vector<String> getFilterExtensions()
    {
		Vector<String> v = new Vector<String>();
		ExtensionDescriptionList edl = lbSvc.getFilterExtensions();
		ExtensionDescription[] ed_array = edl.getExtensionDescription();
		for (int i=0; i<ed_array.length; i++)
		{
     		ExtensionDescription ed = ed_array[i];
     		v.add(ed.getExtensionClass());
 		}
 		return v;
	}

	public Vector<Property> getConceptProperties(String codingScheme, String code, String propertytype, String propertyname) {

		Vector<Property> v = new Vector<Property>();
        CodedEntry ce = getCodedEntry(codingScheme,
	                                version,
									tag,
									code);
		if (ce == null)
		{
   			getLogger().error("Concept code " + code + " not found in " + codingScheme);
		    return null;
		}

		if (propertyname != null && propertyname.compareTo("*") == 0) propertyname = null;

		org.LexGrid.commonTypes.Property[] properties = null;
		if (propertytype.compareToIgnoreCase(TYPE_DEFINITION) == 0)
		{
			properties = ce.getDefinition();
		}
		else if (propertytype.compareToIgnoreCase(TYPE_COMMENT) == 0)
		{
			properties = ce.getComment();
		}
		else if (propertytype.compareToIgnoreCase(TYPE_PROPERTY) == 0)
		{
			properties = ce.getConceptProperty();
		}
		else if (propertytype.compareToIgnoreCase(TYPE_INSTRUCTION) == 0)
		{
			properties = ce.getInstruction();
		}
		else if (propertytype.compareToIgnoreCase(TYPE_PRESENTATION) == 0)
		{
			properties = ce.getPresentation();
		}

		if (properties == null)
		{
			getLogger().error("getConceptProperties returns null");
			return null;
		}

		else
		{
			for (int i=0; i<properties.length; i++)
			{
				org.LexGrid.commonTypes.Property p = properties[i];
				String prop_name = p.getProperty();
				if (propertyname == null || prop_name.compareTo(propertyname) == 0)
				{
					gov.nih.nci.lexbig.ext.Property prop
					   = new gov.nih.nci.lexbig.ext.Property(prop_name, p.getText().getContent());
                    PropertyQualifier[] qualifiers = p.getPropertyQualifier();
					for (int j=0; j<qualifiers.length; j++)
					{
						prop.addQualifier(new gov.nih.nci.lexbig.ext.Qualifier(qualifiers[j].getPropertyQualifierId(), qualifiers[j].getContent()));
					}
					//090107
					org.LexGrid.commonTypes.Source[] sources = p.getSource();
					for (int j=0; j<sources.length; j++)
					{
						org.LexGrid.commonTypes.Source src = (org.LexGrid.commonTypes.Source) sources[j];
						prop.addQualifier(new gov.nih.nci.lexbig.ext.Qualifier("Source", src.getContent()));
					}
					v.add(prop);
			    }
			}
		}
		return v;
	}

/*
    public String getHasSubtype(String codingSchemeName)
    {
//		Vector v = getSupportedRoleNames(codingSchemeName);

		Vector v = getAssociationNames(codingSchemeName);


		//073007
		if (v == null) return IS_A; // hasSubtype

		if (v.contains(IS_A))
		{
			return IS_A;
		}
		else if (v.contains(IS_A_OBO)) // is-a
		{
			return IS_A_OBO;
		}
		else if (v.contains(HAS_SUBCLASS))
		{
			return HAS_SUBCLASS;
		}

		else if (v.contains("PAR"))
		{
			return "PAR";
		}

		else
		{
			cat.error("hasSubtype association is not defined.");
			return null;
		}
	}
	*/
    //GF9458
    public String getHasSubtype(String codingSchemeName)
    {
		Vector v = getAssociationNames(codingSchemeName);
		if (v == null) return IS_A;
		for (int i = 0; i < HIER_ASSOC.length; i++)
		{
			String asso = HIER_ASSOC[i];
			if (v.contains(asso)) return asso;
		}
		System.out.println("WARNING: Hier relationship for " + codingSchemeName + " cannot be identified.");
		return null;
	}

///////////////////////////////////////////////////////////////////////////////////////////////////
	public Vector<CodedEntry> searchCodedEntries(String scheme, String version, String tag, String searchTerm, int limit, String source,
						 boolean cui, boolean shortResult, boolean score)
	{
		SearchUtil searchUtil = new SearchUtil(scheme, version, lbSvc);
	    return searchUtil.searchNodes(scheme, version, tag, searchTerm, limit, source,
						 cui, shortResult, score);
	}


	protected CodedNodeSet restrictToSource(CodedNodeSet cns, String source)
	{
		if (source == null || source.compareTo("*") == 0) return cns;
		Vector<String> v = new Vector<String>();
		v.add(source);
	    LocalNameList sourceList = vector2LocalNameList(v);
		CodedNodeSet.PropertyType[] types = new CodedNodeSet.PropertyType[] {CodedNodeSet.PropertyType.PRESENTATION};
		try {

			cns = cns.restrictToProperties(null, types, sourceList, null, null);

		} catch (Exception e) {
   			getLogger().error("restrictToSource error. Source: " + source);
		    return null;
		}
		return cns;
	}

	public String getPreferredName(CodedEntry ce)
	{
		 if (ce == null)
		 {
			 return null;
		 }
		 Presentation[] presentations = ce.getPresentation();
		 if (presentations == null) return ce.getConceptCode();
//		 String name = null;
		 for (int i=0; i<presentations.length; i++)
		 {
			 if (presentations[i].getIsPreferred().equals(Boolean.TRUE))
			 {
				 return presentations[i].getText().getContent();
			 }
		 }
		 return null;
	}

	public String getName(String codingSchemeName, String version, String tag, String code)
	{
		CodedEntry ce = getCodedEntryByCode(codingSchemeName, version, tag, code);
		String name = getName(ce);
		if (name != null) return name;
		return codeToName(codingSchemeName, version, code);
	}


	public String getName(CodedEntry ce)
	{
		 if (ce == null)
		 {
			 return null;
		 }
		 String pt = getPreferredName(ce);
		 //KLO, 081107
		 if (pt != null) return pt;
		 Presentation[] presentations = ce.getPresentation();
		 if (presentations == null || presentations.length == 0) return ce.getConceptCode();
		 return presentations[0].getText().getContent();
	}


    public static ConceptReferenceList createConceptReferenceList(String[] codes, String codeSystem)
    {
        if (codes == null)
        {
            return null;
        }
        ConceptReferenceList list = new ConceptReferenceList();
        for (int i = 0; i < codes.length; i++)
        {
            ConceptReference cr = new ConceptReference();
            cr.setCodingScheme(codeSystem);
            cr.setConceptCode(codes[i]);
            list.addConceptReference(cr);
        }
        return list;
    }





	 protected Vector<String> getCodedEntrySources(CodedEntry ce)
     {
	    Vector<String> v = new Vector<String>();
	    HashSet<String> hset = new HashSet<String>();
        java.util.Enumeration enumeration = ce.enumeratePresentation();
		while (enumeration.hasMoreElements())
		{
		    Presentation property = (Presentation) enumeration.nextElement();
		    Source[] sources = property.getSource();
		    for (int i=0; i<sources.length; i++)
		    {
				Source src = (Source) sources[i];
				String name = src.getContent();
				if (!hset.contains(name))
				{
					v.add(name);
					hset.add(name);
				}
			}
		}

		return v;
    }

    // KLO, 080307
	public Vector<String> getSourceCollectionbyCode(String vocabularyName, String code) {
	    CodedEntry ce = getCodedEntry(vocabularyName, version, tag, code);
	    if (ce == null)
	    {
			System.out.println("Concept " + code + " not found.");
			return null;
		}
	    Vector<String> v = new Vector<String>();
	    HashSet<String> hset = new HashSet<String>();
        java.util.Enumeration enumeration = ce.enumeratePresentation();
		while (enumeration.hasMoreElements())
		{
		    Presentation property = (Presentation) enumeration.nextElement();
		    Source[] sources = property.getSource();
		    for (int i=0; i<sources.length; i++)
		    {
				Source src = (Source) sources[i];
				String name = src.getContent();
				if (!hset.contains(name))
				{
					v.add(name);
					hset.add(name);
				}
			}
		}
		return v;
	}


	public Boolean isForwardName(java.lang.String codingScheme, java.lang.String version, java.lang.String directionalName)
	{
		CodingSchemeVersionOrTag versionOrTag = new CodingSchemeVersionOrTag();
		versionOrTag.setVersion(version);
		try {
		    return lbConvMethods.isForwardName(codingScheme, versionOrTag, directionalName);
		} catch (Exception e) {
			return null;
		}
	}

	public Boolean isReverseName(java.lang.String codingScheme, java.lang.String version, java.lang.String directionalName)
	{
		CodingSchemeVersionOrTag versionOrTag = new CodingSchemeVersionOrTag();
		versionOrTag.setVersion(version);
		try {
		    return lbConvMethods.isReverseName(codingScheme, versionOrTag, directionalName);
		} catch (Exception e) {
			return null;
		}
	}

 	public CodingSchemeSummary getCodingSchemeSummary(String codingSchemeName, String version)
	{
		try {
			CodingSchemeRenderingList csrl = lbSvc.getSupportedCodingSchemes();
			CodingSchemeRendering[] csrs = csrl.getCodingSchemeRendering();
			for (int i=0; i<csrs.length; i++)
			{
				CodingSchemeRendering csr = (CodingSchemeRendering) csrs[i];
				CodingSchemeSummary css = csr.getCodingSchemeSummary();
				String formalName = css.getFormalName();
				String localName = css.getLocalName();
				if (formalName.compareTo(codingSchemeName) == 0 || localName.compareTo(codingSchemeName) == 0)
				{
					if (css.getRepresentsVersion().compareTo(version) == 0)
					{
						return css;
					}
				}
			}
	    } catch (Exception e) {
			return null;
		}
		return null;
	}

 	public CodingSchemeSummary getCodingSchemeSummary(String codingSchemeName)
	{
		try {
			CodingSchemeRenderingList csrl = lbSvc.getSupportedCodingSchemes();
			CodingSchemeRendering[] csrs = csrl.getCodingSchemeRendering();
			for (int i=0; i<csrs.length; i++)
			{
				CodingSchemeRendering csr = (CodingSchemeRendering) csrs[i];
				CodingSchemeSummary css = csr.getCodingSchemeSummary();
				String formalName = css.getFormalName();
				String localName = css.getLocalName();
				if (formalName.compareTo(codingSchemeName) == 0 || localName.compareTo(codingSchemeName) == 0)
				{
					return css;
				}
			}
	    } catch (Exception e) {
			return null;
		}
		return null;
	}

	public String getRepresentsVersion(String codingSchemeName)
	{
		CodingSchemeSummary css = getCodingSchemeSummary(codingSchemeName);
		if (css == null) return null;
		return css.getRepresentsVersion();
	}


	protected Vector<AssociatedConcept> getSources(String scheme, String version, String tag, String code, String relation, String source)
	{
        Vector<AssociatedConcept> v = new Vector<AssociatedConcept>();
        CodingSchemeVersionOrTag csvt = createCodingSchemeVersionOrTag(version, tag);

		try {
			CodedNodeGraph cng = lbSvc.getNodeGraph(scheme, csvt, null);
			if (relation != null)
			{
				NameAndValue nv = new NameAndValue();
				NameAndValueList nvList = new NameAndValueList();
				nv.setName(relation);
				nvList.addNameAndValue(nv);
				cng = cng.restrictToAssociations(nvList, null);
			}
			if (source != null && source.compareTo("*") != 0)
			{
				CodedNodeSet cns = lbSvc.getCodingSchemeConcepts(scheme, csvt);
				cns = restrictToSource(cns, source);
				cng = cng.restrictToSourceCodes(cns);
			}

			ResolvedConceptReferenceList matches =
				cng.resolveAsList(
					ConvenienceMethods.createConceptReference(code, scheme),
					false, true, 1, 1, new LocalNameList(), null, null, 1024);

			// Analyze the result ...
			if (matches.getResolvedConceptReferenceCount() > 0) {
				ResolvedConceptReference ref =
					(ResolvedConceptReference) matches.enumerateResolvedConceptReference().nextElement();

				// Print the associations
				AssociationList targetof = ref.getTargetOf();
				Association[] associations = targetof.getAssociation();
				for (int i = 0; i < associations.length; i++) {
					Association assoc = associations[i];
					AssociatedConcept[] acl = assoc.getAssociatedConcepts().getAssociatedConcept();
					for (int j = 0; j < acl.length; j++) {
						AssociatedConcept ac = acl[j];
						v.add(ac);
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			return new Vector<AssociatedConcept>();
		}
		return v;
	}


	protected Vector<AssociatedConcept> getTargets(String scheme, String version, String tag, String code, String relation, String source)
	{
        Vector<AssociatedConcept> v = new Vector<AssociatedConcept>();
        CodingSchemeVersionOrTag csvt = createCodingSchemeVersionOrTag(version, tag);

        try {
			CodedNodeGraph cng = lbSvc.getNodeGraph(scheme, csvt, null);
			if (relation != null)
			{
				NameAndValue nv = new NameAndValue();
				NameAndValueList nvList = new NameAndValueList();
				nv.setName(relation);
				nvList.addNameAndValue(nv);
				cng = cng.restrictToAssociations(nvList, null);
			}
			if (source != null && source.compareTo("*") != 0)
			{
				CodedNodeSet cns = lbSvc.getCodingSchemeConcepts(scheme, csvt);
				cns = restrictToSource(cns, source);
				cng = cng.restrictToTargetCodes(cns);
			}


			ResolvedConceptReferenceList matches =
					cng.resolveAsList(
						ConvenienceMethods.createConceptReference(code, scheme),
						true, false, 1, 1, new LocalNameList(), null, null, 1024);

			// Analyze the result ...
			if (matches.getResolvedConceptReferenceCount() > 0) {
				ResolvedConceptReference ref =
					(ResolvedConceptReference) matches.enumerateResolvedConceptReference().nextElement();

				// Print the associations
				AssociationList sourceof = ref.getSourceOf();
				Association[] associations = sourceof.getAssociation();
				for (int i = 0; i < associations.length; i++) {
					Association assoc = associations[i];
					AssociatedConcept[] acl = assoc.getAssociatedConcepts().getAssociatedConcept();
					for (int j = 0; j < acl.length; j++) {
						AssociatedConcept ac = acl[j];
						v.add(ac);
					}
				}
			}
	    } catch (Exception e) {
			e.printStackTrace();
			return new Vector<AssociatedConcept>();
		}
		return v;
	}


	gov.nih.nci.lexbig.ext.ConceptAssociation convertConceptAssociation(String codingScheme,
	                                                                    String version,
	                                                                    String associationName,
	                                                                    AssociatedConcept ac,
	                                                                    boolean returncode)
	{
		if (ac == null) return null;
		gov.nih.nci.lexbig.ext.ConceptAssociation concept_association = null;
		String asso_code = ac.getConceptCode();
		if (returncode)
		{
			concept_association = new gov.nih.nci.lexbig.ext.ConceptAssociation(associationName, asso_code);
		}
		else
		{
			String asso_name = codeToName(codingScheme, version, asso_code);
			concept_association = new gov.nih.nci.lexbig.ext.ConceptAssociation(associationName, asso_name);
        }
        NameAndValueList nvl = ac.getAssociationQualifiers();
        if (nvl != null)
        {
			NameAndValue[] nva = nvl.getNameAndValue();
			for (int i=0; i<nva.length; i++)
			{
				NameAndValue nv = nva[i];
				concept_association.addQualifier(new gov.nih.nci.lexbig.ext.Qualifier(nv.getName(), nv.getContent()));
			}
		}
		return concept_association;
	}

    // GF9072
    public Boolean isCodingSchemeActive(LexBIGService lbSvc, String scheme, String version)
    {
		try {
			Enumeration<CodingSchemeRendering> schemes = lbSvc.getSupportedCodingSchemes().enumerateCodingSchemeRendering();
			while (schemes.hasMoreElements()) {
				CodingSchemeRendering rendering = schemes.nextElement();
				CodingSchemeSummary summary = rendering.getCodingSchemeSummary();
				if (scheme.equalsIgnoreCase(summary.getCodingSchemeURN()) ||
					scheme.equalsIgnoreCase(summary.getFormalName()) ||
					scheme.equalsIgnoreCase(summary.getLocalName()) )
				{
					if (version != null)
					{
						if (version.equalsIgnoreCase(summary.getRepresentsVersion()))
						{
							boolean isActive = rendering.getRenderingDetail().getVersionStatus().equals(CodingSchemeVersionStatus.ACTIVE);
							return new Boolean(isActive);
						}
					}
					else
					{
						boolean isActive = rendering.getRenderingDetail().getVersionStatus().equals(CodingSchemeVersionStatus.ACTIVE);
						return new Boolean(isActive);
					}
				}
			}
	    } catch (Exception e) {
			e.printStackTrace();
		}
	    return null;
	}
}

